/*		Sccsid %Z% %M% %I% %G% */

/*
**
**	Sybase Open Server/Open Client Gateway
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1991, - 1993
**	All rights reserved
**
**  CTOSDEMO.H
**	This is the header file for the ctosdemo sample program.
*/

/*
** Operating system header files required by gateway Open Server
*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<signal.h>
#include	<string.h>
#include	<ctype.h>

/*
** Sybase header files required by Open Server and Open Client
*/
#include	<cspublic.h>
#include	<ctpublic.h>
#include	<ospublic.h>
#include	<oserror.h>
#include	<bkpublic.h>

/*
** Header file required by all sample programs
*/
#include	<ossample.h>

/*
** Manifest Constants
**
**	GW_STDEXIT	Returned to shell for normal exit
**	GW_ERREXIT	Returned to shell for error exit
*/
#define	GW_STDEXIT	0
#define	GW_ERREXIT	1

/*
** Error message defines
*/
#define	GW_ERRMSG	"\tCT-Lib error: error(%d) serverity(%d) OS error(%d)\n"
#define	GW_OSERRMSG	"\tCT-Lib OS Error: error(%d)\n"
#define	GW_RMTMSG	"\tRemote DBMS(%s): msgno(%d) severity(%d) state(%d)\n"

/*
** ctosdemo private defines
*/
#define	GW_INSRTBLK	(CS_INT)0
#define	GW_WRTXTBLK	(CS_INT)1
#define	GW_NONE		(CS_INT)2
#define	GW_REGULARFMT	(CS_INT)3
#define	GW_COMPUTEFMT	(CS_INT)4
#define	GW_GETFMTP	(CS_INT)5
#define	GW_SETFMTP	(CS_INT)6
#define	GW_CLEARFMTP	(CS_INT)7
#define	GW_GETCURID	(CS_INT)8	/* get a new cursor ID		*/
#define	GW_RELCURID	(CS_INT)9	/* release a cursor ID		*/
#define	GW_SETFETCH	(CS_INT)10	/* set row count for cursor	*/
#define	GW_GETFETCH	(CS_INT)11	/* get row count for cursor	*/
#define	GW_SAVEDYNID	(CS_INT)12	/* save a new dynamic SQL stmt	*/
#define	GW_RELDYNID	(CS_INT)13	/* release a dynamic SQL stmt	*/

/*
** Configuration defines
**
**	GW_MAXCOL	Maximum number of columns supported in a table
**	GW_BULKBUFSIZE	Size of bulk copy data buffer
**	GW_MAXCURSORS	Maximum number of active cursors
**	GW_MAXDYNS	Maximum number of active dynmiac SQL statements
*/
#define	GW_MAXCOL	(CS_INT)255
#define	GW_BULKBUFSIZE	(CS_INT)100
#define	GW_MAXCURSORS	(CS_INT)25
#define	GW_MAXDYNS	(CS_INT)25

/*
** Lint Cleaners
*/
#define	NIL(s)		(s)NULL
#define	SPRINTF		(CS_VOID)sprintf
#define	SRV_LOG		(CS_VOID)srv_log
#define	SRV_FREE(P)	(CS_VOID)srv_free((CS_VOID *)P)

/* 
** Cursor Control Structure.  This structure is used by the Open Server
** gateway application to manage active cursors.  These structures are
** accessed using the cursor identifier.
**
** This structure contains the following fields:
**
**	cmdp		command handle for this cursor
**	curid		cursor id	
**	curstatus	status of the current cursor:
**				Cursor Declared
**				Cursor Open
**				Cursor Closed
**				Cursor Read Only
**				Cursor Updatable
**				Row Count Present
**				Cursor Deallocated
**	fetchcnt	the number of rows to return to a client for this 
**			cursor on a FETCH command
**	nokeys		number of keys in current row.
**	keymap		this array is used to map key column numbers
**			when passing KEY data streams during CURUPDATE and
**			CURDELETE commands.
*/
typedef struct curctl
{
	CS_COMMAND	*cmdp;
	CS_INT		curid;
	CS_SMALLINT	curstatus;
	CS_INT		fetchcnt;
	CS_INT		nokeys;
	CS_INT		keymap[GW_MAXCOL];
} CURCTL_T;

/* 
** Dynamic SQL Control Structure.  This structure is used by 
** the Open Server gateway application to manage active dynamic 
** SQL statements.  These structures are accessed using the cursor 
** statement identifier.
**
** This structure contains the following fields:
**
**	cmdp		command handle for this dynamic statement
**	stmtid		statement id	
**	stmtidlen	statement id length	
**	stmtidnum
**	dynstatus	status of the current dynamic stmt:
*/
typedef struct dynctl
{
	CS_COMMAND	*cmdp;
	CS_CHAR		stmtid[CS_MAX_NAME];
	CS_INT		stmtidlen;
	CS_INT		stmtidnum;
	CS_SMALLINT	dynstatus;
} DYNCTL_T;

/*
** DBMS Management Structure.  A pointer to this structure is passed
** around to Open Server event handlers in the SRV_PROC structure's
** user data field.
**
** This structure contains the following fields
**
**	chp		A connection handle to the server connection for 
**			this SRV_PROC.
** 	rowfmtp		Pointer to fmt array for regular rows
**	blkdesc		Bulk copy control structure
** 	computefmtp[]	Array of pointers to fmt arrays for compute rows
**	retparms[]	This array is used to map RPC return parameters
**			during RPC command processing
**	actcur		Array of a active cursors
**	actdyn		Array of dynamically prepared statements
**	NumEncrypts	# of remote svrnames/pwds sent from client
**	EncryptPwdSent	Has encrypted pwd been passed to remote svr?
**	EncryptToGet	which remote svrname/pwd to read in next
**	NumSecLabels	# of secure labels sent from client
**	LabelToSend	running tab of secure labels
*/
typedef struct dbmsctl
{
	CS_CONNECTION	*chp;
	CS_DATAFMT	*rowfmtp;
	CS_BLKDESC	*blkdesc;
	CS_DATAFMT	*computefmtp[GW_MAXCOL];
	CS_INT		retparms[GW_MAXCOL];
	CS_IODESC	iodesc;
	CURCTL_T	actcur[GW_MAXCURSORS];
	DYNCTL_T	actdyn[GW_MAXDYNS];
	CS_INT		NumEncrypts;
	CS_BOOL		EncryptPwdSent;
	CS_INT		EncryptToGet;
	CS_INT		NumSecLabels;
	CS_INT		LabelToSend;
} DBMSCTL_T;

/*
** Prototypes
*/
extern int main PROTOTYPE((
	int argc,
	char *argv[]
	));
extern CS_RETCODE gw_cfgsrv PROTOTYPE((
	CS_CONTEXT *cp,
	CS_CHAR *np
	));
extern CS_RETCODE gw_setversion PROTOTYPE((
	CS_CONTEXT **cpp
	));
extern CS_RETCODE CS_PUBLIC gw_starthandler PROTOTYPE((
	CS_VOID *ssp
	));
extern CS_RETCODE CS_PUBLIC gw_connhandler PROTOTYPE((
	CS_VOID *spp
	));
extern CS_RETCODE CS_PUBLIC gw_langhandler PROTOTYPE((
	CS_VOID *spp
	));
extern CS_RETCODE CS_PUBLIC gw_rpchandler PROTOTYPE((
	CS_VOID *spp
	));
extern CS_RETCODE CS_PUBLIC gw_opthandler PROTOTYPE((
	CS_VOID *spp
	));
extern CS_RETCODE CS_PUBLIC gw_dischandler PROTOTYPE((
	CS_VOID *spp
	));
extern CS_RETCODE CS_PUBLIC gw_bulkhandler PROTOTYPE((
	CS_VOID *spp
	));
extern CS_RETCODE gw_proctextimage PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp
	));
extern CS_RETCODE gw_procbcp PROTOTYPE((
	CS_VOID *spp
	));
extern CS_RETCODE CS_PUBLIC gw_curhandler PROTOTYPE((
	CS_VOID *spp
	));
extern CS_BOOL gw_curdeclare PROTOTYPE((
	CS_VOID *spp,
	SRV_CURDESC *cdp,
	CS_INT opts
	));
extern CS_BOOL gw_curinfo PROTOTYPE((
	CS_VOID *spp,
	SRV_CURDESC *cdp,
	CS_INT opts
	));
extern CS_BOOL gw_curopen PROTOTYPE((
	CS_VOID *spp,
	SRV_CURDESC *cdp,
	CS_INT opts
	));
extern CS_BOOL gw_curfetch PROTOTYPE((
	CS_VOID *spp,
	SRV_CURDESC *cdp,
	CS_INT opts,
	CS_INT *rowp
	));
extern CS_BOOL gw_curupdate PROTOTYPE((
	CS_VOID *spp,
	SRV_CURDESC *cdp,
	CS_INT opts
	));
extern CS_BOOL gw_curend PROTOTYPE((
	CS_VOID *spp,
	SRV_CURDESC *cdp,
	CS_INT opts
	));
extern CS_RETCODE gw_upclause PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp,
	SRV_CURDESC *cdp
	));
extern CS_INT gw_mancurid PROTOTYPE((
	CS_VOID *spp,
	CS_INT action,
	CS_COMMAND *cmdp,
	CS_INT *curidp,
	CS_INT *rowcntp
	));
extern CS_COMMAND *gw_getcontext PROTOTYPE((
	CS_VOID *spp,
	CS_INT *curidp
	));
extern CURCTL_T *gw_getcurp PROTOTYPE((
	CS_VOID *spp,
	CS_INT curid
	));
extern CS_RETCODE CS_PUBLIC gw_dynhandler PROTOTYPE((
	CS_VOID *spp
	));
extern CS_BOOL gw_dynprepare PROTOTYPE((
	CS_VOID *spp,
	CS_INT optype
	));
extern CS_BOOL gw_dyndesc PROTOTYPE((
	CS_VOID *spp,
	CS_INT optype
	));
extern CS_BOOL gw_dynexecute PROTOTYPE((
	CS_VOID *spp,
	CS_INT optype
	));
extern CS_BOOL gw_dyndealloc PROTOTYPE((
	CS_VOID *spp,
	CS_INT optype
	));
extern CS_INT gw_handleresults PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp,
	CS_INT cmd
	));
extern CS_BOOL gw_procdynfmt PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp,
	CS_INT cmd
	));
extern CS_BOOL gw_procrowfmt PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp,
	CS_INT cmd
	));
extern CS_INT gw_procrows PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp,
	CS_INT cmd
	));
extern CS_INT gw_proctextcol PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp
	));
extern CS_BOOL gw_proccrowfmt PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp
	));
extern CS_INT gw_proccrows PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp
	));
extern CS_BOOL gw_controlrow PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp,
	CS_INT numcols
	));
extern CS_BOOL gw_dobrowse PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp,
	CS_INT numcols
	));
extern CS_BOOL gw_orderby PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp
	));
extern CS_RETCODE gw_procstat PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp
	));
extern CS_RETCODE gw_procmsg PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp
	));
extern CS_RETCODE gw_clientparam PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp,
	CS_INT type
	));
extern CS_RETCODE gw_srvparam PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp
	));
extern CS_RETCODE gw_remotemsgs PROTOTYPE((
	CS_CONTEXT *cp,
	CS_CONNECTION *chp,
	CS_SERVERMSG *msgp
	));
extern CS_RETCODE gw_cterr PROTOTYPE((
	CS_CONTEXT *cp,
	CS_CONNECTION *chp,
	CS_CLIENTMSG *emsgp
	));
extern CS_RETCODE gw_defnremotepwds PROTOTYPE((
	CS_VOID *spp,
	CS_CONNECTION *conn
	));
extern CS_RETCODE CS_PUBLIC gw_encryptcb PROTOTYPE((
	CS_CONNECTION *conn,
	CS_BYTE *pwd,
	CS_INT pwdlen,
	CS_BYTE *key,
	CS_INT keylen,
	CS_BYTE *buf,
	CS_INT buflen,
	CS_INT *outlen
	));
extern CS_RETCODE CS_PUBLIC gw_sendseclabels PROTOTYPE((
	CS_CONNECTION *conn,
	CS_INT inmsg,
	CS_INT *outmsg,
	CS_DATAFMT *infmt,
	CS_BYTE *inbuf,
	CS_DATAFMT *outfmt,
	CS_BYTE *outbuf,
	CS_INT *outlen
	));
extern CS_RETCODE CS_PUBLIC gw_responsecb PROTOTYPE((
	CS_CONNECTION *conn,
	CS_INT inmsg,
	CS_INT *outmsg,
	CS_DATAFMT *infmt,
	CS_BYTE *inbuf,
	CS_DATAFMT *outfmt,
	CS_BYTE *outbuf,
	CS_INT *outlen
	));
extern CS_RETCODE CS_PUBLIC gw_attnhandler PROTOTYPE((
	CS_VOID *spp
	));
extern CS_CONNECTION *gw_getchp PROTOTYPE((
	CS_VOID *spp,
	CS_BOOL release
	));
extern CS_INT *gw_getparmp PROTOTYPE((
	CS_VOID *spp
	));
extern CS_BOOL gw_managefmtp PROTOTYPE((
	CS_VOID *spp,
	CS_INT action,
	CS_INT type,
	CS_INT computeid,
	CS_DATAFMT **fmtpvar
	));
extern CS_RETCODE gw_chkwritetext PROTOTYPE((
	CS_VOID *spp,
	CS_CONNECTION *chp,
	CS_CHAR *cmd
	));
extern DYNCTL_T *gw_getdynp PROTOTYPE((
	CS_VOID *spp,
	CS_CHAR *dynid,
	CS_INT dynidlen
	));
extern CS_COMMAND *gw_getdyncontext PROTOTYPE((
	CS_VOID *spp,
	CS_CHAR *dynid,
	CS_INT *dynidlen
	));
extern CS_RETCODE gw_mandynid PROTOTYPE((
	CS_VOID *spp,
	CS_INT action,
	CS_COMMAND *cmdp,
	CS_CHAR *dynid,
	CS_INT dynidlen
	));
extern CS_VOID gw_senderror PROTOTYPE((
	CS_VOID *spp
	));
extern CS_RETCODE gw_initlogin PROTOTYPE((
	CS_VOID *spp,
	CS_CONNECTION *chp
	));
extern CS_RETCODE gw_ctsend PROTOTYPE((
	CS_VOID *spp,
	CS_COMMAND *cmdp
	));
