/*
**	Sybase Registered Procedure Client Application
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
**  CTWAIT
**	This program is used to demonstrate the use of registered procedures.
**	This program registers to watch for the execution of the 'rp_version'
**	registered procedure.  When the client program 'version' is executed,
**	this program notifies the client.
**
**  Usage:
**	ctwait
**	
**		ctwait connects to the Open Server application using the 
**		DSQUERY environment value.
**
**  Returns:
**      0 on successful execution
**      1 on error
*/

/*
** Operating system header files required by this application.
*/
#include	<stdio.h>
#include	<string.h>
#include	<ctype.h>

/*
** Sybase header files required by Open Client.
*/
#include	<ctpublic.h>
#include	<ctsample.h>

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

#define USERNAME	"sa"
#define PASSWORD	""
#define APPLICATION	"ctwait"

/*
** Prototypes
*/
CS_RETCODE notify_handler PROTOTYPE ((	/* Notification handler. 	*/
	CS_CONNECTION	*conn,
	CS_CHAR		*name,
	CS_INT		namelen
	));
CS_RETCODE rpc_regwatch PROTOTYPE ((	/* Perform regwatch rpc test. 	*/
	CS_CONNECTION	*conn,
	CS_COMMAND	*cmd
	));


int	main(argc, argv)
int	argc;
char	*argv[];
{
	CS_INT		ok;
 	CS_INT		ret;
 	CS_INT		compid;
	CS_BOOL		boolval;
	CS_RETCODE	compstatus;
	CS_CONNECTION	*conn;
	CS_COMMAND	*cmd;
	CS_COMMAND	*ready_cmd;
	CS_CONTEXT	*cp;

	/* 
	** Initialization.
	*/
	ok = CS_TRUE;
	cmd = (CS_COMMAND *)NULL;
	conn = (CS_CONNECTION *)NULL;

	/* 
	** Allocate memory for the context structure.
	*/
	if (cs_ctx_alloc(CS_VERSION_100, &cp) != CS_SUCCEED)
	{
		(CS_VOID)fprintf(stderr, 
			"ctwait: Context allocate failed. Exiting.\n");
		exit(1);
	}

	/* Install a CS-Library error handler.                          */
	if (cs_config(cp, CS_SET, CS_MESSAGE_CB, (CS_VOID *)cs_err_handler,
		CS_UNUSED, NULL) != CS_SUCCEED)
	{
		/* Release the context structure.			*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"ctwait: Can't init CSLIB handlers. Exiting\n");
		exit(1);
	}

	/* Initialize Open Client version.				*/
	if (ct_init(cp, CS_VERSION_100) != CS_SUCCEED)
	{
		/* Release the context structure.			*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"ctwait: Can't init CTLIB version. Exiting\n");
		exit(1);
	}

	/*
	** Install message and error handlers.
	*/
	if (ct_callback(cp, (CS_CONNECTION *)NULL, CS_SET, CS_CLIENTMSG_CB, 
		(CS_VOID *)client_err_handler) != CS_SUCCEED)
	{
		/* Release the context structure.			*/
		(CS_VOID)ct_exit(cp, CS_UNUSED);
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"ctwait: Can't init msg handlers. Exiting\n");
		exit(1);
	}
	if (ct_callback(cp, (CS_CONNECTION *)NULL, CS_SET, CS_SERVERMSG_CB, 
		(CS_VOID *)server_err_handler) != CS_SUCCEED)
	{
		/* Release the context structure.			*/
		(CS_VOID)ct_exit(cp, CS_UNUSED);
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"ctwait: Can't init err handlers. Exiting\n");
		exit(1);
	}

	/* 
	** Install a notification handler for all Open Server 
	** notification messages.
	*/
	if (ct_callback(cp, (CS_CONNECTION *)NULL, CS_SET, CS_NOTIF_CB, 
		(CS_VOID *)notify_handler) != CS_SUCCEED)
	{
		/* Release the context structure.			*/
		(CS_VOID)ct_exit(cp, CS_UNUSED);
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"ctwait: Can't init notification handlers. Exiting\n");
		exit(1);
	}


	/* Allocate a connection handle for the connection attempt.	*/
	if (ct_con_alloc(cp, &conn) != CS_SUCCEED)
		ok = CS_FALSE;

	/*
	** Set up the login information for the remote DBMS.
	** Also save control structures for access from other routines.
	*/

	if (ok && ct_con_props(conn, CS_SET, CS_USERNAME, USERNAME, 
		CS_NULLTERM, (CS_INT *)NULL) != CS_SUCCEED)
		ok = CS_FALSE;

	if (ok && ct_con_props(conn, CS_SET, CS_PASSWORD, PASSWORD, 
		CS_NULLTERM, (CS_INT *)NULL) != CS_SUCCEED)
		ok = CS_FALSE;

	if (ok && ct_con_props(conn, CS_SET, CS_APPNAME, APPLICATION, 
		CS_NULLTERM, (CS_INT *)NULL) != CS_SUCCEED)
		ok = CS_FALSE;

	/* Turn on read-ahead notifications.				*/
	boolval = CS_TRUE;
	if (ok && ct_con_props(conn, CS_SET, CS_ASYNC_NOTIFS, &boolval, 
		CS_UNUSED, (CS_INT *)NULL) != CS_SUCCEED)
		ok = CS_FALSE;

	/* 
	** Attempt a connection to the remote server.			
	** By setting the servername to NULL, we will default to use
	** the DSQUERY environment variable.
	*/
	if (ok && ct_connect(conn, (CS_CHAR *)NULL, 0) != CS_SUCCEED)
		ok = CS_FALSE;

	/* Allocate a command buffer.					*/
	if (ok && ct_cmd_alloc(conn, &cmd) != CS_SUCCEED)
		ok = CS_FALSE;

	/* 
	** Perform REGWATCH operations.
	*/

	/* Install a notifier for the 'rp_version' registered procedure.*/
	if (ok && (rpc_regwatch(conn, cmd) != CS_SUCCEED))
		ok = CS_FALSE;

	(CS_VOID)fprintf(stderr, 
		"Now calling ct_poll() to wait for event notification...\n");

	/* Loop waiting for 'rp_version' notifications.			*/
	while(ok)
	{
		ret = ct_poll((CS_CONTEXT *)NULL, conn, 0, 
			(CS_CONNECTION **)NULL, &ready_cmd, &compid, 
			&compstatus);

		switch ((int)ret)
		{
		case CS_SUCCEED:
			(CS_VOID)fprintf(stderr, "ct_poll() returned.\n");
			break;
		
		case CS_QUIET:
			break;

		default:
			(CS_VOID)fprintf(stderr, 
				"ct_poll() failed (%d)...\n", ret);
			ok = CS_FALSE;
			break;
		}

	}
		
	/* Free the command buffer.					*/
	if (cmd != (CS_COMMAND *)NULL)
		(CS_VOID)ct_cmd_drop(cmd);

	/* If connection with server is active, close it now.		*/
	if (conn != (CS_CONNECTION *)NULL)
	{
		(CS_VOID)ct_close(conn, CS_UNUSED);
		(CS_VOID)ct_con_drop(conn);
	}

	if (ok)
		(CS_VOID)fprintf(stderr, "\nctwait: Test was successful!\n");
	else
		(CS_VOID)fprintf(stderr, "\nctwait: Test failed!\n");

	/* Release allocated control structures.			*/
	(CS_VOID)ct_exit(cp, CS_UNUSED);
	(CS_VOID)cs_ctx_drop(cp);

	exit(0);
}

/*
**  RPC_REGWATCH - Register to watch for 'rp_version' registered procedure.
**
**  Parameters:
**	conn	- The connection to the server.
**	cmd	- The command to use to register the request.
**
**  Returns:
**	CS_SUCCEED | CS_FAIL
*/
CS_RETCODE	rpc_regwatch(conn, cmd)
CS_CONNECTION	*conn;
CS_COMMAND	*cmd;
{
	CS_DATAFMT	datafmt;
	CS_SMALLINT	val;
 	CS_INT		ret, res_type;

	/* 
	**	 Install a handler for event 'rp_version'.
	*/
 	if (ct_command(cmd, CS_RPC_CMD, "sp_regwatch", CS_NULLTERM, CS_UNUSED)
		!= CS_SUCCEED)
	{
		return (CS_FAIL);
	}

	datafmt.namelen = strlen("@proc_name");
	(CS_VOID)strcpy(datafmt.name, "@proc_name");
	datafmt.datatype = CS_CHAR_TYPE;
	datafmt.format = CS_FMT_UNUSED;
	datafmt.status = CS_INPUTVALUE;
	datafmt.locale = (CS_LOCALE *)NULL;

	if (ct_param(cmd, &datafmt, (CS_VOID *)"rp_version", CS_NULLTERM, 
		CS_UNUSED) != CS_SUCCEED)
	{
		return (CS_FAIL);
	}

	datafmt.namelen = strlen("@options");
	(CS_VOID)strcpy(datafmt.name, "@options");
	datafmt.datatype = CS_SMALLINT_TYPE;
	datafmt.format = CS_FMT_UNUSED;
	datafmt.status = CS_INPUTVALUE;
	datafmt.locale = (CS_LOCALE *)NULL;

	val = CS_NOTIFY_NOWAIT | CS_NOTIFY_ALWAYS;

	if (ct_param(cmd, &datafmt, (CS_VOID *)&val, CS_SIZEOF(val), CS_UNUSED) 
		!= CS_SUCCEED)
	{
		return (CS_FAIL);
	}

	/*
	** Send the command to the server.
	*/
	if (ct_send(cmd) != CS_SUCCEED)
		return (CS_FAIL);

 	/*
 	** Process the results of the cursor.
	** We are expecting only a CS_CMD_SUCCEED in the following result
	** processing.
 	*/
	while((ret = ct_results(cmd, &res_type)) == CS_SUCCEED)
	{
		switch((int)res_type)
		{

		case CS_ROW_RESULT:
			/*
			** This means rows were returned.
			*/
			break;

		case CS_STATUS_RESULT:
			/*
			** This means status was returned.
			*/
			break;

		case CS_PARAM_RESULT:
			/*
			** This means params were returned.
			*/
			break;

		case CS_CMD_SUCCEED:
			/*
			** This means no rows were returned.
			*/
			break;

		case CS_CMD_DONE:

			/*
			** Done with one result set, let's go
			** on to the next.
			*/
			break;

		case CS_CMD_FAIL:

			/*
			** The server encountered an error while
			** processing our command.
			*/
			(CS_VOID)fprintf(stderr,
			"ct_results() result type is CS_CMD_FAIL for rpc\n");

			return (CS_FAIL);

		default:
			/*
			** We got something unexpected.
			*/
			(CS_VOID)fprintf(stderr,
			"ct_command() returned bad result type for rpc\n");

			return (CS_FAIL);
		}
	}

	/*
	** We're done processing results. Let's check the
	** return value of ct_results to see if everything
	** went ok.
	*/
	switch((int)ret)
	{
		case CS_END_RESULTS:

			/*
			** Everything went fine.
			*/
			break;

		case CS_FAIL:

			/*
			** Something went awry.
			*/
			(CS_VOID)fprintf(stderr, 
				"ct_results() returned FAIL for rpc\n");
			return (CS_FAIL);

		default:

			/*
			** We got an unexpected return value.
			*/
			(CS_VOID)fprintf(stderr,
			"ct_results() returned bad ret value for rpc\n");
			return(CS_FAIL);
	}

	return (CS_SUCCEED);
}
/*
**  NOTIFY_HANDLER - Notification handler.
**
**	This routine receives all notification messages from the Open
**	Server application and prints them to the console.
**
**  Parameters:
**	conn	- The connection to the server
**	name	- The event notification name.
**	namelen	- The length of the event name.
**
**  Returns:
**	CS_SUCCEED
*/
CS_RETCODE	notify_handler(conn, name, namelen)
CS_CONNECTION	*conn;
CS_CHAR		*name;
CS_INT		namelen;
{
	printf("notify_handler() called - event %s...\n", name);
	return (CS_SUCCEED);
}
