/*
**
**	Sybase Registered Procedure Client Application
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
**  DBWAIT
**	This program demonstrates the use of registered procedures. This
**	program registers to watch for the execution of the 'rp_version'
**	registered procedure.  When the client program 'version' is executed,
**	this program notifies the client.
**
**  Usage:
**	dbwait
**	
**		dbwait connects with the Open Server application
**		using the DSQUERY environment value.
**
**  Returns:
**      0 on successful execution.
**      1 on error.
*/

/*
** Operating system header files required by this application.
*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

/*
** SYBASE header files required by DB-Library.
*/
#include 	<sybfront.h>
#include 	<sybdb.h>

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

#define USERNAME	"sa"
#define PASSWORD	""
#define APPLICATION	"dbwait"

/*
** Prototypes.
*/
int CS_PUBLIC notify_handler PROTOTYPE ((		/* Notification handler.	*/
	DBPROCESS       *dbproc,
	DBCHAR          *name,
	int		options
	));
int CS_PUBLIC err_handler PROTOTYPE (( 		/* Server error handler.	*/
	DBPROCESS       *dbproc,
	int             severity,
	int             dberr,
	int             oserr,
	char            *errstr,
	char            *oserrstr
	));
int CS_PUBLIC msg_handler PROTOTYPE ((		/* DB-Library message handler.	*/
	DBPROCESS       *dbproc,
	DBINT           msgno,
	int             msgstate,
	int             severity,
	char            *msgtext,
	char            *srvname,
	char            *procname,
	int    		line
	));


int	main(argc, argv)
int	argc;
char	*argv[];
{
	int		response;
	DBPROCESS	*conn;
	DBPROCESS	*ready_conn;
	LOGINREC	*login;

	/* Initialization.						*/
	conn = (DBPROCESS *)NULL;

	/* Initialize DB-library.					*/
	if (dbinit() != SUCCEED) 
		exit(1);

	/* Install DB-Library error/message handlers.			*/
	dberrhandle(err_handler);
	dbmsghandle(msg_handler);

	/* Open connection with the Open Server application. 		*/
	if ((login = dblogin()) == (LOGINREC *)NULL)
		exit(1);

	DBSETLUSER(login, USERNAME);
	DBSETLPWD(login, PASSWORD);
	DBSETLAPP(login, APPLICATION);

	if ((conn = dbopen(login, (char *)NULL)) == (DBPROCESS *)NULL)
		exit(1);

	dbloginfree(login);

	/* 
	** Perform REGWATCH operations. 
	*/

	/* Install a handler for all notification events.		*/
	if (dbreghandle(conn, (char *)NULL, 0, 
			(INTFUNCPTR)notify_handler) != SUCCEED)
		exit(1);

	/* Install a notifier for 'rp_version' registered procedure.	*/
	if (dbregwatch(conn, "rp_version", DBNULLTERM, DBASYNC | DBNOTIFYALWAYS)
		!= SUCCEED)
		exit(1);

	/* Loop waiting for 'rp_version' notifications.                  */
	while (TRUE)
	{
		(void)fprintf(stderr,
		"Now calling dbpoll() to wait for event notification...\n");

		if (dbpoll(conn, -1, &ready_conn, &response) != SUCCEED)
			(void)fprintf(stderr, "dbpoll() failed...\n");

		(void)fprintf(stderr, "dbpoll() returned.\n");
	}
		
	/* If connection with server is active, close it now.		*/
	if (conn != (DBPROCESS *)NULL)
		(void)dbclose(conn);

	(void)fprintf(stderr, "\ndbwait: Test was successful!\n");

	exit(0);
}

/*
**  NOTIFY_HANDLER
**      This is the handler for notification messages from an Open
**	Server application. This function prints the notification
**	message to the console.
**
**  Parameters:
**      dbproc   - The process handle for the Open Server application.
**      name 	 - The event notification name.
**
**  Returns:
**      INT_CONTINUE
*/
int		CS_PUBLIC notify_handler(dbproc, name, options)
DBPROCESS	*dbproc;
DBCHAR		*name;
DBUSMALLINT	options;
{
	printf("notify_handler() called - event %s...\n", name);
	return(INT_CONTINUE);
}


/*
**  ERR_HANDLER
**      This is the handler for DB-Library error messages.
**	This function prints the error message to the console.
**
**  Parameters:
**      dbproc   - The process handle for the Open Server application.
**      severity - The severity of the error.
**      dberr    - The DB-Library error number.
**      oserr    - The operating system error, if any.
**      errstr   - The text of the DB-Library error.
**      oserrstr - The text of operating system error, if any.
**
**  Returns:
**      INT_EXIT to exit the program.
**      INT_CANCEL to cause a FAIL return from the DB-Library routine that got
**      the error.
*/

int CS_PUBLIC err_handler(dbproc, severity, dberr, oserr, errstr, oserrstr)
DBPROCESS       *dbproc;
int             severity;
int             dberr;
int             oserr;
char            *errstr;
char            *oserrstr;
{
	(void)fprintf(stderr, "DB-Library error:\n\t%s\n", errstr);

	if (oserr != DBNOERR)
	{
		(void)fprintf(stderr, "Operating-system error:\n\t%s\n", 
			oserrstr);
	}

	if( (dbproc == NULL || (DBDEAD(dbproc) ) ) )
		return(INT_EXIT);
	else
		return(INT_CANCEL);
}

/*
**  MSG_HANDLER
**      DB-Library calls this routine when any messages are received
**      from the  Open Server application.  This function prints the
**	server messages to the console.
**
**  Parameters:
**      dbproc   - The handler to the Open Server process that sent the message.
**      msgno    - The message number.
**      msgstate - The message state.
**      severity - The message severity.
**      msgtext  - The text of the message.
**      srvname  - The name of the Open Server application that sent the message.
**      procname - The procedure name, if any, of the remote dbms command.
**      line     - The remote dbms command buffer line to which the msg applies.
**
**  Returns:
**      DBNOSAVE
*/

int CS_PUBLIC msg_handler(dbproc, msgno, msgstate, severity, msgtext, 
                	srvname, procname, line)
DBPROCESS       *dbproc;
DBINT           msgno;
int             msgstate;
int             severity;
char            *msgtext;
char            *srvname;
char            *procname;
DBUSMALLINT    	line;
{
	(void)fprintf(stderr, "Msg %ld, Severity %d, State %d\n", 
	        msgno, severity, msgstate);

	if (strlen(srvname) > 0)
		(void)fprintf(stderr, "Server '%s', ", srvname);
	if (strlen(procname) > 0)
		(void)fprintf(stderr, "Procedure '%s', ", procname);
	if (line > 0)
		(void)fprintf(stderr, "Line %d", line);

	(void)fprintf(stderr, "\n\t%s\n", msgtext);

	return(DBNOSAVE);
}
