/*
**	Sybase Open Server Version 10.0
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
** EXFDS.C 
**
**	This sample program demonstrates the use of external file descriptors
**	in an Open Server application.  This program starts two service threads
**	that communicate using pipes.  They implement a simple command/response
**	protocol.  srv_poll is used to coordinate the scheduling of these two
**	service threads.
**
**	A "real" Open Server application could use pipes between two
**	Open Servers applications (or any other UNIX process) to coordinate
**	activity. Using srv_poll allows the Open Server to be non-blocking while
**	waiting for the external work to finish.
**
**	This sample runs for NUMLOOPS interations and then queues a SRV_STOP
**	event to shutdown.
*/

/*
** Include the operating system specific header files required by this
** Open Server application.
*/
#include	<stdio.h>

/*
** If we are using TLI platform we need to include poll.h
*/
#if	(sun_svr4 || ncr || unixware)
#include	<poll.h>
#endif	/* (sun_svr4 || ncr || unixware) */

/*
** Include the required Open Server header files.
**
**	ospublic.h	Public Open Server structures, typedefs, defines, and
**			function prototypes.
**	ossample.h	Utility routines for the sample program
*/
#include	<ospublic.h>
#include	<ossample.h>

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

/*
** Prototypes.
*/
extern int main PROTOTYPE((
	int argc,
	char *argv[]
	));
extern CS_RETCODE exfds_start_handler PROTOTYPE((
	CS_VOID *ssp
	));
extern CS_RETCODE exfds_response_thread PROTOTYPE((
	CS_INT *responsefdp
	));
extern CS_RETCODE exfds_command_thread PROTOTYPE((
	CS_INT *commandfdp
	));
extern CS_RETCODE exfds_read_pipe PROTOTYPE((
	CS_INT readfd,
	CS_CHAR *bufp,
	CS_INT len,
	CS_INT *countp
	));
extern CS_RETCODE exfds_write_pipe PROTOTYPE((
	CS_INT writefd,
	CS_CHAR *bufp,
	CS_INT len
	));
extern CS_RETCODE exfds_wait PROTOTYPE((
	SRV_POLLFD *pollfdp,
	CS_INT count
	));
extern int close PROTOTYPE((
	int fd
	));
extern int pipe PROTOTYPE((
	int fd[2]
	));
extern int read PROTOTYPE((
	int fd,
	char *buf,
	int nbyte
	));
extern int write PROTOTYPE((
	int fd,
	char *buf,
	int nbyte
	));

/*
** Extern declarations.
*/
extern int	errno;

/*
** Manifest constants.
**
**	NUMLOOPS	Number of command response loops to perform.
*/
#define	NUMLOOPS	100

/*
** Local defines.
*/
#define	COMMAND		"Do this for me"
#define	RESPONSE	"OK"
#define	RESPONSEWAIT	"Response thread issuing a poll\n"
#define	RESPONSEMSG	"Response thread received (%d) bytes: (%s)\n"
#define	RESPONSESEND	"Response thread sending a response\n"
#define	COMMANDSEND	"Command thread sending a command\n"
#define	COMMANDWAIT	"Command thread issuing a poll\n"
#define	COMMANDMSG	"Command thread received (%d) bytes: (%s)\n"

/*
** Global data.
**
**	ServerName	Name of this Open Server application.
**	ResponseFD	Array of file descriptors for the response thread.
**	CommandFD	Array of file descriptors for the command thread.
**	ResponseThread	Response thread.
**	CommandThread	Command thread.
*/
CS_CHAR		ServerName[1024];
CS_INT		ResponseFD[] = 
		{
			(CS_INT)(-1),
			(CS_INT)(-1)
		};
CS_INT		CommandFD[] =
		{
			(CS_INT)(-1),
			(CS_INT)(-1)
		};
SRV_PROC	*ResponseThread = (SRV_PROC *)NULL;
SRV_PROC	*CommandThread = (SRV_PROC *)NULL;

/*
** This is the main entry point for this sample Open Server application.
** The command line syntax for this sample program is:
**
**	exfds <server name> <trace flags>
**
** The <trace flags> argument is optional.
**
** Returns:
**	0		Open Server exited successfully.
**	1		An error was detected during initialization.
*/
int	main(argc, argv)
int	argc;
char	*argv[];
{
	CS_CONTEXT	*cp;		/* Context Structure.		*/
	SRV_SERVER	*ssp;		/* Server control structure.	*/
	CS_BOOL		ok;		/* Error control flag.		*/

	/* Initialization.						*/
	ok = CS_TRUE;

	/*
	** Allocate a CS-Library context structure to define the default
	** localization information.  Global state information will also
	** be stored by Open Server in this structure during initialization.
	*/
	if(cs_ctx_alloc(CS_VERSION_100, &cp) != CS_SUCCEED)
	{
		(CS_VOID)fprintf(stderr, "cs_ctx_alloc failed");
		exit(1);
	}

	/*
	** Install a CS-Library error handler.  CS-Library is used
	** internally in Open Server for conversions.  All Open Server 
	** applications must install a CS-Library error handler if they want to
	** receive conversion errors.
	*/
	if(cs_config(cp, CS_SET, CS_MESSAGE_CB, (CS_VOID *)cs_err_handler,
		CS_UNUSED, (CS_INT *)NULL) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "cs_config(CS_MESSAGE_CB) failed");
		exit(1);
	}

	/*
	** Set the Open Server version and context information.
	*/
	if(srv_version(cp, CS_VERSION_100) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "srv_version failed \n");
		exit(1);
	}

	/*
	** Read the command line arguments.
	*/
	if(proc_args(cp, argc, argv, ServerName, CS_SIZEOF(ServerName),
		(CS_CHAR *)NULL, (CS_INT)NULL) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);
		exit(1);
	}

	/*
	** Install the Open Server error handler.
	*/
	if(srv_props(cp, CS_SET, SRV_S_ERRHANDLE,
		(CS_VOID *)server_err_handler, CS_SIZEOF(CS_VOID *),
		(CS_INT *)NULL) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);
		exit(1);
	}

	/*
	** Initialize Open Server.  This causes Open Server to allocate
	** internal control structures based on the server default properties.
	** The Open Server name is also looked up in the interfaces file.
	*/
	if((ssp = srv_init((SRV_CONFIG *)NULL, ServerName, CS_NULLTERM))
		== (SRV_SERVER *)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);
		exit(1);
	}

	/*
	** Make sure the file descriptor polling is supported on this
	** platform.
	*/
	if(!(srv_capability(SRV_C_POLL)))
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr,
			"File descriptor polling not supported\n");
		exit(1);
	}

	/*
	** Define a SRV_START handler to complete the initialization
	** of the pipe and the service threads.
	*/
	if(srv_handle(ssp, SRV_START, exfds_start_handler)
		== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);
		exit(1);
	}

	/*
	** Print a version string to the screen and to the log file.
	*/
	print_version(cp, ServerName);

	/*
	** Start the Open Server running.  We don't install any event
	** handlers in this simple example.  This will cause Open Server
	** to use the default event handlers.  The call to srv_run will not 
	** return until a fatal error is detected by this Open Server
	** application, or a SRV_STOP event is queued.  Since we haven't
	** installed any event handlers the only way to stop this Open Server
	** is to kill the operating system process in which it is running.
	*/
	if(srv_run((SRV_SERVER *)NULL) == CS_FAIL)
	{
		ok = CS_FALSE;
	}

	/*
	** Close the pipe file descriptors if they were opened.
	*/
	if(CommandFD[0] != (CS_INT)(-1))
	{
		(CS_VOID)close((int)CommandFD[0]);
	}
	if(CommandFD[1] != (CS_INT)(-1))
	{
		(CS_VOID)close((int)CommandFD[1]);
	}
	if(ResponseFD[0] != (CS_INT)(-1))
	{
		(CS_VOID)close((int)ResponseFD[0]);
	}
	if(ResponseFD[1] != (CS_INT)(-1))
	{
		(CS_VOID)close((int)ResponseFD[1]);
	}

	/*
	** NOTE:  We don't destroy our service threads here because
	** Open Server will kill them when the SRV_STOP event that causes
	** srv_run to return is raised.
	*/

	/*
	** Release the context structure.
	*/
	(CS_VOID)cs_ctx_drop(cp);

	if ( ok )
	{
		(CS_VOID)fprintf(stderr, "End test!\n");
	}
	else
	{
		(CS_VOID)fprintf(stderr, "Error dectected.\n");
	}
	exit(ok ? 0 : 1);
}

/*
** EXFDS_START_HANDLER
**
**	This is the start handler. The start handler creates the 
**	two service threads and opens the pipe that they will
**	use for communication.
**
**	If an error is detected during the start handler, CS_FAIL is returned
**	so that srv_run will return CS_FAIL to the main line code without
**	starting Open Server.
**
** Parameters:
**	ssp	A pointer to the Open Server control structure.			
**
** Returns:
**	CS_SUCCEED	Initialization succeeded.
**	CS_FAIL		An error was detected during initialization.
*/
CS_RETCODE	exfds_start_handler(ssp)
SRV_SERVER	*ssp;
{
	int		filedes[2];	/* Pipe file descriptors.	*/

	/*
	** Open the pipe that will be used for commands.  A UNIX pipe
	** consists of two file descriptors.  One is opened for reading
	** and the other is opened for writing.
	*/
	if(pipe(filedes) < 0)
	{
		(CS_VOID)fprintf(stderr, "%s: pipe open failed errno(%d)\n",
			ServerName, errno);
		return(CS_FAIL);
	}

	/*
	** Save the read and write file descriptors in the global command
	** and response file descriptor arrays.  We save the read file
	** descriptor in the array for the response thread, and the write
	** file descriptor in the array for the command thread.
	*/
	ResponseFD[0] = (CS_INT)filedes[0];
	CommandFD[1] = (CS_INT)filedes[1];

	/*
	** Open the pipe that will be used for responses.  This call will open
	** more file descriptors will be opened by this call.  Again, one
	** is for reading and the other for writing.
	*/
	if(pipe(filedes) < 0)
	{
		(CS_VOID)fprintf(stderr, "%s: pipe open failed errno(%d)\n",
			ServerName, errno);
		return(CS_FAIL);
	}

	/*
	** Save the read and write file descriptors in the global command
	** and response file descriptor arrays.  We save the read file
	** descriptor in the array for the command thread, and the write
	** file descriptor in the array for the response thread.
	*/
	CommandFD[0] = (CS_INT)filedes[0];
	ResponseFD[1] = (CS_INT)filedes[1];

	/*
	** Spawn the response service thread.
	*/
	if(srv_spawn(&ResponseThread, SRV_DEFAULT_STACKSIZE,
		(SRV_SERVICETHREAD_FUNC)exfds_response_thread, ResponseFD,
		SRV_DEFPRIORITY) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	/*
	** Spawn the command service thread.
	*/
	if(srv_spawn(&CommandThread, SRV_DEFAULT_STACKSIZE,
		(SRV_SERVICETHREAD_FUNC)exfds_command_thread, CommandFD,
		SRV_DEFPRIORITY) == CS_FAIL)
	{
		return(CS_FAIL);
	}
	return(CS_SUCCEED);
}

/*
** EXFDS_RESPONSE_THREAD
**
**	This is the response thread.  It issues a srv_poll call to 
**	wait for a command from the command thread.  It logs the
**	received command and sends a response back to the command thread.
**	It performs this NUMLOOPS number of iterations.
**
** Parameters:
**	responsefdp	A pointer to an array of read and write file descriptors
**			for the response thread.  The file descriptor at offset
**			0 is the read descriptor.  The file descriptor at offset
**			1 is the write file descriptor.
**
** Returns:
**
**	This service thread returns CS_SUCCEED when NUMLOOPS of iterations has 
**	occurred or CS_FAIL if an error is detected.
*/
CS_RETCODE	exfds_response_thread(responsefdp)
CS_INT		*responsefdp;
{
	SRV_POLLFD	pollfd;		/* poll structure		*/
	CS_INT		readfd;		/* read pipe file descriptor	*/
	CS_INT		writefd;	/* write pipe file descriptor	*/
	CS_CHAR		buf[512];	/* read buffer			*/
	CS_CHAR		msg[1024];	/* message buffer		*/
	CS_INT		count;		/* number of bytes read		*/
	CS_INT		loopcount;	/* number of loops left		*/

	/* Initialization.						*/
	loopcount = NUMLOOPS;
	readfd = responsefdp[0];
	writefd = responsefdp[1];
	srv_bzero(&pollfd, CS_SIZEOF(pollfd));

	/*
	** Set up the poll structure to wait until there is something to
	** read on the pipe file descriptor.
	*/
	pollfd.srv_fd = readfd;
	pollfd.srv_events = SRV_POLLIN;

	/*
	** This service thread exits when loopcount goes to 0 or an error
	** is detected.
	*/
	while(loopcount-- != (CS_INT)0)
	{
		/*
		** Log a message indicating that we are waiting for a 
		** command.
		*/
		(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, RESPONSEWAIT,
			CS_NULLTERM);

		/*
		** Wait for a command.
		*/
		if(exfds_wait(&pollfd, (CS_INT)(1)) == CS_FAIL)
		{
			return(CS_FAIL);
		}

		/*
		** A command was detected on the pipe.  Read the command.
		*/
		if(exfds_read_pipe(readfd, buf, (sizeof(buf) - 1), &count)
			== CS_FAIL)
		{
			return(CS_FAIL);
		}

		/*
		** Null terminate the data received from the command thread.
		*/
		buf[count] = '\0';

		/*
		** Build a message to write to the log file.
		*/
		(CS_VOID)sprintf(msg, RESPONSEMSG, count, buf);

		/*
		** Log the message in the log file.
		*/
		(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, msg, CS_NULLTERM);

		/*
		** Log a message indicating that we are responding to a command.
		*/
		(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, RESPONSESEND,
			CS_NULLTERM);

		/*
		** Send the command response message.
		*/
		if(exfds_write_pipe(writefd, RESPONSE, strlen(RESPONSE))
			== CS_FAIL)
		{
			return(CS_FAIL);
		}
	}
	return(CS_FAIL);
}

/*
** EXFDS_COMMAND_THREAD
**
**	This is the command service thread.  It issues a command to the
**	response thread and then does a poll to wait for the acknowledgement.
**	It performs this NUMLOOPS number of iterations.  When NUMLOOPS
**	iterations have occured a SRV_STOP event is queued to shut down
**	the Open Server.
**
** Parameters:
**	commandfdp	A pointer to an array of read and write file
**			descriptors for the command thread.  The file
**			descriptor at offset.
**			0 is the read descriptor. The file descriptor
**			at offset.
**			1 is the write descriptor.
**
** Returns:
**
**	This service thread returns CS_SUCCEED when NUMLOOPS of iterations has 
**	occurred or CS_FAIL if an error is detected.
*/
CS_RETCODE	exfds_command_thread(commandfdp)
CS_INT		*commandfdp;
{
	CS_INT		readfd;		/* Read file descriptor.	*/
	CS_INT		writefd;	/* Write file descriptor.	*/
	CS_INT		count;		/* Number of bytes read.	*/
	CS_INT		loopcount;
	SRV_POLLFD	pollfd;		/* Poll structure.		*/
	CS_CHAR		buf[512];	/* Read buffer.			*/
	CS_CHAR		msg[1024];	/* Message buffer.		*/

	/* Initialization.						*/
	loopcount = NUMLOOPS;
	readfd = commandfdp[0];
	writefd = commandfdp[1];
	srv_bzero(&pollfd, CS_SIZEOF(pollfd));

	/*
	** Set up the poll structure to wait until there is something to
	** read on the pipe file descriptor.
	*/
	pollfd.srv_fd = readfd;
	pollfd.srv_events = SRV_POLLIN;

	/*
	** This service thread exits when loopcount goes to 0 or an error
	** is detected.
	*/
	while(loopcount-- != (CS_INT)0)
	{
		/*
		** Log a message indicating that we are sending a command.
		*/
		(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, COMMANDSEND,
			CS_NULLTERM);

		/*
		** Send the command to the response thread.
		*/
		if(exfds_write_pipe(writefd, COMMAND, strlen(COMMAND))
			== CS_FAIL)
		{
			return(CS_FAIL);
		}

		/*
		** Log a message indicating that we waiting for a response.
		*/

		(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, COMMANDWAIT,
			CS_NULLTERM);

		/*
		** Wait for a response from the response thread.
		*/
		if(exfds_wait(&pollfd, (CS_INT)(1)) == CS_FAIL)
		{
			return(CS_FAIL);
		}

		/*
		** Read the response from the response thread.  We leave
		** one byte in the buffer for the null termination byte.
		*/
		if(exfds_read_pipe(readfd, buf, (sizeof(buf) - 1), &count)
			== CS_FAIL)
		{
			return(CS_FAIL);
		}

		/*
		** Null terminate the data received from the command thread.
		*/
		buf[count] = '\0';

		/*
		** Build a message to write to the log file.
		*/
		(CS_VOID)sprintf(msg, COMMANDMSG, count, buf);

		/*
		** Log the message in the log file.
		*/
		(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, msg, CS_NULLTERM);
	}

	/*
	** Raise a SRV_STOP event to shut down the Open Server application.
	** This will cause srv_run to return in main.
	*/
	if(srv_event(CommandThread, SRV_STOP, (CS_VOID *)NULL) == CS_FAIL)
	{
		(CS_VOID)fprintf(stderr, "%s: Cannot raise SRV_STOP\n",
			ServerName);
		exit(1);
	}
	return(CS_SUCCEED);
}

/*
** EXFDS_READ_PIPE
**
**	This routine performs a read on the specified pipe file descriptor.
**
** Parameters:
**	readfd		File descriptor.
**	bufp		A pointer to the buffer for read data.
**	len		Length of buffer.
**	countp		The amount of data read into bufp is returned here.
**
** Returns:
**	CS_SUCCEED	Data was read without error.
**	CS_FAIL		An error was detected while reading the data.
*/
CS_RETCODE	exfds_read_pipe(readfd, bufp, len, countp)
CS_INT		readfd;
CS_CHAR		*bufp;
CS_INT		len;
CS_INT		*countp;
{
	CS_INT	count;

	/* Initialization.						*/
	*countp = (CS_INT)0;

	/*
	** Read the requested amount of data from the pipe.
	*/
	if((count = read(readfd, bufp, len)) < 0)
	{
		(CS_VOID)fprintf(stderr, "%s: read failed (%d)\n", ServerName,
			errno);
		/*
		** Raise a SRV_STOP event so srv_run will return to the
		** mainline code.
		*/

		if(srv_event((SRV_PROC *)NULL, SRV_STOP, (CS_VOID *)NULL)
			== CS_FAIL)
		{
			(CS_VOID)fprintf(stderr, "%s: Cannot raise SRV_STOP\n",
				ServerName);
			exit(1);
		}
		return(CS_FAIL);
	}

	/*
	** Return the amount read to the caller.
	*/
	*countp = count;
	return(CS_SUCCEED);
}

/*
** EXFDS_WRITE_PIPE
**
**	This routine performs a write on the specified pipe file descriptor.
**
** Parameters:
**	writefd		File descriptor.
**	bufp		A pointer to the buffer of data to write.
**	len		Length of data in buffer.
**
** Returns:
**	CS_SUCCEED	Data was written without error.
**	CS_FAIL		An error was detected while writing the data.
*/
CS_RETCODE	exfds_write_pipe(writefd, bufp, len)
CS_INT		writefd;
CS_CHAR		*bufp;
CS_INT		len;
{
	/*
	** Write the data to the pipe.
	*/
	if(write(writefd, bufp, len) < 0)
	{
		(CS_VOID)fprintf(stderr, "%s: write failed (%d)\n",
			ServerName, errno);
		/*
		** Raise a SRV_STOP event so srv_run will return to the
		** mainline code.
		*/
		if(srv_event((SRV_PROC *)NULL, SRV_STOP, (CS_VOID *)NULL)
			== CS_FAIL)
		{
			(CS_VOID)fprintf(stderr, "%s: Cannot raise SRV_STOP\n",
				ServerName);
			exit(1);
		}
		return(CS_FAIL);
	}
	return(CS_SUCCEED);
}

/*
** EXFDS_WAIT
**
**	This routine performs a srv_poll to wait for the indicated activity
**	on the specified file descriptor.
**
** Parameters:
**	pollfdp		A pointer to a poll structure.
**	count		Number of poll structures to which pollfdp points.
**
** Returns:
**	CS_SUCCEED	srv_poll was successful.
**	CS_FAIL		An error was detected by srv_poll.
*/
CS_RETCODE	exfds_wait(pollfdp, count)
SRV_POLLFD	*pollfdp;
CS_INT		count;
{
	/*
	** Call srv_poll to wait for the specified activity.
	*/
	if(srv_poll(pollfdp, count, SRV_M_WAIT) <= (CS_INT)0)
	{
		(CS_VOID)fprintf(stderr, "%s: srv_poll failed\n", ServerName);

		/*
		** Raise a SRV_STOP event so srv_run will return to the
		** mainline code.
		*/
		if(srv_event((SRV_PROC *)NULL, SRV_STOP, (CS_VOID *)NULL)
			== CS_FAIL)
		{
			(CS_VOID)fprintf(stderr, "%s: Cannot raise SRV_STOP\n",
				ServerName);
			exit(1);
		}
		return(CS_FAIL);
	}
	return(CS_SUCCEED);
}
