/*
**	Sybase Open Client Full Passthru Gateway
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
**  FULLPASS
**
**  	This is a sample program that demonstrates the use of full
**  	pass-through mode. It reads packets from a client and passes 
**	them to a remote SQL Server in pass through mode.  It then 
**	gets the results from the remote SQL Server and passes them 
**	to the client in pass-through mode.
**	
**  Syntax:
**
**	fullpass Open_Server_Name -s SQLServer [trace flags]
**
**  Arguments:
**
**	Open_Server_Name	The name of this Open Server.
**	-s SQLServer		SQL server to connect to.
**	traceflags		Any of the valid trace flags.
**
**  Returns:
**    0		Open Server exited successfully.
**    1		An error was detected.
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ospublic.h>
#include	<ctpublic.h>
#include	<oserror.h>
#include	<ossample.h>

#if     (USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif  /* (USE_SCCSID)	*/

/*
**	Prototypes.
*/
CS_RETCODE CS_PUBLIC fpass_conn_handler PROTOTYPE ((
	SRV_PROC	*sproc
	));

CS_RETCODE CS_PUBLIC fpass_lang_handler PROTOTYPE ((
	SRV_PROC	*sproc
	));

CS_RETCODE fpass_init_context PROTOTYPE (( 
	CS_CONTEXT	**context 
	));

CS_RETCODE fpass_cfgsrv PROTOTYPE ((
	CS_CHAR		*oserver,
	CS_CONTEXT	*context
	));

CS_RETCODE CS_PUBLIC fpass_disc_handler PROTOTYPE ((
	SRV_PROC	*sproc
	));

CS_VOID fpass_donestatus PROTOTYPE ((
	SRV_PROC	*sproc,
	CS_INT		status
	));

/*
**	GLOBAL VARIABLES.
**
**	Sqlserver	The name of the SQL Server.  This will be
**			used in the connection handler, and there
**			isn't a way to pass this in.
*/
CS_CHAR		Sqlserver[CS_MAX_CHAR];
CS_CONTEXT	*Context;	/* Make this global to go around
				** a potential bug in cs_ctx_global.
				*/

int	main(argc, argv)
int	argc;
char	*argv[];
{

	CS_CHAR		oserver[CS_MAX_CHAR];	/*
						** The Open Server
						** application name.
						*/

	/*
	** 	Allocate the Open Client context structure and set the 
	** 	Open Server version and context information.
	*/
	if ( fpass_init_context(&Context) == CS_FAIL )
	{
		exit(1);
	}

	/*
	** 	Get the Open Server application name.
	*/
	if ( proc_args(Context, argc, argv, oserver, CS_SIZEOF(oserver),
		Sqlserver, CS_SIZEOF(Sqlserver)) == CS_FAIL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	/*
	** 	Configure and initialize Open Server.
	*/
	if ( fpass_cfgsrv(oserver, Context) == CS_FAIL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	/*
	** 	Install the connection and language handlers.
	*/
	if ( srv_handle( (SRV_SERVER *)NULL, SRV_START, stop_regproc)
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	if ( srv_handle( (SRV_SERVER *)NULL, SRV_CONNECT, fpass_conn_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	if ( srv_handle( (SRV_SERVER *)NULL, SRV_LANGUAGE, fpass_lang_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	if ( srv_handle( (SRV_SERVER *)NULL, SRV_BULK, fpass_lang_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	if ( srv_handle( (SRV_SERVER *)NULL, SRV_CURSOR, fpass_lang_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	if ( srv_handle( (SRV_SERVER *)NULL, SRV_DYNAMIC, fpass_lang_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	if ( srv_handle( (SRV_SERVER *)NULL, SRV_MSG, fpass_lang_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	if ( srv_handle( (SRV_SERVER *)NULL, SRV_OPTION, fpass_lang_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	if ( srv_handle( (SRV_SERVER *)NULL, SRV_RPC, fpass_lang_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	if ( srv_handle( (SRV_SERVER *)NULL, SRV_DISCONNECT, 
		fpass_disc_handler) == (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	/*
	** 	Start Open Server.
	*/
	if ( srv_run((SRV_SERVER *)NULL) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, "Failed to start %s\n", oserver);
		(CS_VOID)cs_ctx_drop(Context);
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		exit(1);
	}

	/*
	** 	Free the context.
	*/
	(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);

	(CS_VOID)cs_ctx_drop(Context);

	exit(0);
}

/*
** fpass_conn_handler()
**
**	This is the connection handler which is invoked whenever
**	a connection has been made to the full passthrough
**	gateway Open Server.
**
** Parameters:
**
**	sproc		A pointer to an internal thread control
**			structure.
**
** Returns:
**
**	CS_SUCCEED	If all goes well.
**	CS_FAIL		If there's an error.
**
*/
CS_RETCODE CS_PUBLIC fpass_conn_handler(sproc)
SRV_PROC	*sproc;
{
	CS_INT			general_csint;
	CS_CONNECTION		*conn;
	CS_CHAR			general_str[CS_MAX_CHAR];
	CS_LOGINFO		*loginfo;
	CS_INT			status;
	CS_CONTEXT		*context;

	/*
	**	Assume that everything is ok to start.
	*/
	status = SRV_DONE_FINAL;

	/*
	**	If this is a site handler, just send a DONE and return.
	*/
	if ( srv_thread_props(sproc, CS_GET, SRV_T_TYPE, 
		(CS_VOID *)&general_csint, sizeof(CS_INT), 
		(CS_INT *)NULL) == CS_FAIL )
	{
		fpass_donestatus(sproc, status | SRV_DONE_ERROR);
		return(CS_FAIL);
	}

	if ( general_csint == SRV_TSITE )
	{
		fpass_donestatus(sproc, status);
		return(CS_SUCCEED);
	}

	/*
	**	Retrieve the context for this Open Server application.
	*/
	if ( cs_ctx_global(CS_VERSION_100, &context) == CS_FAIL )
	{
		(CS_VOID)srv_log( (SRV_SERVER *)NULL, CS_TRUE,
			"Can't get context needed by ct_con_alloc\n",
			CS_NULLTERM);
		
		return(CS_FAIL);
	}

	/*
	** 	Get a connection handle for the connection attempt.
	**
	**	Note that the context used here is the global
	**	CS_CONTEXT data.  The context structure returned
	**	by ct_ctx_global doesn't seem to work.  The client
	**	got "Login incorrect" error message. This may be
	**	a bug.
	*/
	if ( ct_con_alloc(Context, &conn) == CS_FAIL )
	{
		fpass_donestatus(sproc, status | SRV_DONE_ERROR);
		return(CS_FAIL);
	}
		
	/*
	** 	Get and set username, user password,
	**	application name, TDS packet size, etc.
	*/
	if ( srv_getloginfo(sproc, &loginfo) == CS_FAIL )
	{
		(CS_VOID)ct_con_drop(conn);
		return(CS_FAIL);
	}

	/*
	**	We check for CS_SUCCEED because this call
	**	could return CS_BUSY.
	*/
	if ( ct_setloginfo(conn, loginfo) != CS_SUCCEED )
	{
		ct_con_drop(conn);
		return(CS_FAIL);
	}

	/*
	** 	Get and print the TDS packet size.  This is
	**	only for informational purposes, so we will
	**	continue even if the call fails.
	*/
	if ( srv_thread_props(sproc, CS_GET, SRV_T_PACKETSIZE, 
		&general_csint, sizeof(CS_INT), (CS_INT *)NULL)
		== CS_SUCCEED )
	{
		/*
		**	Print out the client's requested packet size to
		**	the Open Server application log file.
		*/
		(CS_VOID)sprintf(general_str, 
			"Packet size requested %d\n", general_csint);

		(CS_VOID)srv_log( (SRV_SERVER *)NULL, CS_TRUE, general_str, 
			CS_NULLTERM);
	}

	/*
	** 	Attempt a connection to the remote DBMS defined in
	** 	the DSQUERY enviroment variable.
	*/
	if ( ct_connect(conn, Sqlserver, CS_NULLTERM) == CS_FAIL )
	{
		(CS_VOID)sprintf(general_str,
			"Can't connect to remote server %s\n", Sqlserver);

		(CS_VOID)srv_log( (SRV_SERVER *)NULL, CS_TRUE, general_str,
			CS_NULLTERM);

		fpass_donestatus(sproc, status | SRV_DONE_ERROR);

		(CS_VOID)ct_con_drop(conn);

		return(CS_FAIL);
	}

	(CS_VOID)sprintf(general_str, "Connected to remote server %s\n",
		Sqlserver);

	(CS_VOID)srv_log( (SRV_SERVER *)NULL, CS_TRUE, general_str,
		CS_NULLTERM);

	/*
	**	Get the login reponse from the remote server,
	**	and pass it to the client.
	*/
	if ( ct_getloginfo(conn, &loginfo) == CS_FAIL )
	{
		(CS_VOID)ct_close(conn, CS_FORCE_CLOSE);

		(CS_VOID)ct_con_drop(conn);
	
		fpass_donestatus(sproc, status | SRV_DONE_ERROR);

		return(CS_FAIL);
	}

	if ( srv_setloginfo(sproc, loginfo) == CS_FAIL )
	{
		(CS_VOID)ct_close(conn, CS_FORCE_CLOSE);

		(CS_VOID)ct_con_drop(conn);

		fpass_donestatus(sproc, status | SRV_DONE_ERROR);

		return(CS_FAIL);
	}

	/*
	**	Save the connection handle in SRV_PROC's user
	**	data space.  This is done almost at the last minute
	**	so that we don't have to clear this user pointer 
	**	if something goes wrong before returning.
	*/
	if ( srv_thread_props(sproc, CS_SET, SRV_T_USERDATA, conn,
		sizeof(CS_CONNECTION *), (CS_INT *)NULL) == CS_FAIL )
	{
		(CS_VOID)ct_close(conn, CS_FORCE_CLOSE);

		(CS_VOID)ct_con_drop(conn);

		fpass_donestatus(sproc, status | SRV_DONE_ERROR);

		return(CS_FAIL);
	}

	/*
	**	Send the client the connection acknowlegement.
	*/
	fpass_donestatus(sproc, status);
			
	return(CS_SUCCEED);
}


/*
** fpass_lang_handler()
**
**	This is the language handler which is invoked whenever
**	a client sends over a language command. It reads
**	whatever the client sends over and passes the packet
**	to the remote server in passthrough mode.  It then
**	reads the results from the remote server and passes
**	them to the client also in pass-through mode.
**
** Parameters:
**
**	sproc		A pointer to an internal thread control
**			structure.
**
** Returns:
**
**	CS_SUCCEED	If all goes well.
**	CS_FAIL		If something is wrong.
**
*/
CS_RETCODE CS_PUBLIC fpass_lang_handler(sproc)
SRV_PROC	*sproc;
{
	CS_CONNECTION	*conn;		/* connection handle. */
	CS_COMMAND	*cmd;
	CS_BYTE		*packet;
	CS_INT		srvinfo;
	CS_RETCODE	ctinfo;
	CS_BOOL		passthru_ok;
	CS_INT		status;

	/*
	**	Assume that everything is fine to start.
	*/
	status = SRV_DONE_FINAL;
	
	/*
	**	Get the connection handle for this connection.
	*/
	if ( srv_thread_props(sproc, CS_GET, SRV_T_USERDATA, &conn,
		sizeof(CS_CONNECTION *), (CS_INT *)NULL) == CS_FAIL )
	{
		fpass_donestatus(sproc, status | SRV_DONE_ERROR);
	
		return(CS_FAIL);
	}

	/*
	**	Allocate a command handle. 
	*/
	if ( ct_cmd_alloc(conn, &cmd) == CS_FAIL )
	{
		fpass_donestatus(sproc, status | SRV_DONE_ERROR);

		return(CS_FAIL);
	}

	/*
	**	Read the packet from the client and send it to
	**	the remote server.
	*/
	srvinfo = SRV_I_PASSTHRU_MORE;
	passthru_ok = CS_TRUE;

	while ( srvinfo != SRV_I_PASSTHRU_EOM )
	{
		if ( srv_recvpassthru(sproc, &packet, &srvinfo) == CS_FAIL )
		{
			passthru_ok = CS_FALSE;
			break;
		}

		/*
		** 	Send the packet to the remote server.
		*/
		ctinfo = ct_sendpassthru(cmd, (CS_VOID *)packet);

		/*
		**	Make sure that ct_sendpassthru didn't
		**	return something other than CS_PASSTHRU_EOM
		**	and CS_PASSTHRU_MORE.
		*/
		if ( (ctinfo != CS_PASSTHRU_EOM) 
			&& (ctinfo != CS_PASSTHRU_MORE) )
		{
			passthru_ok = CS_FALSE;
			break;
		}
	}

	/*
	**	If everything went well, the last ct_sendpassthru
	**	should have returned CS_PASSTHRU_EOM.
	*/
	if ( ctinfo != CS_PASSTHRU_EOM )
	{
		passthru_ok = CS_FALSE;
	}

	/*
	**	If there's a pass-through error, clean things up
	**	and queue a SRV_STOP event.
	*/
	if ( !passthru_ok )
	{
		/*
		**	Free the command handle.
		*/
		(CS_VOID)ct_cmd_drop(cmd);

		/*
		**	A srv_senddone is neccessary after
		**	the srv_recvpassthru call.
		*/
		(CS_VOID)srv_senddone(sproc, SRV_DONE_ERROR,
			CS_TRAN_UNDEFINED, (CS_INT)0);

		/*
		**	Stop Open Server.
		*/
		if ( srv_event(sproc, SRV_STOP, (CS_VOID *)NULL) 
			== CS_FAIL )
		{
			/*
			**	Since we can't queue the SRV_STOP
			**	event, we will just exit.
			*/
			exit(1);
		}

		return(CS_FAIL);
	}

	/*
	**	We finished passing the packets from the client
	**	to the remote server.  Now we are going to read
	**	the results from the remote server and pass them
	**	along to the client.
	*/
	ctinfo = CS_PASSTHRU_MORE;

	while ( ctinfo == CS_PASSTHRU_MORE )
	{
		/*
		**	Read the results from the remote server.
		*/
		ctinfo = ct_recvpassthru(cmd, (CS_VOID **)&packet);
		
		/*
		**	Make sure that ct_recvpassthru didn't
		**	return something other than
		**	CS_PASSTHRU_MORE and CS_PASSTHRU_EOM.
		*/
		if ( (ctinfo != CS_PASSTHRU_MORE) 
			&& (ctinfo != CS_PASSTHRU_EOM) )
		{
			passthru_ok = CS_FALSE;
			break;
		}
			
		/*
		**	Now send the packet to the client.
		*/
		if (srv_sendpassthru(sproc, packet, &srvinfo) == CS_FAIL )
		{
			passthru_ok = CS_FALSE;
			break;
		}
	}

	/*
	**	If all went well, srv_sendpassthru should set srvinfo
	**	to SRV_I_PASSTHRU_EOM.
	*/
	if ( srvinfo != SRV_I_PASSTHRU_EOM )
	{
		passthru_ok = CS_FALSE;
	}

	/*
	**	If there's any error, clean up, and stop the server.
	*/
	if ( !passthru_ok )
	{
		(CS_VOID)ct_cmd_drop(cmd);

		fpass_donestatus(sproc, status | SRV_DONE_ERROR);

		/*
		**	If we can't queue a SRV_STOP event, exit.
		*/
		if ( srv_event(sproc, SRV_STOP, (CS_VOID *)NULL)
			== CS_FAIL )
		{
			(CS_VOID)srv_log( (SRV_SERVER *)NULL, CS_TRUE,
			  "Queuing a SRV_STOP event failed. Exiting..\n",
			  CS_NULLTERM);

			exit(1);
		}

		return(CS_FAIL);
	}

	(CS_VOID)ct_cmd_drop(cmd);
	
	return(CS_SUCCEED);
}

/*
** fpass_init_context()
**
**	This is the routine to set the version and the context
**	information.
**
** Parameters:
**
**	context		The context structure.
**
** Returns:
**
**	CS_SUCCEED	If all goes well.
**	CS_FAIL		If something is wrong.
**
*/
CS_RETCODE fpass_init_context(context)
CS_CONTEXT	**context;
{
	/*
	** 	Allocate memory for the context structure.
	*/
	if ( cs_ctx_alloc(CS_VERSION_100, context) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, 
			"Couldn't allocate Open Client context\n");
		return(CS_FAIL);
	}

	/*
	** 	Install the Open Client error handler.
	*/
	if ( cs_config(*context, CS_SET, CS_MESSAGE_CB, 
		(CS_VOID *)cs_err_handler, CS_UNUSED, NULL) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, 
			"Failed to install Open Client error handler\n");

		(CS_VOID)cs_ctx_drop(*context);

		return(CS_FAIL);
	}


	/*
	** 	Set up the version and context information.
	*/
	if ( srv_version(*context, CS_VERSION_100) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, "Couldn't set Open Server version\n");
		(CS_VOID)cs_ctx_drop(*context);
		return(CS_FAIL);
	}

	/*
	** 	Initialize the Open Client Library.
	*/
	if ( ct_init(*context, CS_VERSION_100) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, 
			"Couldn't initialize the Open Client\n");
		(CS_VOID)cs_ctx_drop(*context);
		return(CS_FAIL);
	}
	
	return(CS_SUCCEED);
}


/*
** fpass_cfgsrv()
**
**	This is the routine to configure the Open Server application.  It
**	installs error handlers, sets the Open Server application error log
**	file, etc., initializes Open Server, and prints out the version
**	string.
**
** Parameters:
**
**	oserver		The name of the Open Server application.
**	context		The context structure.
**
** Returns:
**
**	CS_SUCCEED	If all goes well.
**	CS_FAIL		If something is wrong.
*/
CS_RETCODE fpass_cfgsrv(oserver, context)
CS_CHAR		*oserver;
CS_CONTEXT	*context;
{
	CS_CHAR		logname[CS_MAX_CHAR];	/*
						** Open Server application
						** log file
						*/

	CS_BOOL		boolbuf;

	/*
	** 	Install the Open Server application error handler.
	*/
	if ( srv_props(context, CS_SET, SRV_S_ERRHANDLE, 
		(CS_VOID *)server_err_handler, sizeof(CS_VOID *), 
		(CS_INT *)NULL) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, 
			"Failed to intall Open Server error handler\n");
		return(CS_FAIL);
	}

	/*
	** 	Construct the Open Server application log name.
	*/
	(CS_VOID)sprintf(logname, "%s.log", oserver);

	/*
	** 	Configure the Open Server error application log file.
	*/
	if ( srv_props(context, CS_SET, SRV_S_LOGFILE, logname,
		CS_NULLTERM, (CS_INT *)NULL) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, "Failed to set the error log file\n");
		return(CS_FAIL);
	}

	/*
	** 	We want a fresh log file each time Open Server 
	**	starts up. 
	*/
	boolbuf = CS_TRUE;

	if ( srv_props(context, CS_SET, SRV_S_TRUNCATELOG, (CS_VOID *)&boolbuf,
		sizeof(CS_BOOL), (CS_INT *)NULL) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, 
			"Failed to trucate the error log file\n");
		return(CS_FAIL);
	}


	/*
	** 	Initialize the full pass through gateway. 
	*/
	if ( srv_init((SRV_CONFIG *)NULL, oserver, CS_NULLTERM)
		== (SRV_SERVER *)NULL )
	{
		(CS_VOID)fprintf(stderr, "srv_init failed\n");
		return(CS_FAIL);
	}

	/*
	**	Get the version string and print it to both
	**	stderr and the log file.
	*/
	print_version(context, oserver);

	return(CS_SUCCEED);
}

/*
**
** fpass_disc_handler()
**
**	This routine is invoked when a client drops a connection.
**	It closes the connection and frees the memory associated
**	with the connection.
**
** Parameters:
**
**	sproc		A pointer to an internal thread control
**			structure.
**
** Returns:
**
**	CS_SUCCEED	If all goes well.
**	CS_FAIL		If the connection cannot be retrieved.
**
*/
CS_RETCODE CS_PUBLIC fpass_disc_handler(sproc)
SRV_PROC	*sproc;
{
	CS_CONNECTION	*conn;

	/*
	**	Retrieve the connection handler stored in the client
	**	thread's user data space.
	*/
	if ( srv_thread_props(sproc, CS_GET, SRV_T_USERDATA, &conn,
		sizeof(CS_CONNECTION *), (CS_INT *)NULL) == CS_FAIL )
	{
		return(CS_FAIL);
	}

	/*
	**	Close and drop the connection.
	**	We want to go on even if we can't
	**	close or drop the connection successfully.
	*/
	(CS_VOID)ct_close(conn, CS_FORCE_CLOSE);

	(CS_VOID)ct_con_drop(conn);

	return(CS_SUCCEED);
}

/*
**	fpass_donestatus()
**
**
*/
CS_VOID fpass_donestatus(sproc, status)
SRV_PROC	*sproc;
CS_INT		status;
{
	if ( srv_senddone(sproc, status, CS_TRAN_UNDEFINED, (CS_INT)0)
		== CS_FAIL )
	{
		(CS_VOID)srv_log( (SRV_SERVER *)NULL, CS_TRUE,
			"srv_senddone failed. Exiting..\n", CS_NULLTERM);

		/*
		**	Need to stop the server if sending DONE failed.
		*/
		if ( srv_event(sproc, SRV_STOP, (CS_VOID *)NULL)
			== CS_FAIL )
		{
			(CS_VOID)srv_log( (SRV_SERVER *)NULL, CS_TRUE,
				"Queuing SRV_STOP event failed. Exiting..\n",
				CS_NULLTERM);

			exit(1);
		}
	}

	return;
}
