

/*
**
**	Sybase Open Server - Multthrd Sample Program
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
** MULTTHRD
**
** This sample program illustrates the creation of a SERVICE thread 
** using srv_spawn, inter-thread communication using  messages queues,
** and the printing of thread scheduling information using callbacks. 
**
** In this sample program, client threads retrieve the language queries
** sent by clients, and send a message to a service thread called 
** "language logger". The client threads then sleep, waiting for the language 
** logger to log the language queries to a file ("language.log") 
** and wake up the  senders of messages. The language logger loops 
** continuously, waiting for messages. When it gets one, it prints the 
** language query in the message to a file, and wakes up the sender.  
** All client threads and the language logger install 
** callbacks(SRV_C_RESUME, SRV_C_SUSPEND, SRV_C_TIMESLICE and SRV_C_EXIT) 
** to write scheduling information to a file  ("<appl_name>.sched").
** 
** This sample program responds to language queries by sending a message 
** indicating whether the query was successfully logged or not.  
** A client program can stop multthrd  by executing a registered procedure 
** 'stop_serv'. Multthrd queues a SRV_STOP event if any error occurs 
** in the wake-up/sleep or inter-thread communication mechanisms.
**
**  Usage:
**
**      multthrd [server name] [trace flags]
**
**	If <server name> is not specified, the value of the DSLISTEN
**	environment variable will be used as the server name.
**
**  Returns:
**
**	0	Open Server exited successfully.
**	1	An error was detected during initialization.
*/

/*
** Include the operating system-specific header files required by the
** Open Server application.
*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

/*
** Include the required Open Server header files.
**
**      ospublic.h      Public Open Server structures, typedefs, defines, and
**                      function prototypes.
**	oserror.h	Open Server error number defines.  This header file
**			is only required if the Open Server application
**			wants to detect specific errors in the Open Server
**			error handler.
**      ossample.h      Utility routines for the sample program.
*/

/* SYBASE header files required by Open Server. 			*/
#include	<ospublic.h>
#include	<oserror.h>
#include	<ossample.h>

#if     (USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif  /* (USE_SCCSID) */

/* Error message defines.						*/
#define	BADVIRTCONFIG	"Could not configure SRV_S_VIRTTIMER... Exiting...\n"
#define	BADTIMECONFIG	"Could not configure SRV_S_TIMESLICE... Exiting...\n"
#define	BADLOGCONFIG	"Could not configure SRV_LOGFILE...  Exiting...\n"
#define	BADLOGTRUNC	"Could not configure to truncate log... Exiting...\n"
#define BADVERSION	"Could not init Open Server Library version...\n"
#define	BADINIT		"Could not perform initialization... Exiting... \n"
#define	BADCONTEXTALLOC	"Could not allocate Context...  Exiting...\n"
#define SENDDONEFAIL	"srv_senddone() failed... \n"
#define	SENDINFOFAIL	"srv_sendinfo() failed... \n"
#define	BADCSERRHNDL	"cs_config(CS_MESSAGE_CB) failed..\n"
#define	BADSERVERRHNDL	"cs_config(CS_MESSAGE_CB) failed..\n"
#define	BADSRVHANDLE	"srv_handle failed...\n"
#define	BADSRVRUN	"srv_run failed...\n"


/* Local defines.							*/
#define	LOGGER		"language_logger"
#define	LOGWAIT		"Waiting for Language Logger"
#define	LOG_SUFFIX	"log"
#define	SCHED_SUFFIX	"sch"
#define	STOP_SERVER_MSG	"Stopping Server..."
#define	STOP_SERV	"stop_serv"
#define	MSGSIZE		1024    
#define	LOG_OK_MSG	"Language query logged"
#define	LOG_FAIL_MSG	"Error in logging language query"

/* Configuration defines.						*/
#define	LANGLOG		"language.log"	/* To log language queries	*/
#define TIMESLICE	(CS_INT)2	/* for SRV_S_TIMESLICE property	*/

/* Open Server user defined error code.					*/
#define LOG_FAIL_NUM	(CS_INT)(SRV_MAXERROR +1)
#define LOG_OK_NUM	(CS_INT)(SRV_MAXERROR +2)

/* Lint cleaners and miscellaneous defines.				*/
#define	FPRINTF			(CS_VOID)fprintf
#define	SPRINTF			(CS_VOID)sprintf
#define	FCLOSE			(CS_VOID)fclose
#define STRLEN(str)             (CS_INT)strlen((char *)(str))
#define STRCPY(D,S)             (CS_VOID)strcpy((char *)(D), (char *)(S))
#define	STRNCPY(D,S,N)		(CS_VOID)strncpy((char *)(D), (char *)(S), (size_t)(N))

/* Useful typecasts.							*/
#define	NEG1		(CS_INT)(-1)
#define	ZERO		(CS_INT)0

/*
** Message structure used for inter-thread communication.
** Client threads use this structure for communicating with the
** the language logger thread.
*/
typedef	struct log_msg
{
	CS_INT	spid;		
	CS_CHAR	*lang_query;
}LOG_MSG;

/* 
** Prototypes.
*/
CS_RETCODE multthrd_cfg_init PROTOTYPE((
	CS_CONTEXT	**cpp
	));
CS_RETCODE CS_PUBLIC multthrd_start_handler PROTOTYPE((
	SRV_SERVER	*serverp
	));
CS_RETCODE CS_PUBLIC multthrd_stop_serv PROTOTYPE((
	SRV_PROC	*cl_threadp
	));
CS_RETCODE CS_PUBLIC multthrd_conn_handler PROTOTYPE((
	SRV_PROC	*cl_threadp
	));
CS_RETCODE CS_PUBLIC multthrd_lang_handler PROTOTYPE((
	SRV_PROC	*cl_threadp
	));
CS_RETCODE multthrd_start_languagelogger PROTOTYPE((
	CS_VOID
	));
CS_RETCODE CS_PUBLIC multthrd_language_logger PROTOTYPE((
	CS_VOID *dummy
	));
CS_RETCODE multthrd_send_logmsg PROTOTYPE((
	SRV_PROC	*cl_threadp,
	CS_CHAR		*lang_query
	));
CS_RETCODE multthrd_install_schedinfocb PROTOTYPE((
	SRV_PROC	*threadp
	));
CS_RETCODE	CS_PUBLIC multthrd_resume_cb PROTOTYPE((
	SRV_PROC	*threadp
	));
CS_RETCODE	CS_PUBLIC multthrd_suspend_cb PROTOTYPE((
	SRV_PROC	*threadp
	));
CS_RETCODE	CS_PUBLIC multthrd_timeslice_cb PROTOTYPE((
	SRV_PROC	*threadp
	));
CS_RETCODE	CS_PUBLIC multthrd_exit_cb PROTOTYPE((
	SRV_PROC	*threadp
	));
CS_RETCODE CS_PUBLIC multthrd_queue_stopevent PROTOTYPE((
	SRV_PROC	*threadp
	));
CS_RETCODE multthrd_get_threadinfo PROTOTYPE((
	SRV_PROC	*threadp,
	CS_CHAR         *buf,
	CS_INT          size
	));


/* Global data declarations.						*/
CS_CHAR	*Name = '\0';			/* Application name.		*/
FILE	*Sched_fp = (FILE *)NULL;	/* Scheduling info file pointer.*/
FILE	*Langlog_fp = (FILE *)NULL;	/* Language log file pointer.	*/
CS_CONTEXT *Ctxp = (CS_CONTEXT *)NULL; 	/* Context Control structure.	*/

int	main(argc, argv)
int	argc;
char	*argv[];
{
	CS_INT		rv;		/* Value to return to env.	  */
	SRV_SERVER	*serverp;	/* Open Server control structure  ptr.*/
	CS_CHAR		servername[1024]; /* Open Server application name.  */

	/* Initialization.						*/
	rv = 0; 			/* Assume non-error exit.	*/
	Name = argv[1];			/* Use the servername		*/

	/* Configure Multthrd Open Server.				*/
	if(multthrd_cfg_init(&Ctxp) == CS_FAIL) 
		exit(1);

	/*
	** Read the command line arguments.
	*/
        if(proc_args(Ctxp, argc, argv, servername, CS_SIZEOF(servername),
		(CS_CHAR *)NULL, (CS_INT)0) == CS_FAIL)
        {
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(Ctxp);

                exit(1);
        }

	/* Initialize multthrd. 					*/
	if ((serverp = srv_init((SRV_CONFIG *)NULL, 
		(STRLEN(servername) > 0) ? servername : (CS_CHAR *)NULL,  
		(STRLEN(servername) > 0) ? CS_NULLTERM : ZERO)) 
		== (SRV_SERVER *)NULL)
	{
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(Ctxp);

		FPRINTF(stderr, "%s: %s", Name, BADINIT);
		exit(1);
	}

	/* 
	** Install start handler. It installs other event handlers, 
	** spawns the language logger thread, and opens files to log
	** scheduling information and language queries.
	*/
	if (srv_handle((SRV_SERVER *)NULL, SRV_START, 
		multthrd_start_handler) == (SRV_EVENTHANDLE_FUNC)NULL)
	{
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(Ctxp);
		
		FPRINTF(stderr, "%s: %s", Name, BADSRVHANDLE);
		exit(1);
	}

	/* Print a version string to the screen and to the log file.	*/
	print_version(Ctxp, Name);
		
	/* Start up Open Server.					*/
	if(srv_run((SRV_SERVER *)NULL) == CS_FAIL)
	{
		rv = 1;
		FPRINTF(stderr, "%s: %s", Name, BADSRVRUN);
	}

	/* Release memory allocated for the context control structure.	*/
	(CS_VOID)cs_ctx_drop(Ctxp);
	exit(rv);
	return(0);
}

/*
** MULTTHRD_CFG_INIT 
** 
** Purpose:
**  	Configure the required parameters and initialize multthrd.
**
**  Parameters:
**	cpp - A pointer to  CS_CONTEXT control structure pointer.
**	spp - A pointer to the Open Server application's control
**		 structure pointer.
**	servname - A pointer to server name.
**
**  Returns:
**	CS_SUCCEED:	The application is configured and initialized 
**			successfully.
**	CS_FAIL:	A failure occurred in configuring/initializing.
*/
CS_RETCODE	multthrd_cfg_init(cpp)
CS_CONTEXT	**cpp;
{
	CS_BOOL		boolval;	
	CS_INT		intval;		
	CS_CHAR		filename[CS_MAX_CHAR];

	/* Allocate memory for the context structure.			*/
	if(cs_ctx_alloc(CS_VERSION_100, cpp) == CS_FAIL)
	{
		FPRINTF(stderr, "%s: %s", Name, BADCONTEXTALLOC);
		return (CS_FAIL);
	}
	
	/* Install the CS-Library error handler.			*/
	if(cs_config(*cpp, CS_SET, CS_MESSAGE_CB, (CS_VOID *)cs_err_handler,
                CS_UNUSED, (CS_INT *)NULL) == CS_FAIL)
        {
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(*cpp);
 
                (CS_VOID)fprintf(stderr, "%s: %s", Name, BADCSERRHNDL);
		return (CS_FAIL);
        }

	/* Define the Open Server version.			*/
	if (srv_version(*cpp, CS_VERSION_100) == CS_FAIL)
	{
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(*cpp);

		FPRINTF(stderr, "%s: %s", Name, BADVERSION);
		return (CS_FAIL);
	}

        /* Install the Open Server error handler.		*/
        if(srv_props(*cpp, CS_SET, SRV_S_ERRHANDLE,
                (CS_VOID *)server_err_handler, CS_SIZEOF(CS_VOID *),
                (CS_INT *)NULL) == CS_FAIL)
        {
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(*cpp);
		
		FPRINTF(stderr, "%s: %s", Name, BADSERVERRHNDL);
		return (CS_FAIL);
        }

	/* Make up a log file name and configure logfile.		*/
	SPRINTF(filename, "%s.%s", Name, LOG_SUFFIX);
        if(srv_props(*cpp, CS_SET, SRV_S_LOGFILE, 
		(CS_VOID *)filename, CS_NULLTERM, (CS_INT *)NULL) 
		== CS_FAIL)
	{
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(*cpp);

		FPRINTF(stderr, "%s: %s", Name, BADLOGCONFIG);
		return (CS_FAIL);
	} 	

	/* Truncate the log file on start-up.				*/
	boolval = CS_TRUE;
	if (srv_props(*cpp, CS_SET, SRV_S_TRUNCATELOG, 
		(CS_VOID *)&boolval, CS_SIZEOF(boolval), (CS_INT *)NULL) 
		== CS_FAIL)
	{
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(*cpp);

		FPRINTF(stderr, "%s: %s", Name, BADLOGTRUNC);
		return (CS_FAIL);
	}

	/* Enable virtual timer for SRV_C_TIMESLICE callbacks.		*/
	boolval = CS_TRUE;
	if (srv_props(*cpp, CS_SET, SRV_S_VIRTTIMER, 
		(CS_VOID *)&boolval, CS_SIZEOF(CS_BOOL), (CS_INT *)NULL) 
		== CS_FAIL)
	{
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(*cpp);

		FPRINTF(stderr, "%s: %s", Name, BADVIRTCONFIG);
		return (CS_FAIL);
	}

	/* 
	** Set timeslice; it is the number of clock ticks a thread consumes 
	** before the callback gets called. The timeslice in microseconds
	** is the product of the value for SRV_S_TIMESLICE and the value for 
	** SRV_S_VIRTCLKRATE. Since the value for SRV_S_VIRTCLKRATE depends
	** on the hardware, we accept the default value and adjust the time
	** slice by varying the value for SRV_S_TIMESLICE.
	*/
	intval = TIMESLICE;
	if (srv_props(*cpp,  CS_SET, SRV_S_TIMESLICE, 
		(CS_VOID *)&intval, CS_SIZEOF(CS_INT), (CS_INT *)NULL) 
		== CS_FAIL)
	{
                /* Release the context structure already allocated.	*/
                (CS_VOID)cs_ctx_drop(*cpp);

		FPRINTF(stderr, "%s: %s", Name, BADTIMECONFIG);
		return (CS_FAIL);
	}
		
	/* Multthrd is successfully configured.				*/
	return (CS_SUCCEED);
}

/*
**  MULTTHRD_START_HANDLER 
**
**  Purpose:
**
**	Install SRV_CONNECT and SRV_LANGUAGE handlers.
**	Spawn language logger thread.
**	Open files for logging language queries and scheduling information.
**	Register a procedure, 'stop_serv', to enable clients to stop
**	this application.
**
**  Parameters:
**	serverp - A pointer to the Open Server application's control structure.
**
**  Returns:
**	CS_SUCCEED - If start event processing is successful.
**	CS_FAIL	   - If an error occurred in start event processing.
*/
CS_RETCODE	CS_PUBLIC multthrd_start_handler(serverp)
SRV_SERVER	*serverp;
{
	SRV_PROC   *regthreadp;       		/* Thread to use for registering procedure. */
	CS_CHAR	   filename[CS_MAX_CHAR];	/* Scheduling info filename. */

	/* Install SRV_CONNECT/SRV_LANGUAGE event handlers.		*/
	if (srv_handle(serverp, SRV_CONNECT, multthrd_conn_handler)
		== (SRV_EVENTHANDLE_FUNC)NULL)
		return (CS_FAIL);

	if (srv_handle(serverp, SRV_LANGUAGE, multthrd_lang_handler)
		== (SRV_EVENTHANDLE_FUNC)NULL)
		return (CS_FAIL);

	 /* Get a non-client thread to use for procedure registration.	*/
        regthreadp = srv_createproc(serverp);
	if (regthreadp != (SRV_PROC *)NULL)
	{
        	if (srv_regdefine(regthreadp, STOP_SERV, CS_NULLTERM,
                		multthrd_stop_serv) == CS_FAIL)
			return (CS_FAIL);

	        if (srv_regcreate(regthreadp, (CS_INT *)NULL) 
				== CS_FAIL)
			return (CS_FAIL);
 
        	/* Terminate the thread created for procedure registration. */
		(CS_VOID)srv_termproc(regthreadp);
	}
	else
		return (CS_FAIL);

	/* Start the language logger.					*/
	if (multthrd_start_languagelogger() == CS_FAIL)
		return (CS_FAIL);
	
	/* Create a file to hold scheduling information and open it. */
	SPRINTF(filename, "%s.%s", Name, SCHED_SUFFIX);
	if ((Sched_fp = fopen(filename, "w")) == (FILE *)NULL)
		Sched_fp = stdout;

	/* Open a file to log language queries.  	 		*/
	if ((Langlog_fp = fopen(LANGLOG, "w")) == (FILE *)NULL)
		Langlog_fp = stdout;

	/* Make File I/O unbuffered.					*/
	setbuf(Sched_fp, (char *)NULL); 
	setbuf(Langlog_fp, (char *)NULL); 

	/* All done.							*/
	return(CS_SUCCEED);
}


/*
** MULTTHRD_STOP_SERV
**
** Purpose:
**
**      This is a registered procedure that clients can execute to stop
**	multthrd. This registered procedure acknowledges
**	the client request by calling srv_senddone. It then queues SRV_STOP
**	event and closes files opened for logging language queries and 
**	scheduling information.
**
** Parameters:
**	cl_threadp - A pointer to an internal thread control structure.
**
** Returns:
** 	CS_SUCCEED
*/
CS_RETCODE CS_PUBLIC multthrd_stop_serv (cl_threadp)
SRV_PROC        *cl_threadp;
{
	/* Send the final done to acknowledge the regproc request.	*/
	(CS_VOID)srv_senddone(cl_threadp, SRV_DONE_FINAL, 
		CS_TRAN_UNDEFINED, ZERO);

	/* Queue a SRV_STOP event and close the output files.		*/
	if (multthrd_queue_stopevent(cl_threadp) == CS_FAIL)
		return (CS_FAIL);

        return (CS_SUCCEED);
}

/*
** MULTTHRD_CONN_HANDLER
**
** Purpose:
**	Installs callbacks to get scheduling information.
**
** Parameters
**	cl_threadp - A pointer to an internal thread control structure for 
**			the Open Server thread	handling the client connection.
** Returns:
**	CS_SUCCEED - If connect event processing is successful.
**	CS_FAIL	   - If an error occurred in connect event processing.
*/
CS_RETCODE CS_PUBLIC multthrd_conn_handler(cl_threadp)
SRV_PROC	*cl_threadp;
{
        CS_INT		status;		/* srv_senddone status.		*/
	CS_BOOL		ok;		/* Error Control flag.		*/
	CS_CHAR		info[MSGSIZE+1];

	/* Initialization.						*/
        status = SRV_DONE_FINAL;
	ok = CS_TRUE;

	/* 
	** Install callbacks to print scheduling info; If it fails,
	** stop the server.
	*/
	if (multthrd_install_schedinfocb(cl_threadp) == CS_FAIL)
	{
		ok = CS_FALSE;
		status |= SRV_DONE_ERROR;
	}

	/* 
	** Write a message to the schedule information log indicating that
	** this client thread has now come into existence, and is in the
	** SRV_CONNECT handler. This entry will be useful for identifying
	** the span of entries for a thread (between this and the one ouput
	** by SRV_C_EXIT callback).
	*/
	if (ok)
	{
		if (multthrd_get_threadinfo(cl_threadp, info, CS_SIZEOF(info)) 
			== CS_SUCCEED)
		{
			FPRINTF(Sched_fp, "%s - Starts Execution\n", info);
		}
	}

	if (srv_senddone(cl_threadp, status, CS_TRAN_UNDEFINED, ZERO) 
		== CS_FAIL)
	{
		FPRINTF(stderr, "%s: %s", Name, SENDDONEFAIL);
		return (CS_FAIL);
	}
	
	/* If error installing callbacks, stop the server.		*/
	if ((!ok) && (multthrd_queue_stopevent(cl_threadp) == CS_FAIL))
		return (CS_FAIL);

	return (CS_SUCCEED);
}

/*
**  MULTTHRD_LANG_HANDLER 
**	The SRV_LANGUAGE handler.  It retrieves the language request sent by
**	the client, and sends a message to the logger thread. It then sleeps
**	waiting for the logger thread to wake it up. Finally, it sends
**	a DONE to the client.
**	
**  Parameters:
**
**      cl_threadp - A pointer to the internal control structure for the
**			 Open Server thread handling the client connection.
**  Returns:
**	CS_SUCCEED - If language event processing is successful.
**	CS_FAIL	   - If an error occurred in language event processing.
*/
CS_RETCODE CS_PUBLIC multthrd_lang_handler(cl_threadp)
SRV_PROC	*cl_threadp;
{
	CS_BOOL		ok;	/* Error control variable.	*/
	CS_CHAR		*bp;	/* Language buffer.		*/
	CS_INT		size;	/* Size of language request.	*/
	CS_INT		status;	/* srv_senddone status.		*/
	CS_SERVERMSG	errmsg;	/* For sending error messages to the client. */
	CS_CHAR		*msgp;	/* A pointer to message to be sent to the client.	*/
	CS_INT		msgnum;	/* msgnumber for error message. */ 

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	bp = (CS_CHAR *)NULL;		/* No lang buffer yet.		*/
	status = SRV_DONE_FINAL;	/* Assume no errors.		*/
	srv_bzero(&errmsg, sizeof(errmsg));
	msgp = LOG_OK_MSG;
	msgnum = LOG_OK_NUM;
	
	/* Determine the size of the language string.			*/
	if((size = srv_langlen(cl_threadp)) < 0) ok = CS_FALSE;

	/* Allocate space for the language query. Leave room for NULL.	*/
	if(ok && ((bp = (CS_CHAR *)srv_alloc(size+1)) == (CS_CHAR *)NULL))
		ok = CS_FALSE;

	/* Copy the language query into the allocated buffer.		*/
	if(ok && (srv_langcpy(cl_threadp, ZERO, NEG1, bp) < 0)) 
		ok = CS_FALSE;
		
	/* Send a message to the language logger thread.		*/

	if (ok && (multthrd_send_logmsg(cl_threadp, bp) == CS_FAIL))
		ok = CS_FALSE;

	/*
	** If there was an error detected, set the SRV_DONE_ERROR status bit.
	** Also, send a message reporting the error.
	*/
	if(!ok) 
	{
		status |= SRV_DONE_ERROR;
		msgp = LOG_FAIL_MSG;
		msgnum = LOG_FAIL_NUM;
	}

	/*
	** Send a message to the client to indicate what happened.
	*/
	errmsg.msgnumber = msgnum;
	STRCPY(errmsg.text, msgp);
	errmsg.textlen = STRLEN(msgp);
	if (srv_props(Ctxp, CS_GET, SRV_S_SERVERNAME, 
		(CS_VOID *)errmsg.svrname, CS_SIZEOF(errmsg.svrname),
		&errmsg.svrnlen) == CS_FAIL)
	{
		STRCPY(errmsg.svrname, Name);
		errmsg.svrnlen = CS_NULLTERM;
	}

	errmsg.status = (CS_FIRST_CHUNK|CS_LAST_CHUNK);

	if (srv_sendinfo(cl_threadp, &errmsg, CS_TRAN_UNDEFINED) == CS_FAIL)
	{
		FPRINTF(stderr, "%s: %s", Name, SENDINFOFAIL);
	}

	/* Send the final DONE to the client.				*/
	if (srv_senddone(cl_threadp, status, CS_TRAN_UNDEFINED, ZERO)
		== CS_FAIL)
	{
		FPRINTF(stderr, "%s: %s", Name, SENDDONEFAIL);
	}
	
	/* Release allocated memory.					*/
	if(bp != (CS_CHAR *)NULL) (CS_VOID)srv_free(bp);

	return(CS_SUCCEED);
}

/*
** MULTTHRD_START_LANGUAGELOGGER
**
** Parameters:
**	None.
**
** Purpose:
**	Spawn a language logger thread and install callbacks to
**	get scheduling information.
**	
** Returns:
**	CS_SUCCEED	- If logger thread successfully spawned. 
**	CS_FAIL		- If logger thread was not spawned.
*/
CS_RETCODE	multthrd_start_languagelogger()
{
	SRV_PROC	*logger_threadp;
				/*
				**A pointer to logger thread control structure.
				*/
	
	/* 
	** Spawn the language logger.  The entry point for the thread is
	** multthrd_language_logger.
	*/
	if (srv_spawn(&logger_threadp, SRV_C_DEFSTACKSIZE, 
		multthrd_language_logger, (CS_VOID *)NULL, SRV_C_DEFAULTPRI) 
		== CS_FAIL)
	{
		return (CS_FAIL);
	}

	/* 
	** Install callbacks to get scheduling information.
	*/
	if (multthrd_install_schedinfocb(logger_threadp) == CS_FAIL)
	{
		return (CS_FAIL);
	}
	return (CS_SUCCEED);
}

/*
** MULTTHRD_LANGUAGE_LOGGER
**
** Parameters:
**	None.
**
** Purpose
**	Entry function for the language logger thread. There is no return
**	from this  function unless a failure occurs in thread communication,
**	or in the sleep/wake-up mechanisms.
**
** Returns:
**	CS_FAIL	- If a failure occurred.
*/
CS_RETCODE	CS_PUBLIC multthrd_language_logger(dummy)
CS_VOID *dummy;
{
	SRV_OBJID	msgqid;   /* Id for message queue.		*/
	LOG_MSG		*msgp;	  /* A pointer to message exchanged between threads. */ 
	CS_INT		info;     /* For use by srv_createmsq.		*/
	SRV_PROC	*threadp; /* Thread control structure pointer.	*/

	/* Initialization.						*/
	threadp = SRV_CURPROC;	/* Get a pointer to thread control structure.*/

	/*
	** Set user state information for the thread; may be useful
	** for debugging.
	*/
	(CS_VOID)srv_thread_props (threadp, CS_SET, SRV_T_USTATE,
		(CS_VOID *)LOGGER, CS_NULLTERM, (CS_INT *)NULL);

	/* Create a message queue for inter-thread communication.	*/
	if (srv_createmsgq(LOGGER, CS_NULLTERM, &msgqid) == CS_FAIL)
	{
		/* 
		** If  a failure occurred, stop the server by queuing a 
		** SRV_STOP event.
		*/
		(CS_VOID)multthrd_queue_stopevent(threadp);
		return (CS_FAIL);
	}

	/*
	** Loop here forever, reading messages from client threads and
	** logging them.
	*/
	while (CS_TRUE)
	{
		/* 		
		** Get the message from client thread. If a failure occurs
		** break out of the loop and stop the server.
		*/
		if (srv_getmsgq(msgqid, (CS_VOID *)&msgp, 
			SRV_M_WAIT, &info) == CS_FAIL)
		{
			break;
		}

		/* Log the language request.				*/
		FPRINTF(Langlog_fp, "From spid: %d, Query: %s\n",
				msgp->spid, msgp->lang_query);

		/* 
		** Wake up the sender of the message.  If a failure occurs,
		** break out of the loop and stop the server.
		*/
		if (srv_wakeup((CS_VOID *)msgp, SRV_M_WAKE_FIRST, 
			(CS_VOID *)NULL, (CS_VOID *)NULL) == CS_FAIL)
		{
			break;

		}


	}

	/* 
	** If we get here, a serious error occcurred, or the server
	** is being shut down.
	*/
	return (CS_FAIL);
}
		
/*
** MULTTHRD_SEND_LOGMSG
**
** Parameters:
**	cl_threadp - A pointer to an internal thread control structure
**			for the client thread that received the language
**			request.
**
**	lang_query - Language query to be sent to the language logger thread.
**
** Purpose:
**	Send a message to the language logger thread, and sleep until
**	the logger thread does a wake-up.
**
** Returns:
**	CS_SUCCEED 	- If the message was sent successfully.
**	CS_FAIL		- If a failure occurred.
*/
CS_RETCODE	multthrd_send_logmsg(cl_threadp, lang_query)
SRV_PROC	*cl_threadp;
CS_CHAR		*lang_query;
{
	LOG_MSG		msg;	
	SRV_OBJID	logmsgqid;
	CS_INT		ignore;

	/* Get the thread id.						*/
	if (srv_thread_props(cl_threadp, CS_GET, SRV_T_SPID, 
		(CS_VOID *)&(msg.spid), CS_SIZEOF(CS_INT), (CS_INT *)NULL) 
		== CS_FAIL)
	{
		return (CS_FAIL);
	}

	/* Set the query pointer.					*/
	msg.lang_query = lang_query;

	/* Get the object id of the logger thread's message queue.	*/
	if (srv_getobjid(SRV_C_MQUEUE, LOGGER, CS_NULLTERM, 
			&logmsgqid,  &ignore) == CS_FAIL)
	{
		return (CS_FAIL);
	}

	/* Send a message to the language logger thread.		*/
	if (srv_putmsgq(logmsgqid, &msg, SRV_M_NOWAIT)
			== CS_FAIL)
	{
		return (CS_FAIL);
	}

	/* Sleep until the language logger thread wakes up this thread.	*/
	if (srv_sleep((CS_VOID *)&msg, LOGWAIT, ZERO, &ignore,
		(CS_VOID *)NULL, (CS_VOID *)NULL) == CS_FAIL)
	{
		return (CS_FAIL);
	}

	/* 
	** Language logger will wake up this thread after receiving 
	** the message.
	*/
	return (CS_SUCCEED);
}

/*
**
** MULTTHRD_INSTALL_SCHEDINFOCB
**
** Parameters:
**	threadp - A pointer to an internal thread control structure.
**
** Purpose:
**	Installs callback functions to print the thread's
**	 scheduling information.
**
** Returns:
**	CS_SUCCEED	- If all callbacks were installed successfully.
**	CS_FAIL		- If a failure occurred in installing
**			  	a callback.
*/
CS_RETCODE	multthrd_install_schedinfocb(threadp)
SRV_PROC	*threadp;
{
	/* Install a callback to be called when the thread resumes execution.	*/
	if (srv_callback(threadp, SRV_C_RESUME, multthrd_resume_cb)
		== CS_FAIL)
		return (CS_FAIL);

	/* Install a callback to be called when the thread is suspended. 	*/
	if (srv_callback(threadp, SRV_C_SUSPEND, multthrd_suspend_cb)
		== CS_FAIL)
		return (CS_FAIL);

	/* Install a callback to be called when the thread's timeslice expires.	*/ 
	if (srv_callback(threadp, SRV_C_TIMESLICE, multthrd_timeslice_cb)
		== CS_FAIL)
		return (CS_FAIL);

	/* Install a callback to be called when the thread completes execution.	*/
	if (srv_callback(threadp, SRV_C_EXIT, multthrd_exit_cb)
		== CS_FAIL)
		return (CS_FAIL);

	return (CS_SUCCEED);
}

/*
** MULTTHRD_RESUME_CB
**
** Parameters:
**	threadp	- A pointer to an internal thread control structure for
**			the thread.
**
** Purpose:
**	Callback function called when a thread resumes execution.
**
** Returns:
**	CS_SUCCEED/CS_FAIL
*/
CS_RETCODE	CS_PUBLIC multthrd_resume_cb(threadp)
SRV_PROC	*threadp;
{
	CS_CHAR	info[MSGSIZE+1]; /* To get thread information.		*/

	/* Get information on the current thread.			*/
	if (multthrd_get_threadinfo(threadp, info, CS_SIZEOF(info))
		== CS_FAIL)
		return (CS_FAIL);

	/* Log scheduling information.					*/
	FPRINTF(Sched_fp, "%s - Resumes\n", info);
	return (CS_SUCCEED);
}

/*
**  MULTTHRD_SUSPEND_CB
**
** Parameters:
**	threadp - A pointer to an internal thread control structure for
**			the thread.
**
** Purpose:
**	Callback function called when a thread is suspended.
**
** Returns:
**	CS_SUCCEED/CS_FAIL
*/
CS_RETCODE	 CS_PUBLIC multthrd_suspend_cb(threadp)
SRV_PROC	*threadp;
{
	CS_CHAR	info[MSGSIZE+1]; /* To get thread information.		*/

	/* Get information on current thread.				*/
	if (multthrd_get_threadinfo(threadp, info, CS_SIZEOF(info))
		== CS_FAIL)
		return (CS_FAIL);

	/* Log scheduling information.					*/
	FPRINTF(Sched_fp, "%s - Suspended\n", info);
	return (CS_SUCCEED);
}

/*
** MULTTHRD_TIMESLICE_CB
**
** Parameters:
**	threadp - A pointer to an internal thread control structure for
**			the thread.
**
**
** Purpose:
**	Callback function called when timeslice quantum for a thread
**	has expired.
**
** Returns:
**	CS_SUCCEED/CS_FAIL
*/
CS_RETCODE	CS_PUBLIC multthrd_timeslice_cb(threadp)
SRV_PROC	*threadp;
{
	CS_CHAR	info[MSGSIZE+1]; /* to get thread info			*/

	/* Get information on current thread.				*/
	if (multthrd_get_threadinfo(threadp, info, CS_SIZEOF(info))
		== CS_FAIL)
		return (CS_FAIL);

	/* Log scheduling information.					*/
	FPRINTF(Sched_fp, "%s - Time Quantum Expires \n", info);
	return (CS_SUCCEED);

}

/*
** MULTTHRD_EXIT_CB
**
** Parameters:
**	threadp - A pointer to an internal thread control structure for
**			the thread.
**
** Purpose:
**	Callback function called when a thread has returned from the entry 
**	point specified in srv_spawn, or when a thread is associated
**	with a disconnected client.
**
** Returns:
**	CS_SUCCEED/CS_FAIL
*/
CS_RETCODE	CS_PUBLIC multthrd_exit_cb(threadp)
SRV_PROC	*threadp;
{
	CS_CHAR	info[MSGSIZE+1]; /* To get thread information.		*/

	/* Get information on current thread. 				*/
	if (multthrd_get_threadinfo(threadp, info, CS_SIZEOF(info))
		== CS_FAIL)
		return (CS_FAIL);

	/* Log scheduling information.					*/
	FPRINTF(Sched_fp, "%s - Completes Execution \n", info);
	return (CS_SUCCEED);

}

/*
** MULTTHRD_QUEUE_STOPEVENT
**
** Purpose:
**	Queues a SRV_STOP event to stop Open Server. It also closes the files
**	opened for logging scheduling information and language queries.
**
** Arguments:
**	threadp	- A pointer to an internal thread control structure for
**                      the thread that is queuing the SRV_STOP event.
** Returns:
**	Nothing.
*/	
CS_RETCODE CS_PUBLIC multthrd_queue_stopevent(threadp)
SRV_PROC	*threadp;
{
	/* Log a message in the log file.				*/
	(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, STOP_SERVER_MSG, 
		CS_NULLTERM);	

	/* Close the file opened for holding scheduling information.	*/
	if (Sched_fp != (FILE *)NULL)
		FCLOSE(Sched_fp);

	/* Close the file opened for logging language queries.		*/
	if (Langlog_fp != (FILE *)NULL)
		FCLOSE(Langlog_fp);

	/* Delete the message queue created for the language logger thread.*/
	if (srv_deletemsgq(LOGGER, CS_NULLTERM, (SRV_OBJID)0) == CS_FAIL)
		return (CS_FAIL);

	/* Queue a SRV_STOP event.					*/
	if (srv_event(threadp, SRV_STOP, (CS_VOID *)NULL) == CS_FAIL)
		return (CS_FAIL);

	return (CS_SUCCEED);
}

/*
** MULTTHRD_GET_THREADINFO
**
**	Returns a pointer to a string containing thread information, such as
**	thread id, thread_type, and user state.
** 
** Arguments:
**	threadp	- A pointer to an internal thread control structure for
**                the thread.
**	buf	- A buffer to assemble thread information.
**	size	- The size of the buffer.
**
** Side Effects:
**	The string copied to the buffer is null terminated.
**
** Returns:
**	CS_SUCCEED	- If the thread information was successfully obtained. 
**	CS_FAIL		- If an error occurred.
*/
CS_RETCODE	multthrd_get_threadinfo(threadp, buf, size)
SRV_PROC	*threadp;
CS_CHAR		*buf;
CS_INT		size;
{
	CS_INT	spid;
	CS_CHAR	*thread_type;
	CS_CHAR	info[MSGSIZE];
	CS_CHAR	ustate[CS_MAX_CHAR];
	CS_INT	len;

	/* Get the id for the thread.					*/
	if (srv_thread_props(threadp, CS_GET, SRV_T_SPID, (CS_VOID *)&spid,
			CS_SIZEOF(CS_INT), (CS_INT *)NULL) == CS_FAIL)
	{
		return (CS_FAIL);
	}

	/* Get the thread type.						*/
	if ((thread_type = (CS_CHAR *)srv_pfield(threadp, SRV_TYPE, 
		(CS_INT *)NULL)) == (CS_CHAR *)NULL)
		return (CS_FAIL);

	/* Get user state information					*/
	if (srv_thread_props(threadp, CS_GET, SRV_T_USTATE, (CS_VOID *)ustate,
			CS_SIZEOF(ustate), &len) == CS_FAIL)
	{
		return (CS_FAIL);
	}
	ustate[len] = '\0';

	SPRINTF(info, "Thread: %d, Type: %s, Ustate: %s", spid, 
		(thread_type == (CS_CHAR *)NULL) ? "Nil" : thread_type, 
		(len == 0) ? "Nil" : ustate);

	STRNCPY(buf, info, size);
	return (CS_SUCCEED);
}
