/*
**	Sybase Open Server Version 10.0
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
** OSINTRO.C 
**
**	This program takes the minimum steps necessary to initialize and 
**	start an Open Server application.  No user defined event handlers 
**	are installed.  
*/

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

/*
** Include the operating system specific header files required by this
** Open Server application.
*/
#include	<stdio.h>

/*
** Include the required Open Server header files.
**
**	ospublic.h	Public Open Server structures, typedefs, defines, 
**			and function prototypes.
**
**	oserror.h	Open Server error number defines.  This header file
**			is only required if the Open Server application
**			wants to detect specific errors in the Open Server
**			error handler.
*/
#include	<ospublic.h>
#include	<oserror.h>
#include	<ossample.h>

/*
** Local defines.
**
**	OS_ARGCOUNT	Expected number of command line arguments.
*/
#define	OS_ARGCOUNT	2

/*
** This is the main entry point for this sample Open Server application.
** The following arguments are expected on the command line: 
**
**	servername	The name of the Open Server application. 
**			This name must exist in the interfaces file
**			defined by the SYBASE environment variable.
**
** Returns:
**	0		Open Server exited successfully.
**	1		An error was detected during initialization.
*/
int	main(argc, argv)
int	argc;
char	*argv[];
{
	CS_CONTEXT	*cp;		/* Context structure.		*/
	CS_CHAR		*servername;	/* Open Server application name.	*/
	CS_CHAR		logfile[512];	/* Log file name.		*/ 
	CS_BOOL		ok;		/* Error control flag.		*/

	/* Initialization.						*/
	ok = CS_TRUE;

	/*
	** Read the command line options.  There must be one argument
	** specifying the server name.
	*/
	if(argc != OS_ARGCOUNT)
	{
		(CS_VOID)fprintf(stderr, "Invalid number of arguments(%d)\n",
			argc);
		(CS_VOID)fprintf(stderr, "Usage: <program> <server name>\n");
		exit(1);
	}

	/*
	** Initialize `servername' to the command line argument provided.
	*/
	servername = (CS_CHAR *)argv[1];

	/*
	** Allocate a CS-Library context structure to define the default
	** localization information. Open Server will also store global 
	** state information in this structure during initialization.
	*/
	if(cs_ctx_alloc(CS_VERSION_100, &cp) == CS_FAIL)
	{
		(CS_VOID)fprintf(stderr, "%s: cs_ctx_alloc failed", servername);
		exit(1);
	}

	/*
	** Default Open Server localization information can be changed
	** here before calling srv_version using cs_config and cs_locale.
	** A CS-Library error handler should also be installed.  Open
	** Server uses CS-Library internally for data conversion so all
	** applications should install a CS-Library error handler if they
	** are interested in receiving data conversion errors.
	*/

	/*
	** Set the Open Server version and context information.
	*/
	if(srv_version(cp, CS_VERSION_100) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "%s: srv_version failed", servername);
		exit(1);
	}

	/*
	** There is no Open Server error handler installed in this simple 
	** sample Open Server.  Any errors detected by Open Server will be 
	** written to the Open Server log file configured above.  A
	** real Open Server application would want to install an error
	** handler right after calling srv_version so that any subsequent
	** errors would be detected by Open Server application code.
	*/

	/*
	** Default Open Server global properties can be changed here
	** before calling srv_init.  We just change the default log
	** file name to use the name of this Open Server.
	*/

	/*
	** Build a new Open Server log file name using `servername'.
	*/
	(CS_VOID)sprintf(logfile, "%s.log", servername);

	/*
	** Set the new log file name using the global SRV_S_LOGFILE
	** property.
	*/
	if(srv_props(cp, CS_SET, SRV_S_LOGFILE, logfile, CS_NULLTERM,
		(CS_INT *)NULL) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr,
			"%s: srv_props(SRV_S_LOGFILE) failed\n", servername);
		exit(1);
	}

	/*
	** Initialize Open Server.  This causes Open Server to allocate
	** internal control structures based on the global properties set
	** above.  Open Server also looks up the servername in the 
	** interfaces files.
	*/
	if(srv_init((SRV_CONFIG *)NULL, servername, CS_NULLTERM)
		== (SRV_SERVER *)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "%s: srv_init failed\n", servername);
		exit(1);
	}

	/*
	**	This is a way to stop the Open Server
	*/
	if ( srv_handle((SRV_SERVER *)NULL, SRV_START, stop_regproc)
		== CS_FAIL )
	{
		(CS_VOID)cs_ctx_drop(cp);
		(CS_VOID)fprintf(stderr, "%s: srv_handle failed\n",
			servername);
		exit(1);
	}

	/*
	** Start up Open Server.  We don't install any event
	** handlers in this simple example.  This will cause Open Server
	** to use the default event handlers.  This will still allow
	** us to connect with this Open Server using Sybase client programs
	** (e.g. isql).  The call to srv_run will not return until
	** a fatal error is detected by Open Server, or a SRV_STOP
	** event is queued.  Since we haven't installed any event handlers
	** the only way to stop Open Server is to kill the operating
	** system process in which it is running.
	*/
	if(srv_run((SRV_SERVER *)NULL) == CS_FAIL)
	{
		(CS_VOID)fprintf(stderr, "%s: srv_run failed\n", servername);
		ok = CS_FALSE;
	}

	/*
	** Release all allocated control structures and exit.
	*/
	(CS_VOID)cs_ctx_drop(cp);
	exit(ok ? 0 : 1);
}
