
/*
**
**	Sybase Open Server/Registered Procedure Sample
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
**  REGPROC
**	This program demonstrates the use of registered procedures.  The
**	sample client programs, dbwait.c and ctwait.c, are registered
**	to watch for the execution of a registered procedure.  To execute
**	the registered procedure the client programs are waiting on, 
**	compile and run version.c.
**
**	The application writes the information returned to the clients to
**	the srv.log file.
**
**  Usage:
**	regproc [server name] [trace flags]
**
**		If the server name argument is given, it is used.  Otherwise
**		the value of the DSLISTEN environment variable is used.
**	 	The optional trace flags result in diagnostic information 
**		being written to the Open Server log file. 
**
**  Registered Procedures:
**
** 	rp_version			Return the Open Server version number.
**
**	rp_select1			Return a single column with a single
**					row as if a 'select 1' query was 
**					executed.
**
**	rp_shutdown @passwd		Shut down Open Server if the
**					password is correct ('yes').
**
**  Returns:
**	0 on successful execution.
**	1 on error.
*/

/*
** Operating system header files required by Open Server.
*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

/*
** SYBASE header files required by Open Server and Open Client.
*/
#include	<ospublic.h>
#include	<oserror.h>
#include	<ossample.h>

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

/*
** Open Server user-defined message codes.
*/
#define	RPC_FAIL	(CS_INT)SRV_MAXERROR+1	/* rpc failed		*/
#define	RPC_INFO	(CS_INT)SRV_MAXERROR+2	/* rpc failed		*/
#define	SHUTDOWN	(CS_INT)SRV_MAXERROR+3	/* shutdown of server	*/

/*
** Shutdown password.
*/
#define DEFAULT_PASSWD		"yes"

/*
** Prototypes.
*/
CS_RETCODE CS_PUBLIC rp_select1 PROTOTYPE ((
	SRV_PROC	*srvproc
	));
CS_RETCODE CS_PUBLIC rp_shutdown PROTOTYPE ((
	SRV_PROC	*srvproc
	));
CS_RETCODE CS_PUBLIC rp_version PROTOTYPE ((
	SRV_PROC	*srvproc
	));
CS_RETCODE CS_PUBLIC start_handler PROTOTYPE ((
	SRV_PROC	*srvproc
	));

int	main(argc, argv)
int	argc;
char	*argv[];
{
	CS_INT		rv;		 /* Value to return to 	environment. */
	CS_CHAR		sname[CS_MAX_CHAR];
	CS_CONTEXT	*cp;
	CS_CHAR		*servername;    /* Open Server application name.*/
	CS_CHAR 	logfile[CS_MAX_CHAR]; /* Log file name. 	*/


	/* Initialization.						*/
	rv = 0;				/* Assume non-error exit.	*/

	/*
	** Initialize `servername' to the command line argument provided.
	*/
	servername = (CS_CHAR *)argv[1];

	/*
	** Allocate the context structure, and initialize Open Server.	
	*/

	/* Allocate the context structure.				*/
	if (cs_ctx_global(CS_VERSION_100, &cp) != CS_SUCCEED)
	{
		(CS_VOID)fprintf(stderr, "cs_ctx_alloc() failed.  Exiting.\n");
		exit(1);
	}

	/* Install a CS-Library error handler.  			*/
	if (cs_config(cp, CS_SET, CS_MESSAGE_CB, (CS_VOID *)cs_err_handler, 
		CS_UNUSED, NULL) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "cs_config() failed.  Exiting.\n");
		exit(1);
	}

	/*
	** Define the Open Server application behavior and CS_CONTEXT to be used.
	*/
	if (srv_version(cp, CS_VERSION_100) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "srv_version() failed.  Exiting.\n");
		exit(1);
	}

	/* Parse the command line arguments.                             */
	if (proc_args(cp, argc, argv, sname, CS_MAX_CHAR,
		(CS_CHAR *)NULL, (CS_INT)0) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);
		exit(1);
	}

	/*
	** Build a new Open Server log file name using `servername'.
	*/
	(CS_VOID)sprintf(logfile, "%s.log", servername);

        /*
        ** Set the new log file name using the global SRV_S_LOGFILE
        ** property.
        */
        if(srv_props(cp, CS_SET, SRV_S_LOGFILE, logfile, CS_NULLTERM,
                (CS_INT *)NULL) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr,
			"%s: srv_props(SRV_S_LOGFILE) failed\n", servername);
		exit(1);
	}

	/* Install the Open Server error handler.			*/
	if (srv_props(cp, CS_SET, SRV_S_ERRHANDLE, 
		(CS_VOID *)server_err_handler, CS_SIZEOF(CS_VOID *), 
		(CS_INT *)NULL) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"srv_props(ERRHANDLE) failed.  Exiting.\n");
		exit(1);
	}

	/* Initialize Open Server.					*/
	if (srv_init((SRV_CONFIG *)NULL, sname, CS_NULLTERM) 
		== (SRV_SERVER *)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "srv_init failed.  Exiting.\n");
		exit(1);
	}

	/*
	** Call start_handler to initialize the remote server structure.
	** Other event handlers are defined in start_handler.
	*/
	if (srv_handle((SRV_SERVER *)NULL, SRV_START, start_handler) ==  
		(SRV_EVENTHANDLE_FUNC)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "srv_handle failed.  Exiting.\n");
		exit(1);
	}

	/*
	** Start up Open Server. et the Open Server version and
	** null terminate it.  Write the version string to stderr
	** and to the log file. 
	*/
	print_version(cp, sname);

	/*
	** srv_run returns CS_SUCCEED when stopped normally via a SRV_STOP
	** event.
	*/
	if (srv_run((SRV_SERVER *)NULL) != CS_SUCCEED) 
		rv = 1;

	/* Release allocated control structures.			*/
	(CS_VOID)cs_ctx_drop(cp);
	exit(rv);
}

/*
**  START_HANDLER - Start handler.
**	Processes SRV_START events.  All registered procedures are
**	installed here.
**	
**  Parameters:
**	server - A pointer to the Open Server application's control structure.
**
**  Returns:
**	CS_SUCCEED | CS_FAIL
*/
CS_RETCODE	CS_PUBLIC start_handler(server)
SRV_SERVER	*server;
{
	CS_BOOL		ok;
	SRV_PROC        *srvproc;

	/* Initialization.						*/
	ok = CS_TRUE;

	/*
	**  Create a temporary service process to use for registering the
	**  procedures.
	*/
	if ((srvproc = srv_createproc(server)) == (SRV_PROC *)NULL)
		return (CS_FAIL);

	/*
	** Register the "Select 1" procedure. 
	*/
	if (srv_regdefine(srvproc, "rp_select1", CS_NULLTERM, 
		(SRV_EVENTHANDLE_FUNC)rp_select1) != CS_SUCCEED)
		ok = CS_FALSE;
	if (ok && srv_regcreate(srvproc, (CS_INT *)NULL) != CS_SUCCEED)
		ok = CS_FALSE;


	/*
	**  Register the version procedure.
	*/
	if (ok && srv_regdefine(srvproc, "rp_version", CS_NULLTERM, 
		(SRV_EVENTHANDLE_FUNC)rp_version) != CS_SUCCEED)
		ok = CS_FALSE;
	if (ok && srv_regcreate(srvproc, (CS_INT *)NULL) != CS_SUCCEED) 
		ok = CS_FALSE;
						 

	/*
	**  Register the "shutdown gateway" procedure.
	*/
	if (ok && srv_regdefine(srvproc, "rp_shutdown", CS_NULLTERM, 
		(SRV_EVENTHANDLE_FUNC)rp_shutdown) != CS_SUCCEED)
		ok = CS_FALSE;
	if (ok && srv_regparam(srvproc, "@passwd", CS_NULLTERM, CS_CHAR_TYPE,
		SRV_NODEFAULT, (CS_BYTE *)NULL) != CS_SUCCEED)
		ok = CS_FALSE;
	if (ok && srv_regcreate(srvproc, (CS_INT *)NULL) != CS_SUCCEED) 
		ok = CS_FALSE;

	/*
	** We don't need the service thread anymore, so let's
	** drop it.
	*/
	srv_dropproc(srvproc);

	return (ok ? CS_SUCCEED : CS_FAIL);
}

/*
**  RP_VERSION
**      We got an RPC asking us what program we are and what version of
**      our program and of Open Server we're using.
**
**  Parameters:
**      srvproc - A pointer to the thread that got the RPC.
**
**  Returns:
**      CS_SUCCEED
*/
CS_RETCODE     	CS_PUBLIC rp_version(srvproc)
SRV_PROC        *srvproc;
{
	CS_INT		status;
	CS_CONTEXT	*cp;
	CS_SERVERMSG	msg;

	/* Initialization.						*/
	status = SRV_DONE_FINAL;
	srv_bzero(&msg, CS_SIZEOF(CS_SERVERMSG));

	/* Allocate the context structure.				*/
	if (cs_ctx_global(CS_VERSION_100, &cp) != CS_SUCCEED)
	{ 
		/* Without a context structure, we cannot proceed.	*/
		status |= SRV_DONE_ERROR;

		if (srv_senddone(srvproc, status, CS_TRAN_COMPLETED, 0) 
			!= CS_SUCCEED)
			(CS_VOID)fprintf(stderr, "srv_senddone() failed.\n");

		return (CS_SUCCEED);
	}

	/*
	** We'll send the version of Open Server that is currently
	** running back as a message.
	*/
	if (srv_props(cp, CS_GET, SRV_S_VERSION, msg.text, 
		CS_SIZEOF(msg.text), &msg.textlen) != CS_SUCCEED)
	{
		/* 
		** An error occurred trying to return the Open Server version. 	
		*/
		status |= SRV_DONE_ERROR;
	}
	else
	{
		/* 
		** Send the version to the client as a message.
		** The message text field of the message structure was
		** filled in above.
		*/
		msg.msgnumber = RPC_INFO;

		/* Fill in the server name field.			*/
		(CS_VOID)srv_props(cp, CS_GET, SRV_S_SERVERNAME,
			(CS_VOID *)msg.svrname, CS_SIZEOF(msg.svrname),
			&msg.svrnlen);

		/* Send the message to the client.			*/
		msg.status = (CS_FIRST_CHUNK|CS_LAST_CHUNK);
		if (srv_sendinfo(srvproc, &msg, (CS_INT)0) != CS_SUCCEED)
		{
			status |= SRV_DONE_ERROR;
			(CS_VOID)fprintf(stderr, "srv_sendinfo() failed.\n");
		}

		/* Set the status for the RPC as 1, indicating success.	*/
		if (srv_sendstatus(srvproc,(CS_INT) 1) != CS_SUCCEED)
		{
			status |= SRV_DONE_ERROR;
			(CS_VOID)fprintf(stderr, "srv_sendstatus() failed.\n");
		}
	}

	/* Send the final DONE packet to the client.			*/
	if (srv_senddone(srvproc, status, CS_TRAN_COMPLETED, 0) != CS_SUCCEED)
		(CS_VOID)fprintf(stderr, "srv_senddone() failed.\n");

	return (CS_SUCCEED);
}

/*
**  RP_SHUTDOWN
**	Registered procedure to shut down the Open Server application.
**
**  Parameters:
**	srvproc - A pointer to an internal thread control structure.
**
**  Returns:
**      CS_SUCCEED
*/

CS_RETCODE	CS_PUBLIC rp_shutdown(srvproc)
SRV_PROC	*srvproc;
{
	CS_BOOL		ok;
	CS_INT		status;
	CS_INT		len;
	CS_INT		numparams;
	CS_SMALLINT	outlen;
	CS_SERVERMSG	msg;
	CS_DATAFMT	datafmt;
	CS_CHAR		buf[CS_MAX_CHAR];
	CS_CONTEXT	*cp;

	/* Initialization.						*/
	ok = CS_TRUE;
	status = SRV_DONE_FINAL;
	srv_bzero(&datafmt, CS_SIZEOF(CS_DATAFMT));

	/* Verify the password to shut down. 				*/
	datafmt.maxlength = CS_MAX_CHAR;
	len = datafmt.maxlength;
	datafmt.datatype = CS_CHAR_TYPE;

	/* Check for parameters. 					*/
	if (srv_numparams(srvproc, &numparams) != CS_SUCCEED) 
		ok = CS_FALSE;

	/* Make sure there is one and only one parameter.		*/
	if (numparams != 1)
		ok = CS_FALSE;

	/* Get the password from the password parameter. */
	if (ok && srv_bind(srvproc, CS_GET, SRV_RPCDATA, 1, &datafmt, 
		(CS_BYTE *)buf, &len, &outlen) != CS_SUCCEED)
	{
		/* Error has already been raised.			*/
		ok = CS_FALSE;
	}

	/* Read parameters into allocated program variables.             */
	if (ok && (srv_xferdata(srvproc, CS_GET, SRV_RPCDATA) != CS_SUCCEED))
		ok = CS_FALSE;

	/* 
	** Was this a "valid" password for shutting down the server?	
	**
	** NOTE: The following check will only work for ASCII/ISO_1 clients.
	*/
	if (!ok || memcmp(buf, DEFAULT_PASSWD, strlen(DEFAULT_PASSWD)) != 0)
	{
		ok = CS_FALSE;

		/*
		** Initialize the message to send to the client indicating
		** that an invalid password was sent.
		*/
		srv_bzero(&msg, CS_SIZEOF(CS_SERVERMSG));
		msg.msgnumber = RPC_FAIL;
		msg.severity = 10;

		/* Fill in the message text field.                      */
		(CS_VOID)sprintf(msg.text, "rp_shutdown: Invalid password.\n");
		msg.textlen = strlen(msg.text);

		/* Allocate the context structure.			*/
		if (cs_ctx_global(CS_VERSION_100, &cp) != CS_SUCCEED)
		{ 
			(CS_VOID)fprintf(stderr, "cs_ctx_alloc() failed.\n");
		}
		else
		{
			/* Fill in the server name field.		*/
			(CS_VOID)srv_props(cp, CS_GET, SRV_S_SERVERNAME,
				(CS_VOID *)msg.svrname, CS_SIZEOF(msg.svrname),
				&msg.svrnlen);
		}

		/* Notify the client of the command failure.             */
		msg.status = (CS_FIRST_CHUNK|CS_LAST_CHUNK);
		if (srv_sendinfo(srvproc, &msg, (CS_INT)0) != CS_SUCCEED)
			(CS_VOID)fprintf(stderr, "srv_sendinfo() failed.\n");

		status |= SRV_DONE_ERROR;

		/* Set the status for the RPC as -1 indicating failure.	*/
		if (srv_sendstatus(srvproc,(CS_INT) -1) != CS_SUCCEED)
		{
			(CS_VOID)fprintf(stderr, "srv_sendstatus() failed.\n");
		}
	}

	/* Inform the client of the shutdown.				*/
	if (ok)
	{
		/* Set the status for the RPC as 1 indicating success. 	*/
		if (srv_sendstatus(srvproc,(CS_INT) 1) != CS_SUCCEED)
		{
			status |= SRV_DONE_ERROR;
			(CS_VOID)fprintf(stderr, "srv_sendstatus() failed.\n");
		}

		/* Stop the application.					*/
		if (srv_event(srvproc, SRV_STOP, (CS_VOID *) NULL) != SRV_STOP)
		{
			status |= SRV_DONE_ERROR;
			(CS_VOID)fprintf(stderr, 
				"\nError shutting down server.\n");
		}
	}

	/* Send the final done packet to the client.			*/
	if (srv_senddone(srvproc, status, CS_TRAN_COMPLETED, 0) != CS_SUCCEED)
		(CS_VOID)fprintf(stderr, "srv_senddone() failed.\n");

	return (CS_SUCCEED);
}

/*
**  RP_SELECT1
**      Perform a a select x = 1 in response to a user RPC request.
**
**  Parameters:
**      srvproc - A pointer to the thread that got the RPC.
**
**  Returns:
**      CS_SUCCEED
*/
CS_RETCODE     	CS_PUBLIC rp_select1(srvproc)
SRV_PROC        *srvproc;
{
	CS_BOOL		ok;
	CS_INT		status;
	CS_INT		len;
	CS_INT		answer;
	CS_DATAFMT	datafmt;

	/* Initialization.						*/
	ok = CS_TRUE;
	status = SRV_DONE_FINAL;
	answer = 1;
	srv_bzero(&datafmt, CS_SIZEOF(CS_DATAFMT));

	/* Prepare the description of the return data.			*/
	datafmt.name[0] = 'x';
	datafmt.namelen = 1;
	datafmt.format = CS_FMT_UNUSED;
	datafmt.datatype = CS_INT_TYPE;
	datafmt.status = CS_CANBENULL;
	datafmt.maxlength = CS_SIZEOF(answer);
	len = datafmt.maxlength;
	 
	/* Set the value to be returned.					*/
	if (srv_descfmt(srvproc, CS_SET, SRV_ROWDATA, 1, &datafmt)
		!= CS_SUCCEED)
	{
		ok = CS_FALSE;
	}
	if (ok && srv_bind(srvproc, CS_SET, SRV_ROWDATA, 1, &datafmt, 
		(CS_BYTE *)&answer, &len, (CS_SMALLINT *)NULL) != CS_SUCCEED)
	{
		ok = CS_FALSE;
	}

	if (ok && srv_xferdata(srvproc, CS_SET, SRV_ROWDATA) != CS_SUCCEED)
		ok = CS_FALSE;


	if (!ok)
		status |= SRV_DONE_ERROR;

	/* Send the final DONE packet to the client.			*/
	if (srv_senddone(srvproc, status, CS_TRAN_COMPLETED, 1) != CS_SUCCEED)
		(CS_VOID)fprintf(stderr, "srv_senddone() failed.\n");

	return (CS_SUCCEED);
}
