/*
**	Sybase Open Server Version 10.0
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
** SIGALARM.C 
**
**	This sample application demonstrates the use of signal alarms in 
**	an Open Server application.  This program starts a service thread
**	that goes to sleep.  A SIGALRM handler is installed which issues
**	a wakeup on that service thread.  When the service thread wakes up,
**	it writes a message to the log file.
*/

/*
** Include the Operating-system-specific header files required by this
** Open Server application.
*/
#include	<stdio.h>
#include	<sys/signal.h>

/*
** Include the required Open Server header files.
**
**	ospublic.h	Public Open Server sturctures, typedefs, defines, and
**			function prototypes.
**	ossample.h	Utility routines for the sample program.
*/
#include	<ospublic.h>
#include	<ossample.h>

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

/*
** Prototypes.
*/
extern int main PROTOTYPE((
	int argc,
	char *argv[]
	));
extern CS_RETCODE alrm_start_handler PROTOTYPE((
	CS_VOID *ssp
	));
extern CS_VOID alrm_alarm_handler PROTOTYPE((
	int signo,
	int code,
	struct sigcontext *scp,
	char *addr
	));
extern CS_RETCODE alrm_service_thread PROTOTYPE((
	CS_VOID **spp
	));
extern unsigned int alarm PROTOTYPE((
	unsigned int seconds
	));

/*
** Local defines.
**
**	ALARMFREQ	The alarm frequency in seconds.
**	WAKEUPMSG	Alarm handler wakeup message.
**	SLEEPMSG	Service thread sleep message.
**	WAKEMSG		Service thread wakeup message.
**	SLEEPLABEL	Sleep label for service thread.
**	TERMINATOR	Termination line between alarm sleep/wakeup pairs.
*/
#define	ALARMFREQ	1
#define	WAKEUPMSG	"Waking up service thread\n"
#define	SLEEPMSG	"Service thread sleeping\n"
#define	WAKEMSG		"Service thread wokeup\n"
#define	SLEEPLABEL	"Alarm Service Thread"
#define	TERMINATOR	"--------------------------------------------\n"

/*
** Global data.
**
**	ServiceThread	This is the internal thread control structure for
**			the service thread.  This global is used as the
**			sleep event for the service thread.
**	ServerName	Name of this Open Server.
*/
SRV_PROC	*ServiceThread = (SRV_PROC *)NULL;
CS_CHAR		ServerName[1024];

/*
** This is the main entry point for this sample Open Server application.
** The only argument expected on the command line for this sample is
** the server name.  If this argument is not provided a usage message
** is printed and Open Server exits.
**
** Returns:
**	0		Open Server exited successfully.
**	1		An error was detected during initialization.
*/
int	main(argc, argv)
int	argc;
char	*argv[];
{
	CS_CONTEXT	*cp;		/* Context structure.		*/
	SRV_SERVER	*ssp;		/* Server control structure.	*/
	CS_BOOL		ok;		/* Error control flag.		*/

	/* Initialization.						*/
	ok = CS_TRUE;

	/*
	** Allocate a CS-Library context structure to define the default
	** localization information  Open Server will also store global
	** state information in this structure.
	*/
	if(cs_ctx_alloc(CS_VERSION_100, &cp) != CS_SUCCEED)
	{
		(CS_VOID)fprintf(stderr, "cs_ctx_alloc failed");
		exit(1);
	}

	/*
	** Install a CS-Library error handler.  All Open Server applications
	** that want to receive conversion errors must install a CS-Library
	** error handler.
	*/
	if(cs_config(cp, CS_SET, CS_MESSAGE_CB, (CS_VOID *)cs_err_handler,
		CS_UNUSED, (CS_INT *)NULL) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "cs_config(CS_MESSAGE_CB) failed");
		exit(1);
	}

	/*
	** Set the Open Server version and context information.
	*/
	if(srv_version(cp, CS_VERSION_100) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "%s: srv_version failed", ServerName);
		exit(1);
	}

	/*
	** Read the command line arguments.
	*/
	if(proc_args(cp, argc, argv, ServerName, CS_SIZEOF(ServerName),
		(CS_CHAR *)NULL, (CS_INT)0) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);
		exit(1);
	}

	/*
	** Install the Open Server error handler.
	*/
	if(srv_props(cp, CS_SET, SRV_S_ERRHANDLE,
		(CS_VOID *)server_err_handler, CS_SIZEOF(CS_VOID *),
		(CS_INT *)NULL) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);
		exit(1);
	}

	/*
	** Initialize Open Server. This causes Open Server to allocate
	** internal control structures based on the default server
	** properties. The Open Server name is also looked up in
	** the interfaces file.
	*/
	if((ssp = srv_init((SRV_CONFIG *)NULL, ServerName, CS_NULLTERM))
		== (SRV_SERVER *)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "%s: srv_init failed\n", ServerName);
		exit(1);
	}

	/*
	** Define a SRV_START handler to complete the initialization
	** of the alarm handler, alarm cycle, and service thread.
	*/
	if(srv_handle(ssp, SRV_START, alrm_start_handler)
		== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "%s: srv_handle failed\n", ServerName);
		exit(1);
	}

	/*
	** Print a version string to the screen and to the log file.
	*/
	print_version(cp, ServerName);

	/*
	** Start up the Open Server application.  The call to srv_run will
	** not return until the Open Server application detects a fatal error
	** or a SRV_STOP event is queued.  In this simple example, no event
	** handlers are installed.  Open Server will use its default event
	** handlers.  Because no event handlers are installed, the only way
	** to stop this Open Server application is to kill the operating 
	** system process in which it is running.
	*/
	if(srv_run((SRV_SERVER *)NULL) == CS_FAIL)
	{
		(CS_VOID)fprintf(stderr, "%s: srv_run failed\n", ServerName);
		ok = CS_FALSE;
	}

	/*
	** Disable the SIGALRM.
	*/
	(CS_VOID)alarm(0);

	/*
	** Release the context structure.
	*/
	(CS_VOID)cs_ctx_drop(cp);
	exit(ok ? 0 : 1);
}

/*
** ALRM_START_HANDLER
**
**	This is the start handler.  The start handler creates a service
**	thread that sleeps, waiting for SIGALRMs.  The handler also installs
**	a SIGALRM handler and requests that SIGALRM signals be generated.
**
**	If an error is detected during the start handler, CS_FAIL is returned
**	so that srv_run will return CS_FAIL to the main line code without
**	starting up the Open Server application.
**
** Parameters:
**	ssp	A pointer to the Open Server control structure.
**
** Returns:
**	CS_SUCCEED	Initialization succeeded.
**	CS_FAIL		An error was detected during initialization.
*/
CS_RETCODE	alrm_start_handler(ssp)
SRV_SERVER	*ssp;
{
	/*
	**	Register the procedure to stop the Open Server.
	*/
	if ( stop_regproc(ssp) == CS_FAIL )
	{
		return(CS_FAIL);
	}

	/*
	** Spawn a service thread that sleeps waiting for a SIGALRM
	** to wake it up.
	*/
	if(srv_spawn(&ServiceThread, SRV_DEFAULT_STACKSIZE,
		(SRV_SERVICETHREAD_FUNC)alrm_service_thread, &ServiceThread,
		SRV_DEFPRIORITY) == CS_FAIL)
	{
		(CS_VOID)fprintf(stderr, "%s: srv_spawn failed\n", ServerName);
		return(CS_FAIL);
	}

	/*
	** Install a SIGALRM handler.
	*/
	(CS_VOID)srv_signal(SIGALRM, (SRV_SIGNAL_FUNC)alrm_alarm_handler);

	/*
	** Schedule a SIGALRM signal to be generated at the frequency
	** indicated by ALARAMFREQ.
	*/
	(CS_VOID)alarm(ALARMFREQ);
	return(CS_SUCCEED);
}

/*
** ALRM_ALARM_HANDLER
**
**	This is the SIGALRM handler.  It is called by the operating
**	system each time a SIGALRM is generated.  It issues a wakeup
**	call to the sleeping service thread.
**
** Parameters:
**	signo	Signal number.
**	code	Signal parameter.
**	scp	A pointer to a signal context structure.
**	addr	A pointer to additional address information.
**
** Returns:
**	Nothing.
*/
CS_VOID			alrm_alarm_handler(signo, code, scp, addr)
int			signo;
int			code;
struct sigcontext	*scp;
char			*addr;
{
	/*
	** Log a message indicating that we are waking up the service
	** thread.
	*/
	(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, WAKEUPMSG, CS_NULLTERM);

	/*
	** Wake up the sleeping service thread.  We ignore the
	** return code from srv_wakeup because the service thread
	** may not be sleeping when the alarm handler is called.
	** srv_wakeup returns CS_FAIL if no one is sleeping on the
	** specified sleep event.
	*/
	(CS_VOID)srv_wakeup(ServiceThread, SRV_M_WAKE_INTR|SRV_M_WAKE_FIRST,
		(CS_INT)0, (CS_INT)0);

	(CS_VOID)alarm(ALARMFREQ);
	return;
}

/*
** ALRM_SERVICE_THREAD
**
**	This is the service thread.  This thread logs a message saying
**	it is going to sleep.  It then sleeps, waiting to be woken
**	by the alarm handler.  It then logs another message indicating
**	that is has woken up.
**
** Parameters:
**	spp	A pointer to an internal thread control structure.
**
** Returns:
**	This event handler only returns if an error is detected.  In that
**	case, it return CS_FAIL.
*/
CS_RETCODE	alrm_service_thread(spp)
SRV_PROC	**spp;
{
	/*
	** This service thread exits only if an error is detected.
	*/
	for(;;)
	{
		/*
		** Log a message indicating that we are going to sleep.
		*/
		(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, SLEEPMSG,
			CS_NULLTERM);

		/*
		** Go to sleep.
		*/
		if(srv_sleep(*spp, SLEEPLABEL, (CS_INT)0, (CS_INT *)NULL,
			(CS_INT)0, (CS_INT)0) == CS_FAIL)
		{
			(CS_VOID)fprintf(stderr, "%s: srv_sleep failed\n",
				ServerName);

			/*
			** Raise a SRV_STOP event so srv_run will return
			** to the main line code.
			*/
			if(srv_event(spp, SRV_STOP, (CS_VOID *)NULL) == CS_FAIL)
			{
				(CS_VOID)fprintf(stderr,
					"%s: Cannot raise SRV_STOP\n",
					ServerName);
				exit(1);
			}
			return(CS_FAIL);
		}

		/*
		** Log a message indicating that the thread woke up.
		*/
		(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, WAKEMSG,
			CS_NULLTERM);
		(CS_VOID)srv_log((SRV_SERVER *)NULL, CS_TRUE, TERMINATOR,
			CS_NULLTERM);
	}
}
