/*
**	Sybase Registered Procedure Sample Application
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
**  VERSION
**	This program is used to demonstrate the use of registered procedures.
**	This program executes the registered procedure 'rp_version' against
**	the Open Server application.  The waiting programs (dbwait.c and
**	ctwait.c) will be notified of the event.
**
**  Usage:
**	version
**	
**		Version connects with the open server using the DSQUERY 
**		environment value.
**
**  Returns:
**      0 on successful execution
**      1 on error
*/

/*
** Operating system header files required by this application
*/
#include	<stdio.h>
#include	<string.h>
#include	<ctype.h>

/*
** SYBASE header files required by Open Client.
*/
#include	<ctpublic.h>
#include	<ctsample.h>

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

#define USERNAME	"sa"
#define PASSWORD	""
#define APPLICATION	"version"


int	main(argc, argv)
int	argc;
char	*argv[];
{
	CS_INT		ok;
 	CS_INT		res_type;
	CS_CONNECTION	*conn;
	CS_COMMAND	*cmd;
	CS_CONTEXT	*cp;

	/* Initialization						*/
	ok = CS_TRUE;
	cmd = (CS_COMMAND *)NULL;
	conn = (CS_CONNECTION *)NULL;

	/* Allocate memory for the context structure.			*/
	if (cs_ctx_alloc(CS_VERSION_100, &cp) != CS_SUCCEED)
	{
		(CS_VOID)fprintf(stderr, 
			"version: Context allocate failed. Exiting.\n");
		exit(1);
	}

	/* Install a CS-Library error handler.                          */
	if (cs_config(cp, CS_SET, CS_MESSAGE_CB, (CS_VOID *)cs_err_handler,
		CS_UNUSED, NULL) != CS_SUCCEED)
	{
		/* Release the context structure			*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"version: Can't init CSLIB handlers. Exiting\n");
		exit(1);
	}

	/* Initialize Open Client version.				*/
	if (ct_init(cp, CS_VERSION_100) != CS_SUCCEED)
	{
		/* Release the context structure.			*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"version: Can't init CTLIB version. Exiting\n");
		exit(1);
	}

	/*
	** Install message and error handlers.
	*/
	if (ct_callback(cp, (CS_CONNECTION *)NULL, CS_SET, CS_CLIENTMSG_CB, 
		(CS_VOID *)client_err_handler) != CS_SUCCEED)
	{
		/* Release the context structure.			*/
		(CS_VOID)ct_exit(cp, CS_UNUSED);
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"version: Can't init msg handlers. Exiting\n");
		exit(1);
	}
	if (ct_callback(cp, (CS_CONNECTION *)NULL, CS_SET, CS_SERVERMSG_CB, 
		(CS_VOID *)server_err_handler) != CS_SUCCEED)
	{
		/* Release the context structure.			*/
		(CS_VOID)ct_exit(cp, CS_UNUSED);
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"version: Can't init err handlers. Exiting\n");
		exit(1);
	}

	/* Allocate a connection handle for the connection attempt.	*/
	if (ct_con_alloc(cp, &conn) != CS_SUCCEED)
		ok = CS_FALSE;

	/*
	** Set up the login information for the remote DBMS.
	** Also save control structures for access from other routines.
	*/

	if (ok && ct_con_props(conn, CS_SET, CS_USERNAME, USERNAME, 
		CS_NULLTERM, (CS_INT *)NULL) != CS_SUCCEED)
		ok = CS_FALSE;

	if (ok && ct_con_props(conn, CS_SET, CS_PASSWORD, PASSWORD, 
		CS_NULLTERM, (CS_INT *)NULL) != CS_SUCCEED)
		ok = CS_FALSE;

	if (ok && ct_con_props(conn, CS_SET, CS_APPNAME, APPLICATION, 
		CS_NULLTERM, (CS_INT *)NULL) != CS_SUCCEED)
		ok = CS_FALSE;

	/* Attempt a connection to the remote SQL server.		*/
	if (ok && ct_connect(conn, NULL, 0) != CS_SUCCEED)
		ok = CS_FALSE;

	/* Allocate a command buffer.					*/
	if (ok && ct_cmd_alloc(conn, &cmd) != CS_SUCCEED)
		ok = CS_FALSE;

	/* 
	** Execute the 'rp_version' stored procedure. 
	*/
	if (ok && ct_command(cmd, CS_RPC_CMD, "rp_version", CS_NULLTERM, 
		CS_UNUSED) != CS_SUCCEED)
		ok = CS_FALSE;

	/* Send the RPC command.						*/
	if (ok && ct_send(cmd) != CS_SUCCEED)
		ok = CS_FALSE;

	/* Process the results from the RPC execution.			*/
	while (ok && ct_results(cmd, &res_type) == CS_SUCCEED)
	{
		/* Make sure the appropriate result was received.	*/
		switch((int)res_type)
		{
		case CS_STATUS_RESULT:
			/* Ignore the status coming back.		*/
			while (ct_fetch(cmd, CS_UNUSED, CS_UNUSED, 
				CS_UNUSED, (CS_INT *)NULL) == CS_SUCCEED)
			{
				(CS_VOID)fprintf(stderr, 
					"RPC Status fetch loop\n");
			}
			break;
		case CS_CMD_SUCCEED:
			(CS_VOID)fprintf(stderr, "RPC Command Succeeded\n");
			break;
		case CS_CMD_DONE:
			(CS_VOID)fprintf(stderr, "RPC Command Done\n");
			break;
		default:
			(CS_VOID)fprintf(stderr, 
				"RPC returned unexpected result\n");
			(CS_VOID)ct_cancel(NULL, cmd, CS_CANCEL_ALL);
			ok = CS_FALSE;
		}
	}

	/* Free the command buffer.					*/
	if (cmd != (CS_COMMAND *)NULL)
		(CS_VOID)ct_cmd_drop(cmd);

	/* If the connection with server is active, close it now.	*/
	if (conn != (CS_CONNECTION *)NULL)
	{
		(CS_VOID)ct_close(conn, CS_UNUSED);
		(CS_VOID)ct_con_drop(conn);
	}

	if (ok)
		(CS_VOID)fprintf(stderr, 
			"\nversion: RPC was successfully executed!\n");
	else
		(CS_VOID)fprintf(stderr, "\nversion: RPC failed!\n");

	/* Release allocated control structures.				*/
	(CS_VOID)ct_exit(cp, CS_UNUSED);
	(CS_VOID)cs_ctx_drop(cp);

	exit(0);
}
