
/*
**
**	Sybase Open Server/Open Client Gateway Sample
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
**  CTOSDEMO.C
**
**	This program accepts commands from a client and passes them
**	to a remote SQL Server.  The results from the remote SQL 
**	Server are then returned to the client.  It is a complete
**	"virtual" SQL server.  Any command that can be sent to a SQL
**	server can be sent to this Open Server.
**	
**  Usage:
**	ctosdemo 
**		-s		Remote server to connect to.
**		[trace flags]	Any valid trace flags.
**	
**	ctosdemo sets up a listener using the value in the DSLISTEN
**	environmental variable.
**
**  Returns:
**	GW_STDEXIT	Normal exit.
**	GW_ERREXIT	Error exit.
**
** History:
** Chg#	Date	Description						Resp
** ----	-------	-------------------------------------------------------	----
**	19Mar93	System 10 release					djs
*/

/*
** Include demo header file.
*/
#include	"ctosdemo.h"

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

/*
** Global data declarations.
**
** The Open Server gateway application uses the following global data
** (besides the DBMSCTL_T which is in the SRV_PROC structure):
**
**	Name		Open Server name.
**	Context		Open Server context.
**	SqlServername	Name of remote SQL Server to connect to.
*/
CS_CHAR		Name[1024];
CS_CHAR		SqlServername[1024];
CS_CONTEXT	*Context = NIL(CS_CONTEXT *);

CS_INT		Bulkrows = (CS_INT)0;
CS_BOOL		NoMoreCurrows = CS_FALSE;

int	main(argc, argv)
int	argc;
char	*argv[];
{
	CS_INT		rv;		/* value to return to environment */

	/* Initialization.						*/
	rv = GW_STDEXIT;		/* Assume normal exit.		*/

	/*
	** Set version information for Open Server.
	*/
	if(gw_setversion(&Context) == CS_FAIL)
	{
		exit(GW_ERREXIT);
	}

	/*
	** Read the command line arguments.
	*/
	if(proc_args(Context, argc, argv, Name, CS_SIZEOF(Name), 
		SqlServername, CS_SIZEOF(SqlServername)) == CS_FAIL)
	{
		exit(GW_ERREXIT);
	}

	/*
	** Configure Open Server and initialize CT-Library.
	*/
	if(gw_cfgsrv(Context, Name) == CS_FAIL)
	{
		(CS_VOID)cs_ctx_drop(Context);
		Context = NIL(CS_CONTEXT *);
		exit(GW_ERREXIT);
	}

	/*
	** Install a start handler.  All other event handlers are initialized
	** in the start handler.
	*/
	if(srv_handle(NIL(SRV_SERVER *), SRV_START, gw_starthandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
		(CS_VOID)cs_ctx_drop(Context);
		Context = NIL(CS_CONTEXT *);
		exit(GW_ERREXIT);
	}

	/*
	** Display and log the Open Server version string.
	*/
	print_version(Context, Name);

	/*
	** Start the Open Server.  srv_run doesn't return until the
	** Open Server is stopped.
	*/
	if(srv_run(NIL(SRV_SERVER *)) == CS_FAIL)
	{
		rv = GW_ERREXIT;
	}

	/*
	** Drop the context handle and shutdown CT-Library before exiting.
	*/
	(CS_VOID)ct_exit(Context, CS_FORCE_EXIT);
	(CS_VOID)cs_ctx_drop(Context);
	Context = NIL(CS_CONTEXT *);
	exit(rv);
}

/*
**  GW_CFGSRV
**
**	Perform Open Server configuration.  Also initializes CT-Library.
**
**  Parameters:
**	cp		Open Server context.
**	np		Name of Open Server being initialized.
**
**  Returns:
**	CS_SUCCEED	Initialization was successful.
**	CS_FAIL		Initialization failed.
*/
CS_RETCODE	gw_cfgsrv(cp, np)
CS_CONTEXT	*cp;
CS_CHAR		*np;
{
	CS_BOOL		boolval;	/* ct_config parameter.		*/
	CS_CHAR		logname[1024];	/* Name of log file.		*/
	CS_CHAR		ctlogname[1024];/* Name of file for CT-lib tracing */

	/* Initialization.						*/
	boolval = CS_TRUE;		/* Truncate log on startup	*/

	/*
	** Install the Open Server error handler.
	*/
	if(srv_props(cp, CS_SET, SRV_S_ERRHANDLE,
		(CS_VOID *)server_err_handler, CS_SIZEOF(CS_VOID *),
		(CS_INT *)NULL) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	/*
	** Configure the logfile to be truncated on start-up.
	*/
        if(srv_props(cp, CS_SET, SRV_S_TRUNCATELOG, (CS_VOID *)&boolval, 
		CS_SIZEOF(boolval), NIL(CS_INT *)) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	/*
	** Configure the log file to have the same name as the server.
	*/
	(CS_VOID)sprintf(logname, "%s.log", np);
        if(srv_props(cp, CS_SET, SRV_S_LOGFILE, (CS_VOID *)logname, CS_NULLTERM,
		NIL(CS_INT *)) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	/*
	** Initialize Open Server.
	*/
	if(srv_init(NIL(SRV_CONFIG *), np, CS_NULLTERM) == NIL(SRV_SERVER *))
	{
		(CS_VOID)ct_exit(cp, CS_FORCE_EXIT);
		return(CS_FAIL);
	}

	/*
	** Initialize Client-Library. 
	*/
	if(ct_init(cp, CS_VERSION_100) == CS_FAIL)
	{
		(CS_VOID)fprintf(stderr, "ct_init failed\n");
		return(CS_FAIL);
	}

	/*
	** If CT-Lib context level tracing was set, activate it
	** now.
	*/
	if(Ctcflags != (CS_INT)0 || Conflags != (CS_INT)0)
	{
		/*
		** Configure the CT-lib trace files to have the same name
		** as the server.
		*/
		(CS_VOID)sprintf(ctlogname, "%s.ctlog", np);
		if(ct_debug(cp, (CS_CONNECTION *)NULL, CS_SET_DBG_FILE,
			CS_UNUSED, ctlogname, (CS_INT)strlen(ctlogname))
			!= CS_SUCCEED)
		{
			(CS_VOID)fprintf(stderr,
				"ct_debug(CS_SET_DBG_FILE) failed\n");
			return(CS_FAIL);
		}
	}
	if(Ctcflags != (CS_INT)0)
	{
		if(ct_debug(cp, (CS_CONNECTION *)NULL, CS_SET_FLAG, Ctcflags,
			(CS_CHAR *)NULL, (CS_INT)0) != CS_SUCCEED)
		{
			(CS_VOID)fprintf(stderr, "ct_debug(Ctcflags) failed\n");
			return(CS_FAIL);
		}
	}

	/*
	** Install CT-Libary error handler.
	*/
	if(ct_callback(cp, NIL(CS_CONNECTION *), CS_SET, CS_CLIENTMSG_CB,
		(CS_VOID *)gw_cterr) == CS_FAIL)
	{
		(CS_VOID)ct_exit(cp, CS_FORCE_EXIT);
		(CS_VOID)fprintf(stderr,
			"ct_callback(CS_CLIENTMSG_CB) failed\n");
		return(CS_FAIL);
	}

	/*
	** Install an encryption callback.
	*/
	if(ct_callback(cp, NIL(CS_CONNECTION *), CS_SET, CS_ENCRYPT_CB,
		(CS_VOID *)gw_encryptcb) == CS_FAIL)
	{
		(CS_VOID)ct_exit(cp, CS_FORCE_EXIT);
		return(CS_FAIL);
	}

	/*
	** Install a challenge callback.
	*/
	if(ct_callback(cp, NIL(CS_CONNECTION *), CS_SET, CS_CHALLENGE_CB,
		(CS_VOID *)gw_responsecb) == CS_FAIL)
	{
		(CS_VOID)ct_exit(cp, CS_FORCE_EXIT);
		return(CS_FAIL);
	}

	/*
	** Install CT-Library remote server message handler.
	*/
	if(ct_callback(cp, NIL(CS_CONNECTION *), CS_SET, CS_SERVERMSG_CB,
		(CS_VOID *)gw_remotemsgs) == CS_FAIL)
	{
		(CS_VOID)ct_exit(cp, CS_FORCE_EXIT);
		return(CS_FAIL);
	}

	/*
	** Request format results from ct_results.  This result allows us
	** to describe the parameters and rows to Open Server before we
	** receive the actual results.
	*/
	if(ct_config(cp, CS_SET, CS_EXPOSE_FMTS, (CS_VOID *)&boolval,
		CS_UNUSED, NIL(CS_INT *)) == CS_FAIL)
	{
		(CS_VOID)ct_exit(cp, CS_FORCE_EXIT);
		return(CS_FAIL);
	}

	return(CS_SUCCEED);
}

/*
** GW_SETVERSION
**
**	This routine allocate a context and installs the CS-Library error 
**	handler.  The version is also set for Open Server and CT-Library.
**
**  Parameters:
**	cpp		Return context here.
**
**  Returns:
**	CS_SUCCEED	Version information set successfully.
**	CS_FAIL		An error was detected.
*/
CS_RETCODE	gw_setversion(cpp)
CS_CONTEXT	**cpp;
{
	CS_CONTEXT	*cp;		/* Open Server context.		*/

	/*
	** Allocate a context structure for this Open Server application.
	*/
	if(cs_ctx_alloc(CS_VERSION_100, &cp) == CS_FAIL)
	{
		(CS_VOID)fprintf(stderr, "Could not allocate context\n");
		return(CS_FAIL);
	}

	/*
	** Install a CS-Library error handler.  CS-Library is used
	** internally in Open Server for conversions.  All Open Server 
	** applications must install a CS-Library error handler if they want to
	** receive conversion errors.
	*/
	if(cs_config(cp, CS_SET, CS_MESSAGE_CB, (CS_VOID *)cs_err_handler,
		CS_UNUSED, (CS_INT *)NULL) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "cs_config(CS_MESSAGE_CB) failed\n");
		return(CS_FAIL);
	}

	/*
	** Set the Open Server version and context.
	*/
	if(srv_version(cp, CS_VERSION_100) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "srv_version failed\n");
		return(CS_FAIL);
	}

	/*
	** Return the allocated context to the caller.
	*/
	*cpp = cp;
	return(CS_SUCCEED);
}

/*
**  GW_STARTHANDLER
**
**	This is the handler for SRV_START events.  It installs handlers
**	for the following events:
**	
**	Event		Routine		Comments
**	-----		-------		---------------------------------------
**	SRV_ATTENTION	gw_attnhandler	Process client attention requests.
**	SRV_BULK	gw_bulkhandler	Bulk data received from a client.
**	SRV_CONNECT	gw_connhandler	Process client connection requests.
**	SRV_CURSOR	gw_curhandler	Process client cursor requests.
**	SRV_DISCONNECT	gw_dischandler	Process client disconnections.
**	SRV_DYNAMIC	gw_dynhandler	Process client dynamic SQL requests.
**	SRV_LANGUAGE	gw_langhandler	Process client language requests.
**	SRV_RPC		gw_rpchandler	Process RPC requests.
**	SRV_OPTION	gw_opthandler	Process option cmds.
**
**  Parameters:
**	ssp		Open Server control structure.
**
**  Returns:
**	CS_SUCCEED	All event handlers were installed with out any
**			errors.
**	CS_FAIL		An error was detected during event handler installation.
**			srv_run will return CS_FAIL to the caller.
*/
CS_RETCODE	CS_PUBLIC gw_starthandler(ssp)
SRV_SERVER	*ssp;
{
	/*
	**	Registering the procedure to stop the
	**	Open Server
	*/
	if ( stop_regproc(ssp) == CS_FAIL )
	{
		return(CS_FAIL);
	}

	if(srv_handle(ssp, SRV_ATTENTION, gw_attnhandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		return(CS_FAIL);
	}

	if(srv_handle(ssp, SRV_BULK, gw_bulkhandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		return(CS_FAIL);
	}

	if(srv_handle(ssp, SRV_CONNECT, gw_connhandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		return(CS_FAIL);
	}

	if(srv_handle(ssp, SRV_CURSOR, gw_curhandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		return(CS_FAIL);
	}

	if(srv_handle(ssp, SRV_DISCONNECT, gw_dischandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		return(CS_FAIL);
	}

	if(srv_handle(ssp, SRV_DYNAMIC, gw_dynhandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		return(CS_FAIL);
	}

	if(srv_handle(ssp, SRV_LANGUAGE, gw_langhandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		return(CS_FAIL);
	}

	if(srv_handle(ssp, SRV_RPC, gw_rpchandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		return(CS_FAIL);
	}

	if(srv_handle(ssp, SRV_OPTION, gw_opthandler) 
			== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		return(CS_FAIL);
	}
	return(CS_SUCCEED);
}

/*
**  GW_CONNHANDLER
**
**	This is the SRV_CONNECT event handler.  A connection attempt is 
**	made to a remote SQL Server.
**
**  Parameters:
**	spp	The handle to the client connection that received the
**		SRV_CONNECT
**
**  Returns:
**	CS_SUCCEED	Connection was established to remote server.
**	CS_FAIL		Connection attempt failed.
*/
CS_RETCODE	CS_PUBLIC gw_connhandler(spp)
SRV_PROC	*spp;
{
	DBMSCTL_T	*rp;		/* Connection control structure */
	CS_INT		status;		/* DONE status.			*/
	CS_INT		conntype;	/* Connection type.		*/
	CS_CAP_TYPE	capmask;	/* capability mask		*/
	CS_CHAR		protologname[1024];/* protocol trace file name	*/
	CS_INT		protolognlen;	/* len of protocol trace file	*/

	/* Initialization.						*/
	status = SRV_DONE_FINAL;	/* Assume no errors.		*/
	rp = NIL(DBMSCTL_T *);		/* Not allocated yet.		*/

	/*
	** Get the thread type.
	*/
        if(srv_thread_props(spp, CS_GET, SRV_T_TYPE, (CS_VOID *)&conntype,
		CS_SIZEOF(conntype), NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}
 
	/*
	** If this is a site handler, just send a done and return.
	*/
	if(conntype == SRV_TSITE)
	{
		/*
		** Send a final done to the server.  We ignore
		** the return code here because Open Server will kill
		** this thread if this call fails.
		*/
		(CS_VOID)srv_senddone(spp, status, CS_TRAN_UNDEFINED,
			(CS_INT)0);
		return(CS_SUCCEED);
	}

	/*
	** Allocate a DBMSCTL_T information structure.
	*/
	if((rp = (DBMSCTL_T *)srv_alloc(CS_SIZEOF(DBMSCTL_T)))
		== NIL(DBMSCTL_T *))
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Clear the allocated control structure.
	*/
	srv_bzero((CS_VOID *)rp, CS_SIZEOF(*rp));

	/*
	** Save the remote data control structure in the thread control
	** structure.
	*/
	if(srv_thread_props(spp, CS_SET, SRV_T_USERDATA, (CS_VOID *)rp,
		CS_SIZEOF(rp), NIL(CS_INT *)) == CS_FAIL)
	{
		SRV_FREE(rp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Allocate a connection handle for the connection attempt.  If
	** we can't allocate a connection handle, report error
	** to client. The control structure will be freed
	** by the disconnect handler
	*/
	if(ct_con_alloc(Context, &(rp->chp)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** If CT-Lib connection level tracing was set, activate it
	** now.
	*/
	if(Conflags != (CS_INT)0)
	{
		if (Conflags & CS_DBG_PROTOCOL)
		{
			/*
			** Configure the CT-lib protocol trace file 
			** to have the same name as the server.
			*/
			(CS_VOID)srv_props(Context, CS_GET, SRV_S_SERVERNAME, 
				(CS_VOID *)protologname, 
				CS_SIZEOF(protologname), &protolognlen);

			(CS_VOID)sprintf(protologname, "%s.protolog",
				protologname);

			/*
			** Set trace file.  We ignore errors here because
			** we don't want to fail a connection just because
			** we can't turn on tracing
			*/
			(CS_VOID)ct_debug(Context, rp->chp,
				CS_SET_PROTOCOL_FILE, CS_UNUSED, protologname, 
				(CS_INT)protolognlen);
		}

		/*
		** Turn on trace flags.  Ignore errors so we don't fail
		** a connection request because we can't turn on tracing.
		*/
		(CS_VOID)ct_debug(Context, rp->chp, CS_SET_FLAG, Conflags,
			(CS_CHAR *)NULL, (CS_INT)0);
	}

	/*
	** Save the thread control structure in the connection handle.
	*/
	if(ct_con_props(rp->chp, CS_SET, CS_USERDATA, &spp, CS_SIZEOF(spp),
		NIL(CS_INT *)) == CS_FAIL)
	{
		(CS_VOID)ct_con_drop(rp->chp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Initialize the login record from the information provided
	** by the remote client.
	*/
	if(gw_initlogin(spp, rp->chp) == CS_FAIL)
	{
		(CS_VOID)ct_con_drop(rp->chp);
		return(CS_FAIL);
	}

	/*
	** Initialize variables associated with negotiated logins
	*/
	rp->NumEncrypts = (CS_INT)0;
	rp->EncryptPwdSent = CS_FALSE;
	rp->EncryptToGet = (CS_INT)0;
	rp->NumSecLabels = (CS_INT)0;
	rp->LabelToSend = (CS_INT)0;

	/*
	** Attempt a connection to the remote SQL Server.
	*/
	if(ct_connect(rp->chp, SqlServername, CS_NULLTERM) == CS_FAIL)
	{
		(CS_VOID)ct_con_drop(rp->chp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Retrieve capabilities negotiated by CTLIB and the remote server, 
	** and set this on this client connection.
	*/

	/* Get and set the REQUEST capabilities				*/
	srv_bzero(&capmask, sizeof(CS_CAP_TYPE));
	if (ct_capability(rp->chp, CS_GET, CS_CAP_REQUEST, CS_ALL_CAPS, 
		&capmask) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}
	if (srv_capability_info(spp, CS_SET, CS_CAP_REQUEST, CS_ALL_CAPS, 
		&capmask) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}
	
	/* Get and set RESPONSE capabilities				*/
	srv_bzero(&capmask, sizeof(CS_CAP_TYPE));
	if (ct_capability(rp->chp, CS_GET, CS_CAP_RESPONSE, CS_ALL_CAPS, 
		&capmask) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}
	if (srv_capability_info(spp, CS_SET, CS_CAP_RESPONSE, CS_ALL_CAPS, 
		&capmask) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Send a final done to the client to acknowledge the connection
	** request. We ignore the return code here because Open Server
	** will kill this thread if this call fails.
	*/
	(CS_VOID)srv_senddone(spp, SRV_DONE_FINAL, CS_TRAN_UNDEFINED,
		(CS_INT)0);
	return(CS_SUCCEED);
}

/*
**  GW_LANGHANDLER
**
**	Execute a client language request on the remote DBMS.
**	Any results from the remote DBMS are passed back to the
**	Open Server client.
**
**  Parameters:
**	spp	Open Server thread control structure.
**
**  Returns:
**	CS_SUCCEED	Language was processed successfully.
**	CS_FAIL		An error was detected while processing language.
*/
CS_RETCODE	CS_PUBLIC gw_langhandler(spp)
SRV_PROC	*spp;
{
	CS_BOOL		ok;		/* Error control variable.	*/
	CS_CHAR		*bp;		/* Language buffer.		*/
	CS_CONNECTION	*chp;		/* Connection handle to server.	*/
	CS_COMMAND	*cmdp;		/* Language command handle.	*/
	CS_INT		size;		/* Size of language request.	*/
	CS_INT		status;		/* DONE status.			*/
	CS_INT		rows;		/* Number of rows processed.	*/
	CS_RETCODE	bulkcmd;	/* gw_chkwritetext return value. */

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	bp = NIL(CS_CHAR *);		/* No lang buffer yet.		*/
	status = SRV_DONE_FINAL;	/* Assume no errors.		*/
	bulkcmd = GW_NONE;		/* Assume not a bulk command.	*/
	rows = 0;			/* Assume no rows are processed. */

	/* Get the connection handle for the server connection.		*/
	if((chp = gw_getchp(spp, CS_FALSE)) == NIL(CS_CONNECTION *)) 
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Allocate a command handle for language command.		*/
	if(ct_cmd_alloc(chp, &cmdp) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Determine the size of the language string.			*/
	if((size = srv_langlen(spp)) < 0)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Allocate language string buffer.  Leave room for NULL. 	*/
	if((bp = (CS_CHAR *)srv_alloc(size + 1)) == NIL(CS_CHAR *))
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Copy language string into allocated buffer.			*/
	if(srv_langcpy(spp, (CS_INT)0, (CS_INT)(-1), bp) < 0)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		gw_senderror(spp);
		return(CS_FAIL);
	}
		
	/* Check for a bulk action by client.				*/
	bulkcmd = gw_chkwritetext(spp, chp, bp);

	/*
	** If we got a valid "insert bulk", all the
	** necessary processing was done in gw_chkwritetext
	** so we can just send a done and return.
	*/
	if((bulkcmd == GW_INSRTBLK) || (bulkcmd == GW_WRTXTBLK))
	{
		/*
		** Send the final DONE to the client.  We ignore the
		** return code because Open Server will kill this thread
		** if this call fails.
		*/
		(CS_VOID)srv_senddone(spp, status, CS_TRAN_UNDEFINED,
			(CS_INT)0);

		/*
		** Release all allocated handles and memory.
		*/
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_SUCCEED);
	}

	/*
	** Define the language command to CT-Library.
	*/
	if(ct_command(cmdp, CS_LANG_CMD, bp, size, CS_UNUSED) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Send the language request to the remote SQL Server.  gw_ctsend
	** will send any required DONES back to the client.
	*/
	if(gw_ctsend(spp, cmdp) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FAIL);
	}

	/*
	** Process the results from the remote SQL Server.
	*/
	if((rows = gw_handleresults(spp, cmdp, CS_LANG_CMD)) < 0)
	{
		ok = CS_FALSE;
	}

	/* Set DONE row status based on number of rows sent to client.	*/
	if(ok && (rows > 0))
	{
		status |= SRV_DONE_COUNT;
	}

	/*
	** If there was an error detected send a DONE with an error
	** and cancel current command.
	*/
	if(!ok) 
	{
		status |= SRV_DONE_ERROR;
		(CS_VOID)ct_cancel(NIL(CS_CONNECTION *), cmdp, CS_CANCEL_ALL);
	}

	/*
	** Send the final done to the client.  We ignore the return
	** value here because Open Server will kill this thread if a final
	** done isn't sent.
	*/
	(CS_VOID)srv_senddone(spp, status, CS_TRAN_UNDEFINED, rows);

	/* Release all allocated handles and memory.			*/
	(CS_VOID)ct_cmd_drop(cmdp);
	SRV_FREE(bp);
	return(CS_SUCCEED);
}

/*
**  GW_RPCHANDLER
**
**	Handle RPC requests from a client connection of a server-to-server
**	connection.
**
**  Parameters:
**	spp - Handle to the current client connection.
**
**  Returns:
**	CS_SUCCEED	RPC was procssed without error.
**	CS_FAIL		An error was detected.
*/
CS_RETCODE	CS_PUBLIC gw_rpchandler(spp)
SRV_PROC	*spp;
{
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_INT		len;		/* RPC name length.		*/
	CS_INT		rows;		/* Number rows sent to client.	*/
	CS_CONNECTION	*chp;		/* Server connection handle.	*/
	CS_COMMAND	*cmdp;		/* RPC command handle.		*/
	CS_INT		status;		/* DONE status.			*/
	CS_CHAR		fullrpcname[CS_MAX_NAME];
	CS_CHAR		*rpcnp;		/* Pointer to RPC name.		*/
	CS_CHAR		*rpcdbp;	/* Pointer to RPC db.		*/
	CS_CHAR		*rpcownerp;	/* Pointer to RPC owner.		*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is well.			*/
	status = SRV_DONE_FINAL;	/* Assume no errors.		*/
	rows = 0;			/* No rows sent to client.	*/
	srv_bzero(fullrpcname, CS_MAX_NAME);

	/* Get connection handle for server connection.			*/
	if((chp = gw_getchp(spp, CS_FALSE)) == NIL(CS_CONNECTION *))
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Allocate a command handle for RPC command.			*/
	if(ct_cmd_alloc(chp, &cmdp) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Get the RPC database received from client.			*/
	if((rpcdbp = srv_rpcdb(spp, &len)) == NIL(CS_CHAR *))
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	strncpy(fullrpcname, rpcdbp, len);
	strcat(fullrpcname, ".");
	
	/* Get the RPC owner received from client.			*/
	if((rpcownerp = srv_rpcowner(spp, &len)) == NIL(CS_CHAR *))
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	strncat(fullrpcname, rpcownerp, len);
	strcat(fullrpcname, ".");
	
	/* Get the RPC name received from client.			*/
	if((rpcnp = srv_rpcname(spp, &len)) == NIL(CS_CHAR *))
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	strncat(fullrpcname, rpcnp, len);
	
	/* Define RPC command.						*/
	if(ct_command(cmdp, CS_RPC_CMD, fullrpcname, strlen(fullrpcname),
		CS_UNUSED) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Process any parameters received from the client.
	*/
	if(gw_clientparam(spp, cmdp, SRV_RPCDATA) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Process the RPC results from the remote server.
	*/
	if((rows = gw_handleresults(spp, cmdp, CS_RPC_CMD)) < 0)
	{
		ok = CS_FALSE;
	}

	/* Set DONE row status based on number of rows sent to client.	*/
	if(ok && (rows > 0))
	{
		status |= SRV_DONE_COUNT;
	}

	/*
	** If there was an error detected send a DONE with an error
	** and cancel current command.
	*/
	if(!ok) 
	{
		status |= SRV_DONE_ERROR;
		(CS_VOID)ct_cancel(NIL(CS_CONNECTION *), cmdp, CS_CANCEL_ALL);
	}

	/*
	** Send the final done to the client.  We ignore the return
	** value here because Open Server will kill this thread if a final
	** done isn't sent.
	*/
	(CS_VOID)srv_senddone(spp, status, CS_TRAN_UNDEFINED, rows);

	/* Release all allocated handles and memory.			*/
	(CS_VOID)ct_cmd_drop(cmdp);
	return(CS_SUCCEED);
}

/*
**  GW_OPTHANDLER
**
**	Handle option commands from a client connection
**
**  Parameters:
**	spp - Handle to the current client connection.
**
**  Returns:
**	CS_SUCCEED	option command was procssed without error.
**	CS_FAIL		An error was detected.
*/
CS_RETCODE	CS_PUBLIC gw_opthandler(spp)
SRV_PROC	*spp;
{
	CS_CHAR		msg[1024];	/* Build message here.		*/
	CS_CONNECTION	*chp;		/* Server connection handle.	*/
	CS_INT		status;		/* DONE status.			*/
	CS_INT		optcmd;
	CS_INT		option;
	CS_INT		outlen;
	CS_INT		sendopt;
	CS_BYTE		optionval[SRV_MAXCHAR];
	CS_BOOL		ok;

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is well.			*/
	status = SRV_DONE_FINAL;	/* Assume no errors.		*/

	srv_bzero(optionval, SRV_MAXCHAR);

	/* Get connection handle for server connection.			*/
	if((chp = gw_getchp(spp, CS_FALSE)) == NIL(CS_CONNECTION *))
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* See what type of option command the client sent.		*/
	if (srv_options(spp, CS_GET, &optcmd, &option, 
		optionval, SRV_MAXCHAR, &outlen) != CS_SUCCEED)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Check what type of action should be taken			*/
	switch(optcmd)
	{
	case SRV_SETOPTION:
		/*
		** The client is setting an option
		*/
		switch(option)
		{
		case CS_OPT_DATEFIRST:
		case CS_OPT_TEXTSIZE:
		case CS_OPT_ROWCOUNT:
		case CS_OPT_DATEFORMAT:
		case CS_OPT_ISOLATION:
		case CS_OPT_STATS_TIME:
		case CS_OPT_SHOWPLAN:
		case CS_OPT_ARITHIGNORE:
		case CS_OPT_TRUNCIGNORE:
		case CS_OPT_ARITHABORT:
		case CS_OPT_PARSEONLY:
		case CS_OPT_ANSINULL:
		case CS_OPT_QUOTED_IDENT:
		case CS_OPT_STATS_IO:
		case CS_OPT_NOEXEC:
		case CS_OPT_GETDATA:
		case CS_OPT_NOCOUNT:
		case CS_OPT_FORCEPLAN:
		case CS_OPT_FORMATONLY:
		case CS_OPT_CHAINXACTS:
		case CS_OPT_CURCLOSEONXACT:
		case CS_OPT_FIPSFLAG:
		case CS_OPT_RESTREES:
			/* Set the option on the remote server		*/
			if (ct_options(chp, CS_SET, option, optionval, 
				CS_UNUSED, (CS_INT *)NULL) != CS_SUCCEED)
			{
				ok = CS_FALSE;
			}
			break;

		case CS_OPT_IDENTITYON:
		case CS_OPT_AUTHON:
		case CS_OPT_AUTHOFF:
		case CS_OPT_CURREAD:
		case CS_OPT_CURWRITE:
		case CS_OPT_IDENTITYOFF:
		case CS_OPT_NATLANG:
		case CS_OPT_CHARSET:
			/* Set the option on the remote server		*/
			if (ct_options(chp, CS_SET, option, optionval, outlen,
				(CS_INT *)NULL) != CS_SUCCEED)
			{
				ok = CS_FALSE;
			}
			break;

		default:
			SPRINTF(msg, "gw_opthandler: Unknown option (0x%x)\n",
				option);
			SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);
			ok = CS_FALSE;
			break;
		}

		break;

	case SRV_GETOPTION:

		sendopt = SRV_SENDOPTION;

		/*
		** The client wants to get an option, so we have
		** to get it and then send it back.
		*/
		switch(option)
		{
		case CS_OPT_DATEFIRST:
		case CS_OPT_TEXTSIZE:
		case CS_OPT_ROWCOUNT:
		case CS_OPT_DATEFORMAT:
		case CS_OPT_ISOLATION:
		case CS_OPT_STATS_TIME:
		case CS_OPT_SHOWPLAN:
		case CS_OPT_ARITHIGNORE:
		case CS_OPT_TRUNCIGNORE:
		case CS_OPT_ARITHABORT:
		case CS_OPT_PARSEONLY:
		case CS_OPT_ANSINULL:
		case CS_OPT_QUOTED_IDENT:
		case CS_OPT_STATS_IO:
		case CS_OPT_NOEXEC:
		case CS_OPT_GETDATA:
		case CS_OPT_NOCOUNT:
		case CS_OPT_FORCEPLAN:
		case CS_OPT_FORMATONLY:
		case CS_OPT_CHAINXACTS:
		case CS_OPT_CURCLOSEONXACT:
		case CS_OPT_FIPSFLAG:
		case CS_OPT_RESTREES:
			/* Get the option from the remote server	*/
			if (ct_options(chp, CS_GET, option, optionval,
				CS_UNUSED, &outlen) != CS_SUCCEED)
			{
				ok = CS_FALSE;
			}

			/* Send the option to the remote client		*/
			if (ok && (srv_options(spp, CS_SET, &sendopt, &option,
				optionval, outlen, (CS_INT *)NULL)
				!= CS_SUCCEED))
			{
				ok = CS_FALSE;
			}
			break;

		case CS_OPT_IDENTITYON:
		case CS_OPT_AUTHON:
		case CS_OPT_AUTHOFF:
		case CS_OPT_CURREAD:
		case CS_OPT_CURWRITE:
		case CS_OPT_IDENTITYOFF:
		case CS_OPT_NATLANG:
		case CS_OPT_CHARSET:
			/* Get the option from the remote server	*/
			if (ct_options(chp, CS_GET, option, optionval,
				CS_SIZEOF(optionval), &outlen) != CS_SUCCEED)
			{
				ok = CS_FALSE;
			}

			/* Send the option to the remote client		*/
			if (ok && (srv_options(spp, CS_SET, &sendopt, &option,
				optionval, outlen, (CS_INT *)NULL)
				!= CS_SUCCEED))
			{
				ok = CS_FALSE;
			}
			break;

		default:
			SPRINTF(msg, "gw_opthandler: Unknown option (0x%x)\n",
				option);
			SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);
			ok = CS_FALSE;
			break;

		}
		break;

	case SRV_CLEAROPTION:
		/* Clear the option on the remote server		*/
		if (ct_options(chp, CS_CLEAR, option, (CS_INT *)NULL, 
			CS_UNUSED, (CS_INT *)NULL) != CS_SUCCEED)
		{
			ok = CS_FALSE;
		}
		break;

	default:
		SPRINTF(msg, "gw_opthandler: Unknown option command (0x%x)\n",
			optcmd);
		SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);
		ok = CS_FALSE;
		break;
	}

	if (ok != CS_TRUE)
	{
		status |= SRV_DONE_ERROR;
	}

	/*
	** Send the final done to the client.  We ignore the return
	** value here because Open Server will kill this thread if 
	** a final done isn't sent.
	*/
	(CS_VOID)srv_senddone(spp, status, CS_TRAN_UNDEFINED, 0);

	return(CS_SUCCEED);
}

/*
**  GW_DISCHANDLER
**
**	Handler for SRV_DISCONNECT events. Closes the remote DBMS connection
**	if appropriate.  SRV_DISCONNECT events are generated when an
**	client disconnects from the Open Server.
**
**  Parameters:
**	spp	Thread control structure
**
**  Returns:
**	CS_SUCCEED
*/
CS_RETCODE	CS_PUBLIC gw_dischandler(spp)
SRV_PROC	*spp;
{
	CS_CONNECTION	*chp;		/* Connection handle to server.	*/

	/* If connection with server active, close it now.		*/
	if((chp = gw_getchp(spp, CS_TRUE)) != NIL(CS_CONNECTION *))
	{
		(CS_VOID)ct_close(chp, CS_UNUSED);
		(CS_VOID)ct_con_drop(chp);
	}
	return(CS_SUCCEED);
}

/*
**  GW_BULKHANDLER
**
**	Bulk event handler.  SRV_BULK events are generated when a client
**	issues a writetext or a write bulk command.
**
**  Parameters:
**	spp	Thread control structure.
**
**  Returns:
**	CS_SUCCEED	Bulk command was processed successfully.
**	CS_FAIL		Bulk command failed.
*/
CS_RETCODE	CS_PUBLIC gw_bulkhandler(spp)
SRV_PROC	*spp;
{
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_CONNECTION	*chp;		/* Connection handle to server.	*/
	CS_INT		iotype;		/* Type of bulk IO event.	*/
	CS_COMMAND	*cmdp;		/* Bulk command handle.		*/
	CS_INT		status;		/* done status			*/

	/* Initialization. 						*/
	ok = CS_TRUE;			/* All is well.			*/
	status = SRV_DONE_FINAL;
	Bulkrows = 0;

	/* Get connection handle for server connection.			*/
	if((chp = gw_getchp(spp, CS_FALSE)) == NIL(CS_CONNECTION *))
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Allocate a command handler for the bulk command.
	*/
	if(ct_cmd_alloc(chp, &cmdp) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Get the current bulk type being sent by the client.  This
	** was set in the language handler.
	*/
	if(srv_thread_props(spp, CS_GET, SRV_T_BULKTYPE, (CS_VOID *)&iotype,
		CS_SIZEOF(iotype), NIL(CS_INT *)) == CS_FAIL) 
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Process according to bulk type.
	*/
	switch((int)iotype)
	{
	case SRV_BULKLOAD:
		if(gw_procbcp(spp) == CS_FAIL)
		{
			ok = CS_FALSE;
		}
		status |= SRV_DONE_COUNT;
		break;

	case SRV_TEXTLOAD:
	case SRV_IMAGELOAD:
		if(gw_proctextimage(spp, cmdp) == CS_FAIL)
		{
			ok = CS_FALSE;
		}
		break;

	default:
		/*
		** This should never happen.
		*/
		ok = CS_FALSE;
		gw_senderror(spp);
		break;
	}

	/*
	** If there was an error, cancel the current result set.
	*/
	if(!ok) 
	{
		(CS_VOID)ct_cancel(NIL(CS_CONNECTION *), cmdp, CS_CANCEL_ALL);
		(CS_VOID)ct_cmd_drop(cmdp);
		return(CS_FAIL);
	}

	/*
	** Send the final done to the client.  We ignore the return code
	** here because Open Server will kill this thread if the final
	** done isn't sent.
	*/

	(CS_VOID)srv_senddone(spp, status, CS_TRAN_UNDEFINED,
		(CS_INT)Bulkrows);
	(CS_VOID)ct_cmd_drop(cmdp);
	return(SRV_CONTINUE);
}

/*
**  GW_PROCTEXTIMAGE
**
**	Process write text/image bulk copy data
**
**  Parameters:
**	spp 		Thread control structure.
**	cmdp		Allocated command handle.
**
**  Returns:
**	CS_SUCCEED	Text/Image was processed successfully.
**	CS_FAIL		An error was detected.
*/
CS_RETCODE	gw_proctextimage(spp, cmdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
{
	CS_IODESC	docdesc;	/* Bulk IO Descriptor.		*/
	CS_BOOL		done;		/* Loop control.		*/
	CS_BYTE		buf[GW_BULKBUFSIZE];/* Bulk data buffer.	*/
	CS_INT		outlen;		/* Amount of data read.		*/
	CS_RETCODE	ret;		/* Return code.			*/
	CS_IODESC	tmpdesc;	/* Temporary Bulk IO Descriptor	*/
	DBMSCTL_T	*rp;		/* Connection control struct	*/

	/* Initialization. 						*/
	done = CS_FALSE;

	/* Get user data from SRV_PROC structure			*/
	if (srv_thread_props(spp, CS_GET, SRV_T_USERDATA, (CS_VOID *)&rp,
		sizeof(rp), NIL(CS_INT *)) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Clear IO descriptor						*/
	srv_bzero(&tmpdesc, sizeof(CS_IODESC));

	/* Get IO descriptor file intialized during chk_write_text()	*/
	docdesc = rp->iodesc;

	/*
	** Get information on the bulk data being sent by the client.
	** The client can only send rows with one column of bulk data
	** so we just hard code item number to 1.
	*/
	if(srv_text_info(spp, CS_GET, 1, &tmpdesc) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	/*
	** Initiate the text/image update operation with the remote SQL
	** server.
	*/
	if(ct_command(cmdp, CS_SEND_DATA_CMD, NIL(CS_CHAR *), CS_UNUSED,
		CS_COLUMN_DATA) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	/*
	** Update the CS_IODESC for CT-Library.
	*/
	docdesc.iotype = CS_IODATA;
	docdesc.datatype = CS_TEXT_TYPE;
	docdesc.total_txtlen = tmpdesc.total_txtlen;
	if(ct_data_info(cmdp, CS_SET, CS_UNUSED, &docdesc) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	while(!done)
	{
		/*
		** Read a buffer of data from the client.
		*/
		if((ret = srv_get_text(spp, buf, CS_SIZEOF(buf), &outlen))
			== CS_FAIL)
		{
			gw_senderror(spp);
			return(CS_FAIL);
		}

		/*
		** Send the data buffer to the remote SQL server.
		*/
		if(ct_send_data(cmdp, buf, outlen) == CS_FAIL)
		{
			gw_senderror(spp);
			return(CS_FAIL);
		}

		/*
		** Have we read all of the data from the client?
		*/
		if(ret == CS_END_DATA)
		{
			done = CS_TRUE;
		}
	}

	/*
	** Complete the sending of the data to the remote server.
	*/
	if(gw_ctsend(spp, cmdp) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	/* Read results from server.					*/
	if (gw_handleresults(spp, cmdp, SRV_BULK) < 0)
	{
		ret = CS_FAIL;
		gw_senderror(spp);
	}
	else
		ret = CS_SUCCEED;

	return(ret);
}

/*
** GW_PROCBCP
**
**	Process insert bulk commands received from a client.  The insert
**	bulk command was detected in the language handler.
**
**  Parameters:
**	spp 	Thread control structure.
**
**  Returns:
**	CS_SUCCEED	Insert bulk was processed successfully.
**	CS_FAIL		An error was detected.
*/
CS_RETCODE	gw_procbcp(spp)
SRV_PROC	*spp;
{
	CS_BLK_ROW	*row;
	CS_BYTE		buf[GW_BULKBUFSIZE];	/* Buffer for bulk data.*/
	CS_INT		outlen;			/* Len of data sent to client.*/
	CS_RETCODE	getrow_res;		/* blk_getrow return value.*/
	CS_RETCODE	gettxt_res;		/* bulk_get_text return value.*/
	DBMSCTL_T	*rp;			/* Connection control struct.*/
#if	defined(NOT_DEFINED)
	CS_INT		tmp_iotype = SRV_TEXTLOAD;
	CS_TINYINT	coloneval;
	CS_SMALLINT	coltwoval;
	CS_INT		colthreeval;
	CS_CHAR		colfourval[100];
	CS_FLOAT	colsevenval;
	CS_CHAR		coleightval[100];
#endif	/* NOT_DEFINED */

	/* Get user data from SRV_PROC structure.			*/
	if(srv_thread_props(spp, CS_GET, SRV_T_USERDATA, (CS_VOID *)&rp,
		CS_SIZEOF(rp), NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Allocate a bulk copy row structure.
	*/
	if(blk_rowalloc(spp, &row) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Get a formatted row at a time from the client.		*/
	while((getrow_res = blk_getrow(spp, rp->blkdesc, row)) != CS_END_DATA)
	{
		if(getrow_res == CS_FAIL)
		{
			(CS_VOID)blk_done(rp->blkdesc, CS_BLK_ALL, &Bulkrows);
			(CS_VOID)blk_rowdrop(spp, row);
			gw_senderror(spp);
			return(CS_FAIL);
		}

#if		defined(NOT_DEFINED)

		/*
		** In order to use blk_colval() on a table column, 
		** the application must know the type of the column.
		** Therefore, the following code won't work for
		** random bulk copying.
		*/
		if(blk_colval(spp, rp->blkdesc, row, 1, &coloneval, 1, &outlen)
			== CS_FAIL )
		{
			FPRINTF(stderr,"blk_colval() FAILED on column 1\n");
		}
		else
		{
			FPRINTF(stderr,"len of column 1: %d\n", outlen);
			FPRINTF(stderr,"value of column 1: %d\n\n", coloneval);
		}

		if(blk_colval(spp, rp->blkdesc, row, 2, &coltwoval, 2, &outlen)
			== CS_FAIL )
		{
			FPRINTF(stderr,"blk_colval() FAILED on column 2\n");
		}
		else
		{
			FPRINTF(stderr,"len of column 2: %d\n", outlen);
			FPRINTF(stderr,"value of column 2: %d\n\n", coltwoval);
		}

		if(blk_colval(spp, rp->blkdesc, row, 3, &colthreeval, 4,
			&outlen) == CS_FAIL )
		{
			FPRINTF(stderr,"blk_colval() FAILED on column 3\n");
		}
		else
		{
			FPRINTF(stderr,"len of column 3: %d\n", outlen);
			FPRINTF(stderr,"value of column 3: %d\n\n",
				colthreeval);
		}

		if(blk_colval(spp, rp->blkdesc, row, 7, &colsevenval, 8,
			&outlen) == CS_FAIL )
		{
			FPRINTF(stderr,"blk_colval() FAILED on column 7\n");
		}
		else
		{
			FPRINTF(stderr,"len of column 7: %d\n", outlen);
			FPRINTF(stderr,"value of column 7: %f\n\n",
				colsevenval);
		}

		if(blk_colval(spp, rp->blkdesc, row, 8, coleightval, 100,
			&outlen) == CS_FAIL )
		{
			FPRINTF(stderr,"blk_colval() FAILED on column 8\n");
		}
		else
		{
			FPRINTF(stderr,"len of column 8: %d\n", outlen);
			coleightval[outlen] = '\0';
			FPRINTF(stderr,"value of column 8: '%s'\n\n",
				coleightval);
		}
#endif		/* NOT_DEFINED */

		/* Copy the formatted row to the SQL-Server.		*/
		if(blk_sendrow(rp->blkdesc, row) == CS_FAIL)
		{
			(CS_VOID)blk_done(rp->blkdesc, CS_BLK_ALL, &Bulkrows);
			(CS_VOID)blk_rowdrop(spp, row);
			gw_senderror(spp);
			return(CS_FAIL);
		}

		Bulkrows++;

		/*
		** Check to see if this bulk copy row has a text or image
		** column.  If it doesn't we are all done with this row.
		** Go get the next one.
		*/
		if(getrow_res != CS_BLK_HAS_TEXT)
		{
			continue;
		}

		/*
		** Read the text/image columns from the client 
		** a chunk at a time.
		*/
		while((gettxt_res = blk_gettext(spp, rp->blkdesc, row, buf,
			CS_SIZEOF(buf), &outlen)) != CS_FAIL)
		{
			/*
			** Write the text/image columns to the SQL-Server 
			** a chunk at a time.
			*/
			if(blk_sendtext(rp->blkdesc, row, buf, outlen)
				== CS_FAIL)
			{
				(CS_VOID)blk_done(rp->blkdesc, CS_BLK_ALL,
					&Bulkrows);
				(CS_VOID)blk_rowdrop(spp, row);
				gw_senderror(spp);
				return(CS_FAIL);
			}

			/*
			** Are we all done with the text/image data?
			*/
			if(gettxt_res == CS_END_DATA)
			{
				break;
			}
		}

		if(gettxt_res == CS_FAIL)
		{
			(CS_VOID)blk_done(rp->blkdesc, CS_BLK_ALL, &Bulkrows);
			(CS_VOID)blk_rowdrop(spp, row);
			gw_senderror(spp);
			return(CS_FAIL);
		}
	}

	/*
	** Clean up and terminate this bulk copy operation.
	*/
	(CS_VOID)blk_rowdrop(spp, row);

	if(blk_done(rp->blkdesc, CS_BLK_ALL, &Bulkrows) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	if(blk_drop(rp->blkdesc) == CS_FAIL)
	{
		return(CS_FAIL);
	}
	rp->blkdesc = NIL(CS_BLKDESC *);
	return(CS_SUCCEED);
}

/*
**  GW_CURHANDLER
**
**	Process cursor events.  Cursor events are generated when a
**	client sends a cursor command.  This routine just determines
**	the cursor command and then calls another routine to do the
**	actual work for that cursor command.  The client's request is
**	responded to with a DONE by this routine based on success or
**	failure of processing the cursor command.
**
**  Parameters:
**	spp	Thread control structure.
**
**  Returns:
**	CS_SUCCEED	Cursor command was processed successfully.
**	CS_FAIL		An error was detected.
*/
CS_RETCODE	CS_PUBLIC gw_curhandler(spp)
SRV_PROC	*spp;
{
	CS_CHAR		msg[1024];	/* Error message buffer.	*/
	CS_INT		options;	/* Options to client's command.	*/
	CS_INT		status;		/* DONE status.			*/
	CS_BOOL		ok;		/* Error control flag.		*/
	SRV_CURDESC	cd;		/* Describes the current cursor.*/
	CS_INT		rows;		/* Number of rows processed.	*/

	/* Initialization.						*/
	status = SRV_DONE_FINAL;
	ok = CS_TRUE;
	rows = (CS_INT)0;
	options = 0;

	/* Get cursor command sent by client.				*/
	if(srv_cursor_props(spp, CS_GET, &cd) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Get cursor command options. This will become
	** the "option" argument to ct_cursor().
	*/
	if(cd.cmdoptions & SRV_CUR_UNUSED)
	{
		options = CS_UNUSED;
	}
	else if(cd.cmdoptions & SRV_CUR_DEALLOC)
	{
		options = CS_DEALLOC;
	}
	else
	{
		/*
		** These are the only option values that
		** are bitmasks
		*/
		if(cd.cmdoptions & SRV_CUR_UPDATABLE)
		{
			options |= CS_FOR_UPDATE;
		}

		if(cd.cmdoptions & SRV_CUR_RDONLY)
		{
			options |= CS_READ_ONLY;
		}

		if(cd.cmdoptions & SRV_CUR_DYNAMIC)
		{
			options |= CS_DYNAMIC;
		}
	}

	/* If nothing was set */
	if (options == 0)
	{
		options = CS_UNUSED;
	}

	/* Process according to cursor command type.			*/
	switch((int)cd.curcmd)
	{
	case CS_CURSOR_DECLARE:
		ok = gw_curdeclare(spp, &cd, options);
		break;

	case CS_CURSOR_INFO:
		ok = gw_curinfo(spp, &cd, options);
		break;

	case CS_CURSOR_OPEN:
		ok = gw_curopen(spp, &cd, options);
		break;

	case CS_CURSOR_FETCH:
		/*
		** We will be returning rows to the client.  Update the
		** done status to indicate that row count is valid.
		*/
		ok = gw_curfetch(spp, &cd, options, &rows);
		status |= SRV_DONE_COUNT;
		break;

	case CS_CURSOR_UPDATE:
	case CS_CURSOR_DELETE:
		ok = gw_curupdate(spp, &cd, options);
		break;

	case CS_CURSOR_CLOSE:
		ok = gw_curend(spp, &cd, options);
		break;

	default:
		SPRINTF(msg, "gw_curhandler: Unknown cursor cmd(0x%x)\n",
			cd.curcmd);
		SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);
		ok = CS_FALSE;
		break;
	}

	/*
	** If an error was detected, send a done error and return CS_FAIL.
	*/
	if(!ok)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Send the final done for the client.
	*/
	(CS_VOID)srv_senddone(spp, status, CS_TRAN_UNDEFINED, rows);
	return(CS_SUCCEED);
}

/*
** GW_CURDECLARE
**
**	Process cursor declare commands.  This routine does no state 
**	validation of cursor commands.  All state validatation is assumed to 
**	be done by the client library.  It is the caller's responsibility to 
**	send the final DONE response to the client's cursor declare request 
**	based on the return value of this routine.  Any required intermediate 
**	DONEs are sent by this routine.
**
**  Parameters:
**	spp		Thread control structure.
**	cdp		Cursor control structure.
**	opts		Cursor command options received from client.
**
**  Returns:
**	CS_TRUE		Command was processed successfully.
**	CS_FALSE 	An error occurred while processing cursor command.
*/
CS_BOOL		gw_curdeclare(spp, cdp, opts)
SRV_PROC	*spp;
SRV_CURDESC	*cdp;
CS_INT		opts;
{
	CS_COMMAND	*cmdp;		/* Declare command handle	*/
	CS_DATAFMT	fmt;		/* Parameter format information.*/
	CS_INT		size;		/* Memory size variable.	*/
	CS_CHAR		*bp;		/* Pointer to select string buffer..*/
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_INT		curid;		/* Cursor ID for client.	*/
	CS_BOOL		keys;		/* Properties requested.	*/
	CS_CONNECTION	*chp;		/* Server connection handle.	*/
	CS_INT		numparams;	/* Number of cursor parameters.	*/
	CURCTL_T	*cp;		/* Current cursor control structure*/
	CS_INT		i;
	CS_INT		optionopt;

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	bp = NIL(CS_CHAR *);		/* No buffer allocated.		*/
	curid = (CS_INT)(-1);		/* No cursor ID set.		*/
	keys = CS_TRUE;			/* We want KEY data streams.	*/
	optionopt = 0;

	/* Get connection handle for this server connection.		*/
	if((chp = gw_getchp(spp, CS_FALSE)) == NIL(CS_CONNECTION *))
	{
		return(CS_FALSE);
	}

	/* Allocate a command handle for cursor command.		*/
	if(ct_cmd_alloc(chp, &cmdp) == CS_FAIL)
	{
		return(CS_FALSE);
	}

	/* Make sure we received a name for the cursor.			*/
	if((cdp->curnamelen == CS_UNUSED) || (cdp->curnamelen == (CS_INT)0))
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FALSE);
	}

	/* Read the size of the cursor select string.			*/
	if((size = srv_langlen(spp)) < 0)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		return(CS_FALSE);
	}

	/*
	** Allocate a buffer for the cursor select string.  Leave room for
	** null termination byte.
	*/
	if((bp = (CS_CHAR *)srv_alloc(size + 1)) == NIL(CS_CHAR *))
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FALSE);
	}

	/* Copy select string into allocated buffer.			*/
	if(srv_langcpy(spp, (CS_INT)0, (CS_INT)(-1), bp) < 0)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}
		
	/*
	** If we're declaring a cursor on a dynamic prepared statement,
	** we process things differently.
	*/
	if (opts & CS_DYNAMIC)
	{
		if(ct_dynamic(cmdp, CS_CURSOR_DECLARE, bp, size, cdp->curname,
			cdp->curnamelen) == CS_FAIL)
		{
			(CS_VOID)ct_cmd_drop(cmdp);
			SRV_FREE(bp);
			return(CS_FALSE);
		}

		/* We use ct_cursor() to set any other options		*/
		if (opts & CS_READ_ONLY)
		{
			optionopt = CS_READ_ONLY;
		}
		else if (opts & CS_FOR_UPDATE)
		{
			optionopt = CS_FOR_UPDATE;
		}

		if(optionopt && (ct_cursor(cmdp, CS_CURSOR_OPTION, NULL,
			CS_UNUSED, NULL, CS_UNUSED, optionopt) == CS_FAIL))
		{
			(CS_VOID)ct_cmd_drop(cmdp);
			SRV_FREE(bp);
			return(CS_FALSE);
		}
	}
	else
	{
		/* Define cursor declare command.			*/
		if(ct_cursor(cmdp, cdp->curcmd, cdp->curname, cdp->curnamelen,
			bp, size, opts) == CS_FAIL)
		{
			(CS_VOID)ct_cmd_drop(cmdp);
			SRV_FREE(bp);
			return(CS_FALSE);
		}
	}

	/*
	** Process the update clause if it exists.
	*/
	if(opts & CS_FOR_UPDATE)
	{
		if(gw_upclause(spp, cmdp, cdp) == CS_FAIL)
		{
			(CS_VOID)ct_cmd_drop(cmdp);
			SRV_FREE(bp);
			return(CS_FALSE);
		}
	}

	/*
	** See if there are any parameters associated with this cursor declare.
	*/
	if(srv_numparams(spp, &numparams) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}

	/*
	** Describe each of the parameters to CT-Library.
	*/
	for(i = 0; i < numparams; i++)
	{
		/*
		** Read the client's description of the parameter.
		*/
		if(srv_descfmt(spp, CS_GET, SRV_CURDATA, (i + 1), &fmt)
			== CS_FAIL)
		{
			ok = CS_FALSE;
			break;
		}

		/*
		** Describe the parameter to the remote server.
		*/
		if(ct_param(cmdp, &fmt, NIL(CS_VOID *), (CS_INT)0, 
			(CS_SMALLINT)0) == CS_FAIL)
		{
			ok = CS_FALSE;
			break;
		}
	}

	/*
	** Check for errors while describing parameters.
	*/
	if(!ok)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}

	/* Send cursor define command to remote server.			*/
	if(gw_ctsend(spp, cmdp) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}

	/* Process results from remote server.				*/
	if(gw_handleresults(spp, cmdp, cdp->curcmd) < 0)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}

	/* Configure connection so we get KEY data streams from server.	*/
	if(ct_cmd_props(cmdp, CS_SET, CS_HIDDEN_KEYS, &keys, CS_UNUSED,
		NIL(CS_INT *)) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}

	/*
	** Get a new cursor ID for the cursor being defined.
	*/
	if(gw_mancurid(spp, GW_GETCURID, cmdp, &curid, NIL(CS_INT *)) < 0)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}

	/* Set the row fetch count for this cursor.			*/
	if(gw_mancurid(spp, GW_SETFETCH, NIL(CS_COMMAND *), &curid,
		&(cdp->fetchcnt)) < 0)
	{
		(CS_VOID)gw_mancurid(spp, GW_RELCURID, cmdp, &curid,
			NIL(CS_INT *));
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}

	/* Send cursor ID back to client.				*/
	cdp->curid = curid;

	if(srv_cursor_props(spp, CS_SET, cdp) == CS_FAIL)
	{
		(CS_VOID)gw_mancurid(spp, GW_RELCURID, cmdp, &curid,
			NIL(CS_INT *));
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}

	/* Get pointer to cursor control structure.			*/
	if((cp = gw_getcurp(spp, curid)) == NIL(CURCTL_T *))
	{
		(CS_VOID)gw_mancurid(spp, GW_RELCURID, cmdp, &curid,
			NIL(CS_INT *));
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(bp);
		return(CS_FALSE);
	}

	/*
	** Save the new cursor information (in case client asks for
	** cursor status).
	*/
	cp->curid = curid;
	cp->curstatus |= CS_CURSTAT_DECLARED;
	if(opts & CS_FOR_UPDATE)
	{
		cp->curstatus |= CS_CURSTAT_UPDATABLE;
	}
	else
	{
		cp->curstatus |= CS_CURSTAT_RDONLY;
	}

	/*
	** Deallocate cursor select string buffer.
	*/
	SRV_FREE(bp);
	return(CS_TRUE);
}

/*
**  GW_CURINFO
**
**	Process cursor information commands.  This routine just updates
**	the number of rows to return on a fetch for this cursor.
**	This routine does no state validation of cursor commands.  
**	All state validatation is assumed to be done by the client library.  
**	It is the caller's responsibility to send the DONE response to the 
**	client's cursor command request based on the return value of this 
**	routine.
**
**  Parameters:
**	spp		Thread control structure.
**	cdp		Cursor control structure.
**	opts		Cursor command options received from client.
**
**  Returns:
**	CS_TRUE 	cursor command was processed successfully.
**	CS_FALSE 	an error occurred while processing cursor command.
*/
CS_BOOL		gw_curinfo(spp, cdp, opts)
SRV_PROC	*spp;
SRV_CURDESC	*cdp;
CS_INT		opts;
{
	CS_BOOL		ok;	/* Error control flag.			*/
	CS_INT		curid;	/* Current cursor ID.			*/
	CS_COMMAND	*cmdp;	/* Command handle.			*/
	CURCTL_T	*cp;	/* Current cursor control structure.	*/
	SRV_CURDESC	tmpcdp;	/* Cursor information.			*/
	CS_INT		i;

	/* Initialization.						*/
	ok = CS_TRUE;			/* All okay.			*/

	/* Get the current cursor context.				*/
	if((cmdp = gw_getcontext(spp, &curid)) == NIL(CS_COMMAND *))
	{
		return(CS_FALSE);
	}
		
	/*
	** If the client is requesting cursor status, step through
	** cursor list and send info for all cursors present currently.
	*/
	if(cdp->cmdoptions == SRV_CUR_ASKSTATUS)
	{
		for(i = 0; i < GW_MAXCURSORS; i++)
		{
			/* Get pointer to cursor control structure.	*/
			if(((cp = gw_getcurp(spp, i)) != NIL(CURCTL_T *)) 
				&& (cp->cmdp != NIL(CS_COMMAND *)) )
			{
				/*
				** fill up a CURDESC with information 
				** from the cursor control structure.
				**/
				tmpcdp.curid = cp->curid;
				tmpcdp.fetchcnt = cp->fetchcnt;
				tmpcdp.curstatus = cp->curstatus;
				tmpcdp.cmdoptions = SRV_CUR_INFORMSTATUS;

				/*
				** Send cursor information to the client.
				*/
				if(srv_cursor_props(spp, CS_SET, &tmpcdp)
					== CS_FAIL)
				{
					ok = CS_FALSE;
					break;
				}
			}
		}
		return(ok);
	}

	/* Define cursor command.					*/
	if(ok && (ct_cursor(cmdp, CS_CURSOR_ROWS, NIL(CS_CHAR *), CS_UNUSED,
		NIL(CS_CHAR *), CS_UNUSED, cdp->fetchcnt) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/*
	** Send the cursor command to the SQL Server.
	*/
	if(ok && (gw_ctsend(spp, cmdp) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Read results from server.					*/
	if(ok && (gw_handleresults(spp, cmdp, cdp->curcmd) < 0))
	{
		ok = CS_FALSE;
	}

	/* Set the current fetch row option for this cursor.		*/
	if(ok && (gw_mancurid(spp, GW_SETFETCH, NIL(CS_COMMAND *), &curid,
		&cdp->fetchcnt) < 0)) 
	{
		ok = CS_FALSE;
	}

	/* Identify cursor to client.					*/
	if(ok && (srv_cursor_props(spp, CS_SET, cdp) == CS_FAIL)) 
	{
		ok = CS_FALSE;
	}

	/* Get pointer to cursor control structure.			*/
	if(ok && ((cp = gw_getcurp(spp, curid)) == NIL(CURCTL_T *)))
	{
		ok = CS_FALSE;
	}

	if (!ok)
	{
		(CS_VOID)gw_mancurid(spp, GW_RELCURID, cmdp, &curid,
			NIL(CS_INT *));
	}
	else
	{
		/*
		** Save the new cursor infomation (in
		** case client asks for cursor status).
		*/
		cp->curstatus |= CS_CURSTAT_ROWCOUNT;
	}

	return(ok);
}

/*
** GW_CUROPEN
**
**	Process cursor open commands.  This routine does no state validation of
**	cursor commands.  All state validatation is assumed to be done by 
**	the client library.  It is the caller's responsibility to send the
**	DONE response to the client's cursor open request based on the 
**	return value of this routine.  Any required intermediate DONEs
**	are sent by this routine.
**
**  Parameters:
**	spp		Thread control structure.
**	cdp		Cursor information received from client.
**	opts		Cursor command options received from client.
**
**  Returns:
**	CS_TRUE 	Open command was processed successfully.
**	CS_FALSE 	An error occurred while processing cursor open command.
*/
CS_BOOL		gw_curopen(spp, cdp, opts)
SRV_PROC	*spp;
SRV_CURDESC	*cdp;
CS_INT		opts;
{
	CS_COMMAND	*cmdp;		/* Cursor command structure	*/
	CS_INT		curid;		/* Current cursor ID		*/
	CURCTL_T	*cp;		/* Current cursor control structure.*/

	CS_BOOL		ok;		/* Error control flag.			*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All okay.			*/

	/* Get command handle and cursor ID for current cursor.		*/
	if((cmdp = gw_getcontext(spp, &curid)) == NIL(CS_COMMAND *))
	{
		return(CS_FALSE);
	}

	/* Define cursor open command. 					*/
	if(ok && (ct_cursor(cmdp, cdp->curcmd, NIL(CS_CHAR *), CS_UNUSED,
		NIL(CS_CHAR *), CS_UNUSED, opts) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Process any parameters.					*/
	if(ok && (gw_clientparam(spp, cmdp, SRV_CURDATA) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Read results from server.					*/
	if(ok && (gw_handleresults(spp, cmdp, cdp->curcmd) < 0))
	{
		ok = CS_FALSE;
	}

	/* Identify cursor to client.					*/
	if(ok && (srv_cursor_props(spp, CS_SET, cdp) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Get pointer to cursor control structure.			*/
	if(ok && ((cp = gw_getcurp(spp, curid)) == NIL(CURCTL_T *)))
	{
		ok = CS_FALSE;
	}

	if (!ok)
	{
		(CS_VOID)gw_mancurid(spp, GW_RELCURID, cmdp, &curid,
			NIL(CS_INT *));
	}
	else
	{
		/*
		** Save the new cursor information (in
		** case client asks for cursor status).
		*/
		cp->curstatus |= CS_CURSTAT_OPEN;
	}

	return(ok);
}

/*
** GW_CURFETCH
**
**	Process cursor fetch commands.  This routine does no state validation 
**	of cursor commands.  All state validatation is 
**	assumed to be done by the client library.  It is the caller's 
**	responsibility to send the DONE response to the client's cursor command
**	request based on the return value of this routine.
**
**  Parameters:
**	spp		Thread control structure.
**	cdp		Cursor information recieved from client.
**	opts		Cursor command options received from client.
**	rowp		The number of rows fetched is returned here.
**
**  Returns:
**	CS_TRUE 	Cursor command was processed successfully.
**	CS_FALSE 	An error occurred while processing cursor command.
*/
CS_BOOL		gw_curfetch(spp, cdp, opts, rowp)
SRV_PROC	*spp;
SRV_CURDESC	*cdp;
CS_INT		opts;
CS_INT		*rowp;
{
	CS_COMMAND	*cmdp;		/* Command handle.		*/
	CS_INT		curid;		/* Current cursor ID.		*/
	CS_INT		tmprows;	/* Number of rows processed.	*/
	CS_BOOL		ok;	/* Error control flag.			*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All okay.			*/
	*rowp = (CS_INT)0;

	/* Get command handle and cursor ID for current cursor.		*/
	if((cmdp = gw_getcontext(spp, &curid)) == NIL(CS_COMMAND *))
	{
		return(CS_FALSE);
	}

	/*
	** Just read more rows from remote server.
	** We're currently in the middle of results processing
	** so we don't call gw_handleresults here.
	*/
	if((tmprows = gw_procrows(spp, cmdp, cdp->curcmd)) < 0)
	{
		ok = CS_FALSE;
		(CS_VOID)gw_mancurid(spp, GW_RELCURID, cmdp, &curid,
			NIL(CS_INT *));
	}
	else
	{
		/*
		** Return the number of rows fetched.
		*/
		*rowp = tmprows;
	}

	return(ok);
}

/*
** GW_CURUPDATE
**
**	Process cursor update commands.  This routine does no state validation 
**	of cursor commands.  All state validatation is assumed to be done by 
**	the client library.  It is the caller's responsibility to send the 
**	DONE response to the client's cursor command request based on the 
**	return value of this routine.
**
**  Parameters:
**	spp		Thread control structure.
**	cdp		Cursor control information received from client.
**	opts		Cursor command options received from client.
**
**  Returns:
**	CS_TRUE 	Cursor command was processed successfully.
**	CS_FALSE 	An error occurred while processing cursor command.
*/
CS_BOOL		gw_curupdate(spp, cdp, opts)
SRV_PROC	*spp;
SRV_CURDESC	*cdp;
CS_INT		opts;
{
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_CHAR		*bp;		/* Language buffer.		*/
	CS_COMMAND	*cmdp;		/* Command handle.		*/
	CS_INT		curid;		/* Current cursor ID.		*/
	CURCTL_T	*cp;		/* Current cursor control structur */
	CS_INT		nokeys;		/* Number of key columns.	*/
	CS_INT		nocols;		/* Number of columns in row.	*/
	CS_INT		*kmp;		/* Local pointer to keymap.	*/
	CS_INT		i;		/* Loop control variable.	*/
	CS_DATAFMT	*fmtp;		/* Parameter formats.		*/
	CS_INT		size;		/* Memory size variable.	*/
	CS_INT		*lenp;		/* Data length space.		*/
	CS_BYTE		*dp[GW_MAXCOL];	/* Key column data.		*/
	CS_SMALLINT	*indp;		/* Null indicator.		*/
	CS_INT		type;
	CS_INT		numparams;

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	i = 1;				/* Parameters start at 1.	*/
	bp = NIL(CS_CHAR *);
	lenp = NIL(CS_INT *);		/* No length array yet.		*/
	indp = NIL(CS_SMALLINT *);	/* No null indicators yet.	*/
	cp = NIL(CURCTL_T *);		/* No valid control structure.	*/
	fmtp = NIL(CS_DATAFMT *);	/* No format descriptors yet.	*/
	type = SRV_KEYDATA;
	srv_bzero((CS_VOID *)dp, CS_SIZEOF(dp));

	/* Get command handle and cursor ID for current cursor.		*/
	if((cmdp = gw_getcontext(spp, &curid)) == NIL(CS_COMMAND *))
	{
		return(CS_FALSE);
	}

	/* Determine the number of columns in the results rows.		*/
	if(ct_res_info(cmdp, CS_NUMDATA, (CS_VOID *)&nocols, CS_SIZEOF(nocols),
		NIL(CS_INT *)) == CS_FAIL)
	{
		ok = CS_FALSE;
	}

	/* Make sure we don't exceed our storage.			*/
	if(ok && (nocols > GW_MAXCOL))
	{
		ok = CS_FALSE;
	}

	/* Get pointer to cursor control structure.			*/
	if(ok && ((cp = gw_getcurp(spp, curid)) == NIL(CURCTL_T *)))
	{
		ok = CS_FALSE;
	}

	/*
	** Process each key column.
	*/
	nokeys = cp->nokeys;
	kmp = cp->keymap;

	/* Calculate memory size of CS_DATAFMT array.			*/
	size = nokeys * CS_SIZEOF(CS_DATAFMT);

	/* Allocate space for the data format structures		*/
	if(ok && ((fmtp = (CS_DATAFMT *)srv_alloc(size)) == NIL(CS_DATAFMT *)))
	{
		ok = CS_FALSE;
	}

	/* Clear the CS_DATAFMT space.					*/
	srv_bzero((CS_VOID *)fmtp, size);

	/* Allocate space for the data length array.			*/
	if(ok && ((lenp = (CS_INT *)srv_alloc(size)) == NIL(CS_INT *)))
	{
		SRV_FREE(fmtp);
		ok = CS_FALSE;
	}

	for(i = 0; ok && (i < nokeys); i++)
	{
		/* Get a description of the key.			*/
		if(srv_descfmt(spp, CS_GET, type, kmp[i], &(fmtp[i]))
			== CS_FAIL)
		{
			ok = CS_FALSE;
			break;
		}

		/* We're only looking for key columns.			*/
		if (!(fmtp[i].status & CS_KEY))
		{
			continue;
		}

		/* Allocate space for parameter.			*/
		if((dp[i] = (CS_BYTE *)srv_alloc((fmtp[i]).maxlength))
			== NIL(CS_BYTE *))
		{
			ok = CS_FALSE;
			break;
		}

		/* Bind allocated space to format description.		*/
		if(srv_bind(spp, CS_GET, type, kmp[i], &(fmtp[i]),
			(CS_VOID *)dp[i], &(lenp[i]), NIL(CS_SMALLINT *))
			== CS_FAIL)
		{
			ok = CS_FALSE;
			break;
		}
	}

	/* Read key columns into allocated program variables.		*/
	if(ok && (srv_xferdata(spp, CS_GET, type) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	for(i = 0; ok && (i < nokeys); i++)
	{
		/* We're only looking for key columns.			*/
		if(!(fmtp[i].status & CS_KEY))
		{
			continue;
		}

		/* Specify key column data to remote server.		*/
		if(ct_keydata(cmdp, CS_SET, kmp[i], dp[i], lenp[i], NULL)
			== CS_FAIL)
		{
			ok = CS_FALSE;
			break;
		}
	}

	if(ok) 
	{
		if (cdp->curcmd == CS_CURSOR_UPDATE)
		{
			/* Read the size of the cursor update string.	*/
			if((size = srv_langlen(spp)) < 0) 
			{
				ok = CS_FALSE;
			}

			/* Allocate a buffer for the cursor select string. */
			if(ok && (bp = (CS_CHAR *)srv_alloc(size + 1)) 
				== NIL(CS_CHAR *))
			{
				ok = CS_FALSE;
			}

			/* Copy update string into allocated buffer.	*/
			if(ok && (srv_langcpy(spp, (CS_INT)0, (CS_INT)(-1), bp)
				< 0))
			{
				ok = CS_FALSE;
			}
		}
		else
		{
			size = CS_UNUSED;
		}
	}

	/* Define cursor command.					*/
	if(ok && (ct_cursor(cmdp, cdp->curcmd, cdp->tabname, cdp->tabnamelen, 
		bp, size, opts) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Check for cursor update values.				*/
	if(ok && (srv_numparams(spp, &numparams) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	if(ok && (numparams > 0))
	{
		/* Calculate space needed for parameters.		*/
		size = numparams * CS_SIZEOF(CS_DATAFMT);

		/* Allocate space for the data format structures.	*/
		if(fmtp != NIL(CS_DATAFMT *))
		{
			SRV_FREE(fmtp);
		}

		if((fmtp = (CS_DATAFMT *)srv_alloc(size)) == NIL(CS_DATAFMT *))
		{
			ok = CS_FALSE;
		}

		/* Calculate space needed for parameters.		*/
		size = numparams * CS_SIZEOF(CS_SMALLINT);

		/* Allocate space for NULL indicator.			*/
		if(ok && (indp = (CS_SMALLINT *)srv_alloc(size))
			== NIL(CS_SMALLINT *))
		{
			ok = CS_FALSE;
		}

		for(i = 0; ok && (i < numparams); i++)
		{
			/*
			** Read a description of the update value from the 
			** client.
			*/
			if(srv_descfmt(spp, CS_GET, SRV_CURDATA, (i + 1),
				&(fmtp[i])) == CS_FAIL)
			{
				ok = CS_FALSE;
				break;
			}

			/* Allocate space for parameter.		*/
			if((dp[i] = (CS_BYTE *)srv_alloc((fmtp[i]).maxlength)) 
				== NIL(CS_BYTE *))
			{
				ok = CS_FALSE;
				break;
			}

			if(srv_bind(spp, CS_GET, SRV_CURDATA, i + 1, &(fmtp[i]),
				(CS_VOID *)dp[i], &lenp[i], &indp[i])
				== CS_FAIL)
			{
				ok = CS_FALSE;
				break;
			}
		}

		/* Read in the update value				*/	
		if(ok && (srv_xferdata(spp, CS_GET, SRV_CURDATA) == CS_FAIL))
		{
			ok = CS_FALSE;
		}

		for(i = 0; ok && (i < numparams); i++)
		{
			/* Describe update value to the remote server.	*/
			if(ct_param(cmdp, &(fmtp[i]), (CS_VOID *)dp[i],
				lenp[i], indp[i]) == CS_FAIL)
			{
				ok = CS_FALSE;
				break;
			}
		}
	}

	/* Send cursor command to server.				*/
	if(ok && (gw_ctsend(spp, cmdp) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Read results from server.					*/
	if(ok && (gw_handleresults(spp, cmdp, cdp->curcmd) < 0))
	{
		ok = CS_FALSE;
	}

	/* Release all allocated memory.				*/
	if(fmtp != NIL(CS_DATAFMT *))
	{
		SRV_FREE(fmtp);
	}
	if(indp != NIL(CS_SMALLINT *))
	{
		SRV_FREE(indp);
	}
	if(lenp != NIL(CS_INT *))
	{
		SRV_FREE(lenp);
	}
	for(i = 0; i < nokeys; i++) 
	{
		if(dp[i] != NIL(CS_BYTE *))
		{
			SRV_FREE(dp[i]);
		}
	}
	/* Deallocate cursor select string buffer.			*/
	if(bp != NIL(CS_CHAR *))
	{
		SRV_FREE(bp);
	}

	if (!ok)
	{
		(CS_VOID)gw_mancurid(spp, GW_RELCURID, cmdp, &curid,
			NIL(CS_INT *));
	}

	return(ok);
}

/*
** GW_CUREND
**
**	Process cursor close and dealloc commands.  This routine 
**	does no state validation of cursor commands.  All state validatation is 
**	assumed to be done by the client library.  It is the caller's 
**	responsibility to send the DONE response to the client's cursor command
**	request based on the return value of this routine.
**
**  Parameters:
**	spp 		Thread control structure.
**	cdp		Cursor information received from client.
**	opts		Cursor command options received from client.
**
**  Returns:
**	CS_TRUE 	Cursor command was processed successfully.
**	CS_FALSE 	An error occurred while processing cursor command.
*/
CS_BOOL		gw_curend(spp, cdp, opts)
SRV_PROC	*spp;
SRV_CURDESC	*cdp;
CS_INT		opts;
{
	CS_COMMAND	*cmdp;	/* Command handle.			*/
	CS_INT		curid;	/* Current cursor ID.			*/
	CURCTL_T	*cp;	/* Current cursor control structure.	*/
	CS_BOOL		ok;	/* Error control flag.			*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All okay.			*/

	/* Get command handle and cursor ID for current cursor.		*/
	if((cmdp = gw_getcontext(spp, &curid)) == NIL(CS_COMMAND *))
	{
		return(CS_FALSE);
	}

	/* Define cursor command.					*/
	if(ok && (ct_cursor(cmdp, cdp->curcmd, NIL(CS_CHAR *), CS_UNUSED,
		NIL(CS_CHAR *), CS_UNUSED, opts) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Send cursor command to server.				*/
	if(ok && (gw_ctsend(spp, cmdp) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Read results from server.					*/
	if(ok && (gw_handleresults(spp, cmdp, cdp->curcmd) < 0))
	{
		ok = CS_FALSE;
	}

	/* Identify cursor to client.					*/
	if(ok && (srv_cursor_props(spp, CS_SET, cdp) == CS_FAIL)) 
	{
		ok = CS_FALSE;
	}

	/* Get pointer to cursor control structure.			*/
	if(ok && ((cp = gw_getcurp(spp, curid)) == NIL(CURCTL_T *)))
	{
		ok = CS_FALSE;
	}

	/*
	** Save the new cursor information (in case client asks for
	** cursor status).
	*/
	cp->curstatus |= CS_CURSTAT_CLOSED;

	/*
	** If this was a CS_CURSOR_DEALLOC command clear the current 
	** cursor context, release the cursor id, and drop the
	** command handle (?)
	*/
	if(ok && (opts == CS_DEALLOC))
	{
		(CS_VOID)gw_mancurid(spp, GW_RELCURID, cmdp, &curid,
			NIL(CS_INT *));
		(CS_VOID)ct_cmd_drop(cmdp);
	}

	if (!ok)
	{
		(CS_VOID)gw_mancurid(spp, GW_RELCURID, cmdp, &curid,
			NIL(CS_INT *));
	}

	return(ok);
}

/*
** GW_UPCLAUSE
**
**	Process update clause in a cursor declare command.  It is the 
**	calling routine's responsibility to cancel the current cursor
**	declare command if this routine detects an error.
**
**  Parameters:
**	spp		Thread control structure.
**	cmdp		Command handle for cursor declare command being
**			prepared for server.
**	cdp		Cursor information from client.
**  Returns:
**	CS_SUCCEED	The update clause was built successfully.
**	CS_FAIL		An error was detected while building the update
**			clause.
*/
CS_RETCODE	gw_upclause(spp, cmdp, cdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
SRV_CURDESC	*cdp;
{
	CS_INT		nocols;		/* Number of update columns.	*/
	CS_INT		colno;		/* Current column. 		*/
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_DATAFMT	fmt;		/* Data description structure.	*/
	CS_DATAFMT	tmpfmt;		/* Data description structure.	*/

	/* Initialization.						*/
	colno = 1;			/* Start at column 1.		*/
	ok = CS_TRUE;			/* All okay.			*/
	srv_bzero((CS_VOID *)&fmt, CS_SIZEOF(fmt));

	nocols = cdp->numupcols;

	/* Process each column in the update clause.			*/
	while(ok && nocols)
	{
		/* Get name of current column.				*/
		if(srv_descfmt(spp, CS_GET, SRV_UPCOLDATA, colno, &tmpfmt)
			== CS_FAIL)
		{
			ok  = CS_FALSE;
			continue;
		}

		/* Make sure that this column has a valid name.		*/
		if((tmpfmt.name == NIL(CS_CHAR *))
			|| (tmpfmt.namelen > CS_MAX_NAME))
		{
			ok = CS_FALSE;
			continue;
		}

		/* Update data description structure with column information.*/
		if(tmpfmt.namelen == CS_NULLTERM)
		{
			/* Make sure string will fit in format name buffer.*/
			if((CS_INT)strlen(tmpfmt.name)  > 
			   (CS_INT)(CS_MAX_NAME - 1))
			{
				ok = CS_FALSE;
				continue;
			}
			(CS_VOID)strncpy(fmt.name, tmpfmt.name, CS_MAX_NAME);
		}	
		else
		{
			(CS_VOID)strncpy(fmt.name, tmpfmt.name, tmpfmt.namelen);
		}
		fmt.namelen = tmpfmt.namelen;
		fmt.datatype = CS_CHAR_TYPE;
		fmt.status = CS_UPDATECOL;

		/* Add update parameter to cursor define command.	*/
		if(ct_param(cmdp, &fmt, (CS_VOID *)fmt.name, fmt.namelen, 0)
			== CS_FAIL)
		{
			ok = CS_FALSE;
			continue;
		}

		/* Adjust current and remaining column counts.		*/
		++colno;		
		--nocols;
	}
	return(ok ? CS_SUCCEED : CS_FAIL);
}

/*
**  GW_MANCURID
**
**	This routine manages cursor identifiers.  It maps command handles
**	and fetch row counts to cursor identifiers.
**
**  Parameters:
**	spp	The thread control structure.
**	action	Requested action.
**
**		   GW_GETCURID - Return a new cursor ID in curidp.
**		   GW_RELCURID - Release cursor ID in curidp.
**		   GW_SETFETCH - Set cursor row fetch count to rowcntp.
**		   GW_GETFETCH - Return cursor row fetch count in rowcntp.
**
**	cmdp	Command handle associated with this cursor ID.  This
**		argument is only valid if `action' is GW_GETCURID.
**	curidp	Cursor ID affected by this command.
**	rowcntp	The fetch row count for this cursor.  This argument is
**		only valid if `action' is SET_FETCH or GET_FETCH.
**
**  Returns:
**	 0	Requested operation was completed successfully.
**	-1	An error occurred during requested operation.
*/
CS_INT		gw_mancurid(spp, action, cmdp, curidp, rowcntp)
SRV_PROC	*spp;
CS_INT		action;
CS_COMMAND	*cmdp;
CS_INT		*curidp;
CS_INT		*rowcntp;
{
	CS_INT		i;		/* Loop control variable.	*/
	CS_INT		curid;		/* Local copy of cursor ID.	*/
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_BOOL		found;		/* CS_TRUE if cursor ID found.	*/
	CURCTL_T	*cp;		/* Cursor control structure.	*/
	static CS_INT	last = 1;	/* Last cursor index.		*/ 

	/* Initialization.						*/
	curid = *curidp;		/* Cache curid locally.		*/
	ok = CS_FALSE;			/* Assume command will fail.	*/
	found = CS_FALSE;		/* Cursor ID not found yet.	*/

	/* Process according to action.					*/
	switch((int)action)
	{
	case GW_RELCURID:
		if((cp = gw_getcurp(spp, curid)) != NIL(CURCTL_T *))
		{
			(CS_VOID)ct_cmd_drop(cp->cmdp);
			cp->cmdp = NIL(CS_COMMAND *);
			cp->fetchcnt = -1;
			ok = CS_TRUE;
		}
		break;

	case GW_GETCURID:
		/*
	 	** Scan cursor map array between last and GW_MAXCURSORS looking
	 	** for an available cursor ID slot.
	 	*/
		for(i = last; i < GW_MAXCURSORS; i++)
		{
			/*
		 	** If slot free in map array, save cmdp and return
		 	** slot index to caller.  Update `last' so we start
		 	** looking right after this one next time through.
		 	*/
			if((cp = gw_getcurp(spp, i)) != NIL(CURCTL_T *))
			{
				if(cp->cmdp == NIL(CS_COMMAND *))
				{
					/* Check for wrap-around.	*/
					last = ((i+1) >= GW_MAXCURSORS) ? 0 : i;
					found = CS_TRUE;
					break;
				}
			}
		}

		/*
	 	** If we didn't find a cursor ID above, scan map array between
	 	** 0 and `last'.  If `last'is equal to zero we scanned the whole
	 	** map array above and there are no available slots.
	 	*/
		if(!found && (last != 1))
		{
			for(i = 1; i < last; i++)
			{
				/*
		 		** if slot free in map array, save cmdp and 
				** return slot index to caller.  Update `last' 
				** so we start looking right after this one 
				** next time through.
		 		*/
				if((cp = gw_getcurp(spp, i)) != NIL(CURCTL_T *))
				{
					if(cp->cmdp == NIL(CS_COMMAND *))
					{
						found = CS_TRUE;
						break;
					}
				}
			}
		}
		/* If we found a slot return index to caller and save cmdp*. */
		if(found)
		{
			*curidp = i;
			cp->cmdp = cmdp;
			ok = CS_TRUE;
		}
		break;

	case GW_SETFETCH:
		if((cp = gw_getcurp(spp, curid)) != NIL(CURCTL_T *))
		{
			cp->fetchcnt = *rowcntp;
			ok = CS_TRUE;
		}
		break;

	case GW_GETFETCH:
		if((cp = gw_getcurp(spp, curid)) != NIL(CURCTL_T *))
		{
			*rowcntp = cp->fetchcnt;
			ok = CS_TRUE;
		}
		break;

	default:
		ok = CS_FALSE;
		break;
	}
	return(ok ? (CS_INT)0 : (CS_INT)(-1));
}

/*
** GW_GETCONTEXT
**
**	This routine gets the current cursor context.  The command handle
**	associated with this cursor context is returned to the caller along
**	with the current cursor ID.
**
**  Parameters:
**	spp		Thread control structure.
**	curidp	 	The current cursor ID is returned here.
**
**  Returns:
**	cmdp			A valid cursor command handle if action was
**				completed successfully.
**	NIL(CS_COMMAND *)	The current cursor context could not be 
**				determined.
*/
CS_COMMAND	*gw_getcontext(spp, curidp)
SRV_PROC	*spp;
CS_INT		*curidp;
{
	CS_COMMAND	*cmdp;		/* Current cursor's cmd handle.	*/
	CURCTL_T	*cp;		/* Cursor control structure.	*/
	SRV_CURDESC	cdp;		/* Current cursor.		*/

	/* Initialization.						*/
	cmdp = NIL(CS_COMMAND *);	/* No cursor cmdp yet.				*/
	cp = NIL(CURCTL_T *);		/* No cursor control structure yet.*/

	/* Get current cursor context.					*/
	if(srv_cursor_props(spp, CS_GET, &cdp) == CS_SUCCEED)
	{
		if((cp = gw_getcurp(spp, cdp.curid)) != NIL(CURCTL_T *))
		{
			cmdp = cp->cmdp;
			*curidp = cp->curid;
		}
	}
	return(cmdp);
}

/*
** GW_GETCURP
**
**	This routine returns a pointer to the cursor control structure
**	for the spp and curid.
**
**  Parameters:
**	spp		Thread control structure.
**	curid	 	The cursor ID to for which to return the control 
**			structure. 
**
**  Returns:
**	 cp		A valid pointer to a cursor control structure for
**			spp and curid.
**	NIL(CURCTL_T *)	There is no active cursor control structure for 
**			spp/curid.
*/
CURCTL_T	*gw_getcurp(spp, curid)
SRV_PROC	*spp;
CS_INT		curid;
{
	CURCTL_T	*cp;		/* Cursor control structure.	*/
	DBMSCTL_T	*rp;		/* DBMS control structure.	*/

	/* Initialization.						*/
	cp = NIL(CURCTL_T *);

	/* Get user data from SRV_PROC structure.			*/
	if((rp = (DBMSCTL_T *)srv_getuserdata(spp)) != NIL(DBMSCTL_T *))
	{
		/* Make sure cursor ID is valid.			*/
		if((curid >= 0) && (curid < GW_MAXCURSORS))
		{
			cp = &(rp->actcur[curid]);
		}
	}
	return(cp);
}

/*
** GW_DYNHANDLER
**
**	Process dynamic SQL events.  SRV_DYNAMIC events are generated when a
**	client sends a dynamic SQL command.  This routine just determines
**	the command type and then calls another routine to do the
**	actual work for that command.  The client's request is
**	responded to with a DONE by this routine based on success or
**	failure of processing the command.
**
**  Parameters:
**	spp		Thread control structure.
**
**  Returns:
**	CS_SUCCEED	Command was processed successfully.
**	CS_FAIL		Command failed.
*/
CS_RETCODE	CS_PUBLIC gw_dynhandler(spp)
SRV_PROC	*spp;
{
	CS_INT		optype;		/* Client's dynamic operation.	*/
	CS_CHAR		msg[1024];	/* Error message buffer.	*/
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_INT		outlen;

	/* Initialization.						*/
	ok = CS_TRUE;

	/* Get dynamic command sent by client.				*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_TYPE, (CS_VOID *)&optype,
		CS_SIZEOF(optype), &outlen) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Process according to cursor command type.			*/
	switch((int)optype)
	{
	case CS_PREPARE:
	case CS_EXEC_IMMEDIATE:
		ok = gw_dynprepare(spp, optype);
		break;

	case CS_EXECUTE:
		ok = gw_dynexecute(spp, optype);
		break;

	case CS_DEALLOC:
		ok = gw_dyndealloc(spp, optype);
		break;

	case CS_DESCRIBE_INPUT:
	case CS_DESCRIBE_OUTPUT:
		ok = gw_dyndesc(spp, optype);
		break;

	default:
		SPRINTF(msg, "gw_dynhandler: Unknown dynamic optype(0x%x)\n",
			optype);
		SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);
		ok = CS_FALSE;
		break;
	}

	if(!ok)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Send the final done to the client.  We don't check the return
	** code here because Open Server will kill this thread if a final
	** done isn't sent.
	*/
	(CS_VOID)srv_senddone(spp, SRV_DONE_FINAL, CS_TRAN_UNDEFINED,
		(CS_INT)0);
	return(CS_SUCCEED);
}

/*
** GW_DYNPREPARE
**
**	Process dynamic sql PREPRARE or EXEC_IMMED commands.  This routine
**	does no state validation of commands.  All state validatation is 
**	assumed to be done by the client library.  It is the caller's 
**	responsibility to send the final DONE response to the client's command
**	based on the return value of this routine.  Any required intermediate 
**	DONEs are sent by this routine.
**
**  Parameters:
**	spp		Thread control structure.
**	optype		Dynamic SQL command received from client.
**
**  Returns:
**	CS_TRUE		Command was processed successfully.
**	CS_FALSE	An error occurred while processing cursor command.
*/
CS_BOOL		gw_dynprepare(spp, optype)
SRV_PROC	*spp;
CS_INT		optype;
{
	CS_COMMAND	*cmdp;		/* Declare command handle.	*/
	CS_INT		outlen;		/* Data read from client.	*/
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_CONNECTION	*chp;		/* Server connection handle.	*/
	CS_INT		idlen;		/* Dynamic SQL statement ID length.*/
	CS_CHAR		*stmtid;	/* Dynamic SQL statement ID	 */
	CS_INT		stmtlen;	/* Dynamic SQL statement length.*/
	CS_CHAR		*stmt;		/* Dynamic SQL statement. 	*/
	CS_INT		send_optype;	/* PROCNAME.			*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	stmtid = NIL(CS_CHAR *);
	stmt = NIL(CS_CHAR *);

	/* Get connection handle for this server connection.		*/
	if((chp = gw_getchp(spp, CS_FALSE)) == NIL(CS_CONNECTION *))
	{
		return(CS_FALSE);
	}

	/* Allocate a command handle for dynamic SQL command.		*/
	if(ct_cmd_alloc(chp, &cmdp) == CS_FAIL)
	{
		return(CS_FALSE);
	}

	/* Read the id length. 						*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_IDLEN, (CS_VOID *)&idlen,
		CS_SIZEOF(idlen), &outlen) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		return(CS_FALSE);
	}

	if((idlen > 0) && ((stmtid = (CS_CHAR *)srv_alloc(idlen))
		== NIL(CS_CHAR *)) )
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FALSE);
	}

	/* Read the statement id.					*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_ID, (CS_VOID *)stmtid, idlen,
		&outlen) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Read the statement length.					*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_STMTLEN, (CS_VOID *)&stmtlen,
		CS_SIZEOF(stmtlen), &outlen) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	if((stmtlen > 0) && ((stmt = (CS_CHAR *)srv_alloc(stmtlen))
		== NIL(CS_CHAR *)))
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(stmtid);
		gw_senderror(spp);
		return(CS_FALSE);
	}

	/* Read the statement.						*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_STMT, (CS_VOID *)stmt, stmtlen,
		&outlen) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(stmt);
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Define prepare or exec-immediate command to remote server.	*/
	switch((int)optype)
	{
	case CS_PREPARE:
		if(ct_dynamic(cmdp, CS_PREPARE, stmtid, idlen, stmt, stmtlen)
			== CS_FAIL)
		{
			ok = CS_FALSE;
		}
		break;

	case CS_EXEC_IMMEDIATE:
		if(ct_dynamic(cmdp, CS_EXEC_IMMEDIATE, NIL(CS_CHAR *),
			CS_UNUSED, stmt, stmtlen) == CS_FAIL)
		{
			ok = CS_FALSE;
		}
		break;

	default:
		/* Can't happen. */
		ok = CS_FALSE;
		break;
	}

	/*
	** Check for an error in the above switch.
	*/
	if(!ok)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(stmt);
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/*
	** Send the prepare or exec-immediate command to remote server.
	*/
	if(gw_ctsend(spp, cmdp) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(stmt);
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Process results from remote server.				*/
	if(gw_handleresults(spp, cmdp, optype) < 0)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(stmt);
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Send prepared statement ID back to client.			*/
	send_optype = CS_ACK;

	if(srv_dynamic(spp, CS_SET, SRV_DYN_TYPE, (CS_VOID *)&send_optype,
		CS_SIZEOF(send_optype), &outlen) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		SRV_FREE(stmt);
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	if(optype == CS_PREPARE)
	{
		if(srv_dynamic(spp, CS_SET, SRV_DYN_IDLEN, (CS_VOID *)&idlen,
			CS_SIZEOF(idlen), &outlen) == CS_FAIL)
		{
			ok = CS_FALSE;
		}

		if(ok && (srv_dynamic(spp, CS_SET, SRV_DYN_ID,
			(CS_VOID *)stmtid, idlen, &outlen) == CS_FAIL))
		{
			ok = CS_FALSE;
		}
	}

	/*
	** If an error was detected drop the command handle.
	*/
	if(!ok) 
	{
		(CS_VOID)ct_cmd_drop(cmdp);
	}

	/* Store the statement ID and the command handle.		*/
	if(ok && (optype == CS_PREPARE) && (gw_mandynid(spp, GW_SAVEDYNID, cmdp,
		stmtid, idlen) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Deallocate statement id buffer.				*/
	SRV_FREE(stmtid);

	/* Deallocate statement buffer.					*/
	SRV_FREE(stmt);
	return(ok);
}

/*
** GW_DYNDESC
**
**	Process dynamic SQL DESCRIBE_[INPUT,OUTPUT] commands.  
**	This routine does no state validation of dynamic SQL 
**	commands.  All state validatation is assumed to be done by 
**	the client library.  It is the caller's responsibility to send the
**	DONE response to the client's command based on the 
**	return value of this routine.  Any required intermediate DONEs
**	are sent by this routine.
**
**  Parameters:
**	spp 		Thread control structure.
**	optype		Dynamic SQL command received from client.
**  Returns:
**	CS_TRUE 	Dynamic description was processed successfully.
**	CS_FALSE 	An error occurred while processing description.
*/
CS_BOOL		gw_dyndesc(spp, optype)
SRV_PROC	*spp;
CS_INT		optype;
{
	CS_COMMAND	*cmdp;		/* Cursor command structure.	*/
	CS_INT		idlen;		/* Dynamic SQL statement ID length.*/
	CS_CHAR		*stmtid;	/* Dynamic SQL statement ID. 	*/
	CS_INT		outlen;		/* Data read from client.	*/
	CS_INT		send_optype;	/* Dynamic command acknowledgement.*/
	CS_BOOL		ok;	/* Error control flag.			*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All okay.			*/

	/* Read the id length. 						*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_IDLEN, (CS_VOID *)&idlen,
		CS_SIZEOF(idlen), &outlen) == CS_FAIL)
	{
		return(CS_FALSE);
	}

	if((stmtid = (CS_CHAR *)srv_alloc(idlen)) == NIL(CS_CHAR *))
	{
		gw_senderror(spp);
		return(CS_FALSE);
	}

	/* Read the statement id.					*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_ID, (CS_VOID *)stmtid, idlen,
		&outlen) == CS_FAIL)
	{
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Get command handle and id for current dynamic SQL statement.	*/
	if((cmdp = gw_getdyncontext(spp, stmtid, &idlen)) == NIL(CS_COMMAND *))
	{
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Send prepared statement ID back to client.			*/
	send_optype = CS_ACK;

	if(srv_dynamic(spp, CS_SET, SRV_DYN_TYPE, (CS_VOID *)&send_optype,
		CS_SIZEOF(send_optype), &outlen) == CS_FAIL)
	{
		ok = CS_FALSE;
	}

	if(ok && (srv_dynamic(spp, CS_SET, SRV_DYN_IDLEN, (CS_VOID *)&idlen,
		CS_SIZEOF(idlen), &outlen) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	if(ok && (srv_dynamic(spp, CS_SET, SRV_DYN_ID, (CS_VOID *)stmtid, idlen,
		&outlen) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Define dynamic CS_DESCRIBE_[INPUT, OUTPUT]  command.		*/
	if(ok && (ct_dynamic(cmdp, optype, stmtid, idlen, NIL(CS_CHAR *), CS_UNUSED)
		== CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Send describe-input command to remote server.		*/
	if(ok && (gw_ctsend(spp, cmdp) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Process input formats from remote server.			*/
	if(ok && (gw_handleresults(spp, cmdp, optype) < 0)) 
	{
		ok = CS_FALSE;
	}

	if (!ok)
	{
		(CS_VOID)gw_mandynid(spp, GW_RELDYNID, cmdp, stmtid, idlen);
	}

	/* Deallocate statement id buffer.				*/
	SRV_FREE(stmtid);

	return(ok);
}

/*
** GW_DYNEXECUTE
**
**	Process dynamic SQL EXECUTE commands.  This routine does no state 
**	validation of dynamic SQL commands.  All state validatation is assumed 
**	to be done by the client library.  It is the caller's responsibility 
**	to send the DONE response to the client's command based on the 
**	return value of this routine.  Any required intermediate DONEs
**	are sent by this routine.
**
**  Parameters:
**	spp 		Thread control structure.
**	optype		Dynamic SQL command received from client.
**
**  Returns:
**	CS_TRUE 	Execute command was successful.
**	CS_FALSE 	An error occurred while processing execute command.
*/
CS_BOOL		gw_dynexecute(spp, optype)
SRV_PROC	*spp;
CS_INT		optype;
{
	CS_COMMAND	*cmdp;		/* Cursor command structure.	*/
	CS_INT		idlen;		/* Dynamic SQL statement ID length.*/
	CS_CHAR		*stmtid;	/* Dynamic SQL statement ID.	*/
	CS_INT		outlen;		/* Data read from client.	*/
	CS_INT		send_optype;	/* ACK				*/
	CS_BOOL		ok;		/* Error control flag.			*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All okay.			*/

	/* Read the id length. 						*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_IDLEN, (CS_VOID *)&idlen,
		CS_SIZEOF(idlen), &outlen) == CS_FAIL)
	{
		return(CS_FALSE);
	}

	if((stmtid = (CS_CHAR *)srv_alloc(idlen)) == NIL(CS_CHAR *))
	{
		gw_senderror(spp);
		return(CS_FALSE);
	}

	/* Read the statement id.					*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_ID, (CS_VOID *)stmtid, idlen,
		&outlen) == CS_FAIL)
	{
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Get command handle and id for current dynamic SQL statement.	*/
	if((cmdp = gw_getdyncontext(spp, stmtid, &idlen)) == NIL(CS_COMMAND *))
	{
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Define dynamic EXECUTE command.				*/
	if(ct_dynamic(cmdp, CS_EXECUTE, stmtid, idlen, NIL(CS_CHAR *),
		CS_UNUSED) == CS_FAIL)
	{
		ok = CS_FALSE;
	}

	/* Process any parameters and then send.			*/
	if(ok && (gw_clientparam(spp, cmdp, SRV_DYNDATA) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Send acknowledgement back to client				*/	send_optype = CS_ACK;

	if(ok && (srv_dynamic(spp, CS_SET, SRV_DYN_TYPE, (CS_VOID *)&send_optype,
		CS_SIZEOF(send_optype), &outlen) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	if(ok && (srv_dynamic(spp, CS_SET, SRV_DYN_IDLEN, (CS_VOID *)&idlen,
		CS_SIZEOF(idlen), &outlen) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	if(ok && (srv_dynamic(spp, CS_SET, SRV_DYN_ID, (CS_VOID *)stmtid, idlen,
		&outlen) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Read results from server.					*/
	if(ok && (gw_handleresults(spp, cmdp, optype) < 0))
	{
		ok = CS_FALSE;
	}

	if (!ok)
	{
		(CS_VOID)gw_mandynid(spp, GW_RELDYNID, cmdp, stmtid, idlen);
	}

	/* Deallocate statement id buffer.				*/
	SRV_FREE(stmtid);
	return(ok);
}

/*
** GW_DYNDEALLOC
**
**	Process dynamic SQL DEALLOC commands.  This routine does no state 
**	validation of dynamic SQL commands.  All state validatation is assumed 
**	to be done by the client library.  It is the caller's responsibility 
**	to send the DONE response to the client's command based on the 
**	return value of this routine.  Any required intermediate DONEs
**	are sent by this routine.
**
**  Parameters:
**	spp 		Thread control structure.
**	optype		Dynamic SQL command received from client.
**  Returns:
**	CS_TRUE 	Dealloc was successful.
**	CS_FALSE 	An error occurred while processing deallocation.
*/
CS_BOOL		gw_dyndealloc(spp, optype)
SRV_PROC	*spp;
CS_INT		optype;
{
	CS_COMMAND	*cmdp;		/* Cursor command structure.	*/
	CS_INT		idlen;		/* Dynamic SQL statement ID length.*/
	CS_CHAR		*stmtid;	/* Dynamic SQL statement ID. 	*/
	CS_INT		send_optype;	/* ACK				*/
	CS_INT		outlen;		/* Size of token read in.	*/
	CS_BOOL		ok;	/* Error control flag.			*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All okay.			*/

	/* Read the id len. 						*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_IDLEN, (CS_VOID *)&idlen,
		CS_SIZEOF(idlen), &outlen) == CS_FAIL)
	{
		return(CS_FALSE);
	}

	if((stmtid = (CS_CHAR *)srv_alloc(idlen)) == NIL(CS_CHAR *))
	{
		gw_senderror(spp);
		return(CS_FALSE);
	}

	/* Read the statement id.					*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_ID, (CS_VOID *)stmtid, idlen,
		&outlen) == CS_FAIL)
	{
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Get command handle and id for current dynamic SQL statement.	*/
	if((cmdp = gw_getdyncontext(spp, stmtid, &idlen))
		== NIL(CS_COMMAND *))
	{
		SRV_FREE(stmtid);
		return(CS_FALSE);
	}

	/* Define dynamic DEALLOC command.				*/
	if(ct_dynamic(cmdp, CS_DEALLOC, stmtid, idlen, NIL(CS_CHAR *),
		CS_UNUSED) == CS_FAIL)
	{
		ok = CS_FALSE;
	}

	/* Send dynamic DEALLOC command	to remote server.		*/
	if(ok && (gw_ctsend(spp, cmdp) == CS_FAIL)) 
	{
		ok = CS_FALSE;
	}

	/* Read results from server.					*/
	if(ok && (gw_handleresults(spp, cmdp, optype) < 0))
	{
		ok = CS_FALSE;
	}

	/* Send acknowledgement back to client.				*/
	send_optype = CS_ACK;

	if(ok && (srv_dynamic(spp, CS_SET, SRV_DYN_TYPE, (CS_VOID *)&send_optype,
		CS_SIZEOF(send_optype), &outlen) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	if(ok && (srv_dynamic(spp, CS_SET, SRV_DYN_IDLEN, (CS_VOID *)&idlen,
		CS_SIZEOF(idlen), &outlen) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	if(ok && (srv_dynamic(spp, CS_SET, SRV_DYN_ID, (CS_VOID *)stmtid, idlen,
		&outlen) == CS_FAIL))
	{
		ok = CS_FALSE;
	}

	/* Remove dynamic statement id from active list.		*/
	if(gw_mandynid(spp, GW_RELDYNID, cmdp, stmtid, idlen) == CS_FAIL)
	{
		ok = CS_FALSE;
	}

	/* Deallocate statement id buffer.				*/
	SRV_FREE(stmtid);
	return(ok);
}

/*
** GW_HANDLERESULTS - Handle Results.
**
**	Once a command has been sent to a remote server this routine is
**	called to process any results returned from the server and
**	passes them back to the Open Server client.  It is the caller's 
**	responsibility to take any required error recovery based on the 
**	return value of this routine and to also send the final DONE required 
**	by the client.
**
**  Parameters:
**	spp		Thread control structure.
**	cmdp		The command handle for the language or RPC command.
**	cmd		The command that is generating this result set.
**
**  Returns:
**	Number rows	The total number of rows affected by all commands.
**	-1		An error was detected during results processing.
*/
CS_INT		gw_handleresults(spp, cmdp, cmd)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
CS_INT		cmd;
{
	CS_INT		rows;		/* Number or rows processed.	*/
	CS_INT		tmprows;	/* Tempory row count.		*/
	CS_BOOL		senddone;	/* Intermediate DONE control.	*/ 
	CS_INT		results;	/* Results from server.		*/
	CS_RETCODE	ret;		/* ct_results return code.	*/
	CS_BOOL		ok;		/* Error control flag.		*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All okay.			*/
	senddone = CS_FALSE;		/* No intermediate DONE needed.	*/
	rows = 0;			/* No rows read yet.		*/

	/*
	** Read results from server.  Process according to type.  All
	** results from the server are read unless an error is detected
	 */
	while(ok && ((ret = ct_results(cmdp, &results)) == CS_SUCCEED))
	{
		/*
		** If there is more than one result set, send the
		** required intermediate DONE to the client for the
		** previous result set.
		*/
		if(senddone && (results != CS_CMD_DONE))
		{
			if(srv_senddone(spp, SRV_DONE_MORE, CS_TRAN_UNDEFINED,
				(CS_INT)0) == CS_FAIL)
			{
				ok = CS_FALSE;
				continue;
			}
			senddone = CS_FALSE;
		}
		/* Process according to result type.			*/
		switch((int)results)
		{
		case CS_CURSOR_RESULT:
			/*
			** Cursor Open completed successfully.  Result
			** rows are pending.  Send intermediate DONE.
			*/
			ok = gw_procrowfmt(spp, cmdp, cmd);

			/*
			** We need to leave rather abruptly
			** here.  The fetches are deferred
			** until we read a CURFETCH stream
			** from the client.
			*/
			return(0);

		case CS_DESCRIBE_RESULT:
			ok = gw_procdynfmt(spp, cmdp, cmd);
			break;

		case CS_ROWFMT_RESULT:
			ok = gw_procrowfmt(spp, cmdp, cmd);
			break;

		case CS_ROW_RESULT:
			/* Process rows from server.			*/
			if((tmprows = gw_procrows(spp, cmdp, cmd)) < 0) 
			{
				ok = CS_FALSE;
			}
			else
			{
				rows += tmprows;
			}
			tmprows = 0;
			break;

		case CS_COMPUTEFMT_RESULT:
			ok = gw_proccrowfmt(spp, cmdp);
			break;

		case CS_COMPUTE_RESULT:
			/* Process compute rows from server.		*/
			if((tmprows = gw_proccrows(spp, cmdp)) < 0)
			{
				ok = CS_FALSE;
			}
			else
			{
				rows += tmprows;
			}
			tmprows = 0;
			break;

		case CS_PARAM_RESULT:
			/* Process RPC or message parameters.		*/
			if(gw_srvparam(spp, cmdp) == CS_FAIL)
			{
				ok = CS_FALSE;
			}
			break;

		case CS_STATUS_RESULT:
			/* Send return status to client.		*/
			if(gw_procstat(spp, cmdp) == CS_FAIL)
			{
				ok = CS_FALSE;
			}
			break;

		case CS_MSG_RESULT:
			/*
			** Send message ID to client.  Any parameters
			** associated with the message will be
			** handled in the CS_PARAM_RESULT case.
			*/
			if(gw_procmsg(spp, cmdp) == CS_FAIL)
			{
				ok = CS_FALSE;
			}
			break;

		case CS_CMD_DONE:
			/*
			** Done with one result set.  Release all allocated
			** format data structures and continue reading
			** results. However, we shouldn't release
			** the formats if we're processing a
			** cursor update/delete or dynamic SQL
			** format description, as there may be
			** another fetch to follow.
			*/
			if((cmd != CS_CURSOR_UPDATE) &&
				(cmd != CS_CURSOR_DELETE) &&
				(cmd != CS_DESCRIBE_INPUT) &&
				(cmd != CS_DESCRIBE_OUTPUT))
			{
				(CS_VOID)gw_managefmtp(spp, GW_CLEARFMTP,
					(CS_INT)0, (CS_INT)0,
					NIL(CS_DATAFMT **));
			}
			senddone = CS_TRUE;
			break;

		case CS_CMD_SUCCEED:
			/* Get the number of rows affected by current cmd. */
			if(ct_res_info(cmdp, CS_ROW_COUNT, (CS_VOID *)&tmprows, 
				CS_UNUSED, NIL(CS_INT *)) != CS_SUCCEED)
			{
				ok = CS_FALSE;
			}
			else if(tmprows > 0)
			{
				rows += tmprows;
			}
			tmprows = 0;
			break;

		case CS_CMD_FAIL:
			/* No rows returned from server.		*/
			rows = 0;
			break;

		default:
			/* Unknown or unsupported result code, give up.	*/
			ok = CS_FALSE;
			break;
		}
	}

	/* Check if ct_results returns CS_CANCELED.			*/
	if(ok && (ret == CS_CANCELED))
	{
		rows = 0;
	}
	/* Check for error from ct_results.				*/
	else if(ok && (ret != CS_END_RESULTS)) 
	{
		ok = CS_FALSE;
	}
	return(ok ? rows : (CS_INT)(-1));
}

/*
** GW_PROCDYNFMT
**
**	Read dynamic (input or output) parameter formats 
**	from a remote server and send them to a client.
**	All memory allocated in this routine is released before 
**	returning.
**
**  Parameters:
**	spp 		Thread control structure.
**	cmdp		The command handle for the language or RPC command.
**
**  Returns:
**	CS_TRUE		Format was processed without any error.
**	CS_FALSE	An error was detected during format processing.
*/
CS_BOOL		gw_procdynfmt(spp, cmdp, cmd)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
CS_INT		cmd;
{
	CS_INT		nocols;		/* Number of columns in row.	*/
	CS_INT		size;		/* Memory size for format array.*/
	CS_DATAFMT	*fmtp;		/* Format array pointer.	*/
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_INT		i;		/* Loop variable.		*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	fmtp = NIL(CS_DATAFMT *);	/* No format descriptors yet.	*/

	/* Determine the number of columns in the results rows.		*/
	if(ct_res_info(cmdp, CS_NUMDATA, (CS_VOID *)&nocols, CS_SIZEOF(nocols),
		NIL(CS_INT *)) != CS_SUCCEED)
	{
		return(CS_FALSE);
	}

	/* Calculate memory size of CS_DATAFMT array.			*/
	size = nocols * CS_SIZEOF(CS_DATAFMT);

	/* Allocate space for the data format structures.		*/
	if((size > 0) && ((fmtp = (CS_DATAFMT *)srv_alloc(size))
		== NIL(CS_DATAFMT *)))
	{
		gw_senderror(spp);
		return(CS_FALSE);
	}

	/* Clear the CS_DATAFMT space.					*/
	srv_bzero((CS_VOID *)fmtp, size);

	/* Get a description of the column data received from the server. */
	for(i = 0; i < nocols; i++)
	{
		/* Get a description of the column format.		*/
		if(ct_describe(cmdp, (i + 1), &(fmtp[i])) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}

		switch((int)cmd)
		{
		case CS_DESCRIBE_OUTPUT:
			fmtp[i].status |= CS_DESCOUT;
			break;

		case CS_DESCRIBE_INPUT:
			fmtp[i].status |= CS_DESCIN;
			break;

		default:
			ok = CS_FALSE;
			break;
		}

		if(!ok)
		{
			gw_senderror(spp);
			break;
		}

		/* Describe the data format to the Open Server Library.	*/
		if(srv_descfmt(spp, CS_SET, SRV_DYNDATA, (i + 1), &(fmtp[i]))
			!= CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
	}

	/*
	** If there was no error and there are columns, send the formats to the
	** client now.
	*/
	if(ok && (nocols > 0))
	{
		if(srv_xferdata(spp, CS_SET, SRV_DYNDATA) != CS_SUCCEED)
		{
			ok = CS_FALSE;
		}
	}

	/* Release allocated memory. 					*/
	SRV_FREE(fmtp);

	return(ok ? CS_TRUE : CS_FALSE);
}

/*
** GW_PROCROWFMT
**
**	Read ROW formats from a remote server and sned them to a client.
**	All memory allocated in this routine is released in gw_procrows.
**
**  Parameters:
**	spp		Thread control structure.
**	cmdp		The command handle for the language or RPC command.
**
**  Returns:
**	CS_TRUE		Format was processed without any error.
**	CS_FALSE	An error was detected during format processing.
*/
CS_BOOL		gw_procrowfmt(spp, cmdp, cmd)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
CS_INT		cmd;
{
	CS_INT		nocols;		/* Number of columns in row.	*/
	CS_INT		size;		/* Memory size for format array.*/
	CS_DATAFMT	*fmtp;		/* Format array pointer.	*/
	CS_BOOL		ok;		/* Error control flag.		*/
	int		i;		/* Loop variable.		*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is well.			*/

	/* Determine the number of columns in the results rows.		*/
	if(ct_res_info(cmdp, CS_NUMDATA, (CS_VOID *)&nocols, CS_UNUSED,
		NIL(CS_INT *)) != CS_SUCCEED)
	{
		return(CS_FALSE);
	}

	/* Calculate memory size of CS_DATAFMT array.			*/
	size = nocols * CS_SIZEOF(CS_DATAFMT);

	/* Allocate space for the data format structures.		*/
	if((fmtp = (CS_DATAFMT *)srv_alloc(size)) == NIL(CS_DATAFMT *))
	{
		gw_senderror(spp);
		return(CS_FALSE);
	}

	/* Clear the CS_DATAFMT space.					*/
	srv_bzero((CS_VOID *)fmtp, size);

	/* Get a description of the column data received from the server. */
	for(i = 0; i < nocols; i++)
	{
		/* Get a description of the column format.		*/
		if(ct_describe(cmdp, (i + 1), &(fmtp[i])) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}

		if (fmtp[i].datatype == CS_TEXT_TYPE)
			fmtp[i].status |= CS_CANBENULL;

		/* Describe the data format to the client.		 */
		if(srv_descfmt(spp, CS_SET, SRV_ROWDATA, (i+1), &(fmtp[i]))
			!= CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
	}
	/* Save fmtp for use by gw_procrows.				*/
	if(ok)
	{
		ok = gw_managefmtp(spp, GW_SETFMTP, GW_REGULARFMT, (CS_INT)0,
			&fmtp);
	}

	/* If there was an error, release the format array memory. 	*/
	if(!ok)
	{
		SRV_FREE(fmtp);
	}
	return(CS_TRUE);
}

/*
** GW_PROCROWS - Process Rows
**
**	Read regular row results from a remote server and return them to a 
**	client.  It is the caller's responsibility to send the required DONE 
**	based on the return code from this routine.  This routine sends no
**	DONEs to a client.
**
**  Parameters:
**	spp 		Thread control structure.
**	cmdp		The command handle for the language or RPC command.
**  Returns:
**	rows		The number of rows sent to a client.
**	-1		An error was detected during results processing.
*/
CS_INT		gw_procrows(spp, cmdp, cmd)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
CS_INT		cmd;
{
	CS_BOOL		ok;		/* Error control flag.		*/
	int		i;		/* Loop control variable.	*/
	CS_INT		nocols;		/* Number of columns in row.	*/
	CS_INT		actcols;	/* Actual number of columns processed.*/
	CS_INT		rows;		/* Number of rows sent.		*/
	CS_INT		cnt;		/* Argument for ct_fetch.	*/
	CS_DATAFMT	*fmtp;		/* Column formats.		*/
	CS_INT		curid;		/* Current cursor ID.		*/
	CS_INT		fetchcnt;	/* Number of rows to fetch in cursor.*/
	CS_SMALLINT	ind[GW_MAXCOL];	/* Bind indication.		*/
	CS_INT		len[GW_MAXCOL];	/* Length of column data.	*/
	CS_BYTE		*dp[GW_MAXCOL];	/* Column data.			*/
	CURCTL_T	*cp;		/* Cursor control structure.	*/
	CS_INT		mapindex;	/* Index into cursor map array.	*/
	CS_BOOL		done;		/* Loop control.		*/
	CS_RETCODE	ret;		/* ct_fetch return value.	*/
	CS_IODESC	docdesc;	/* Bulk I/O descriptor.		*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	rows = 0;			/* No rows fetched yet.		*/
	done = CS_FALSE;		/* Just starting!		*/
	actcols = 0;			/* No columns allocated yet.	*/
	mapindex = (CS_INT)0;

	if((cmd == CS_CURSOR_FETCH) && (NoMoreCurrows))
	{
		NoMoreCurrows = CS_FALSE;
		return(0);
	}

	/* Determine the number of columns in the results rows.		*/
	if(ct_res_info(cmdp, CS_NUMDATA, (CS_VOID *)&nocols, CS_UNUSED,
		NIL(CS_INT *)) != CS_SUCCEED)
	{
		return((CS_INT)(-1));
	}

	if(cmd != CS_CURSOR_FETCH)
	{
		/* Process any browse mode results.			*/
		if(ok)
		{
			ok = gw_dobrowse(spp, cmdp, nocols);
		}

		/* Process any orderby information.			*/
		if(ok)
		{
			ok = gw_orderby(spp, cmdp);
		}

		/* Process any control row information.			*/
		if(ok)
		{
			ok = gw_controlrow(spp, cmdp, nocols);
		}

		/* Make sure we don't exceed our storage.		*/
		if(nocols > GW_MAXCOL)
		{
			gw_senderror(spp);
			return((CS_INT)(-1));
		}
	}
	else
	{
		/* 
	 	** This is a CS_CURSOR_FETCH command, get the current cursor
	 	** context so we can find out how many rows to send to the 
		** client (current fetchcnt).
	 	*/
		if(gw_getcontext(spp, &curid) == NIL(CS_COMMAND *))
		{
			ok = CS_FALSE;
		}

		/* Get pointer to cursor control structure.		*/
		if(ok && (cp = gw_getcurp(spp, curid)) == NIL(CURCTL_T *))
		{
			ok = CS_FALSE;
		}

		if(ok)
		{
			fetchcnt = cp->fetchcnt;
		}
	}

	/* Get the format pointer saved by gw_procrowfmt.		*/
	if(!gw_managefmtp(spp, GW_GETFMTP, GW_REGULARFMT, (CS_INT)0, &fmtp))
	{
		gw_senderror(spp);
		return((CS_INT)(-1));
	}

	/* Make sure we have a valid fmtp.				*/
	if(fmtp == NIL(CS_DATAFMT *))
	{
		gw_senderror(spp);
		return((CS_INT)(-1));
	}

	/*
	** If there is only 1 column and it is text or image, call 
	** gw_proctextcol to "chunk" the column through Open Server.
	*/
	if((nocols == 1) && ((fmtp[0].datatype == CS_TEXT_TYPE) || 
		(fmtp[0].datatype == CS_IMAGE_TYPE)))
	{
		return(gw_proctextcol(spp, cmdp));
	}

	/*
	** Allocate space for the data, and bind the results to the allocated 
	** space.  Then describe the column data to the Open Server. 
	*/
	for(i = 0; i < nocols; i++)
	{
		/*
		** If cmd == CS_CURSOR_FETCH, save key column information
		** so we know which columns are keys when we receive
		** key information from client on cursor updates and 
		** deletes.
		*/
		if((cmd == CS_CURSOR_FETCH) && (fmtp[i].status & CS_KEY)) 
		{
			cp->keymap[mapindex++] = (i + 1);
		}

		/* Allocate space for the column data.			*/
		if((dp[i] = (CS_BYTE *)srv_alloc(fmtp[i].maxlength))
			== NIL(CS_BYTE *))
		{
			ok = CS_FALSE;
			break;
		}

		/* Save number of columns processed for freeing later.	*/
		actcols++;

		/* Bind the results to the allocated space.		*/
		if(ct_bind(cmdp, (i+1), &(fmtp[i]), (CS_VOID *)dp[i], &(len[i]),
			&(ind[i])) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}

		/* Bind the row data from the client.			*/
		if(srv_bind(spp, CS_SET, SRV_ROWDATA, i+1, &(fmtp[i]), 
			(CS_VOID *)dp[i], &(len[i]), &(ind[i])) 
			!= CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
	}
	/* Fetch rows from server and send them to client.		*/
	while(ok && !done)
	{
		/* Fetch a row.						*/
		if((ret = ct_fetch(cmdp, CS_UNUSED, CS_UNUSED, CS_UNUSED, &cnt))
			!= CS_SUCCEED)
		{
			if(ret == CS_FAIL)
			{
				ok = CS_FALSE;
				break;
			}
			if(cmd == CS_CURSOR_FETCH)
			{
				if(gw_handleresults(spp, cmdp, cmd) < 0)
				{
					ok = CS_FALSE;
				}
				else
				{
					done = CS_TRUE;
					if(rows > 0)
					{
						NoMoreCurrows = CS_TRUE;
					}
				}
				break;
			}
			done = CS_TRUE;
			continue;
		}

		/*
		** Check for text and image columns in row.  If there
		** are any, describe them to the Open Server. 
		*/
		for(i = 0; i < nocols; i++)
		{
			if((fmtp[i].datatype == CS_TEXT_TYPE) ||
				(fmtp[i].datatype == CS_IMAGE_TYPE))
			{
				if(ct_data_info(cmdp, CS_GET, i+1, &docdesc) 
					!= CS_SUCCEED)
				{
					ok = CS_FALSE;
					break;
				}
				if(srv_text_info(spp, CS_SET, (i+1), &docdesc)
					!= CS_SUCCEED)
				{
					ok = CS_FALSE;
					break;
				}
			}
		}

		/* Send the data to the client.				*/
		if(srv_xferdata(spp, CS_SET, SRV_ROWDATA) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			continue;
		}

		/* Increment count of successful rows sent.		*/ 
		rows++;

		/* Only send fetchcnt rows if CS_CURSOR_FETCH command	*/
		if((cmd == CS_CURSOR_FETCH) && (rows == fetchcnt))
		{
			done = CS_TRUE;
		}
	}

	/* Save number of valid keymap entries if CS_CURSOR_FETCH	*/
	if(cmd == CS_CURSOR_FETCH)
	{
		if(ok) 
		{
			cp->nokeys = mapindex;
		}
		else 
		{
			cp->nokeys = 0;
		}
	}

	/* Release allocated memory. 					*/
	for(i = 0; i < actcols; i++)
	{
		SRV_FREE(dp[i]);
	}
	return(ok ? rows : (CS_INT)(-1));
}

/*
** GW_PROCTEXTCOL
**
**	Read single-column rows of text or image data from a remote server 
**	and return them to a client.  It is the caller's responsibility to 
**	send the required DONE based on the return code from this routine.  
**	This routine sends no DONEs to a client.
**
**  Parameters:
**	spp 		Thread control structure.
**	cmdp		The command handle for the language or RPC command.
**
**  Returns:
**	rows		The number of rows sent to a client.
**	-1		An error was detected during results processing.
*/
CS_INT		gw_proctextcol(spp, cmdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
{
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_INT		rows;		/* Number of rows sent.		*/
	CS_INT		cnt;		/* Argument for ct_fetch.	*/
	CS_BOOL		done;		/* Loop control.		*/
	CS_RETCODE	ret;		/* ct_fetch return value.	*/
	CS_IODESC	docdesc;	/* Bulk IO descriptor.		*/
	CS_INT		len_read;	/* Amount of data read.		*/
	CS_BYTE		docbuf[GW_BULKBUFSIZE];/* Buffer for data.	*/
	CS_INT		howmuch;	/* To read from server.		*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	rows = 0;			/* No rows fetched yet.		*/
	done = CS_FALSE;		/* Just starting!		*/
	howmuch = GW_BULKBUFSIZE;	/* Buffer size.			*/

	/* Fetch rows from server and send them to client.		*/
	while(ok && !done)
	{
		/* Fetch a row.						*/
		if((ret = ct_fetch(cmdp,CS_UNUSED,CS_UNUSED,CS_UNUSED,&cnt))
			!= CS_SUCCEED)
		{
			if (ret == CS_FAIL)
				ok = CS_FALSE;
			else
				done = CS_TRUE;
			continue;
		}

		/* Get the length of the text column.			*/
		if((ret = ct_get_data(cmdp, 1, docbuf, 0, &len_read)) == CS_FAIL)
		{
			ok = CS_FALSE;
			break;
		}

		/* Describe column format to Open Server.		*/
		if(ct_data_info(cmdp, CS_GET, 1, &docdesc) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}

		if(srv_text_info(spp, CS_SET, 1, &docdesc) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}

		/* If the first ct_get_data() call returned 
		** CS_END_DATA, this is a NULL column
		*/
		if (ret == CS_END_DATA)
		{
			/* We still need to do a write			*/
			if(srv_send_text(spp, docbuf, len_read)!=CS_SUCCEED)
			{
				ok = CS_FALSE;
				break;
			}
			rows++;
			continue;
		}

		/* Read the text from the server.			*/
		while((ret = ct_get_data(cmdp, 1, docbuf, howmuch, &len_read))
			== CS_SUCCEED)
		{
			/* Write text to client a chunck at a time.	*/
			if(srv_send_text(spp, docbuf, len_read)!=CS_SUCCEED)
			{
				ok = CS_FALSE;
				break;
			}
		}

		/* Check for any errors from ct_get_data.		*/
		if(!((ret == CS_END_ITEM) || (ret == CS_END_DATA)))
		{
			ok = CS_FALSE;
			break;
		}

		/* if ct_get_data returns CS_CANCELED.			*/
		if(ret == CS_CANCELED)
		{
			done = CS_TRUE;
		}

		/* One last write.					*/
		if(!done && (srv_send_text(spp, docbuf, len_read)
			!= CS_SUCCEED))
		{
			ok = CS_FALSE;
			break;
		}
		/* Increment count of successful rows sent.		*/ 
		rows++;
	}
	return(ok ? rows : (CS_INT)(-1));
}

/*
** GW_PROCCROWFMT
**
**	Read compute row formats from a remote server and send them to a client.
**	All memory allocated in this routine is released in gw_proccrows().
**
**  Parameters:
**	spp 		Thread control structure.
**	cmdp		The command handle for the language or RPC command.
**
**  Returns:
**	CS_TRUE		Format was processed without any error.
**	CS_FALSE	An error was detected during format processing.
*/
CS_BOOL		gw_proccrowfmt(spp, cmdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
{
	CS_INT		nocols;			/* Number of columns in row.*/
	CS_INT		size;			/* Memory size for format   */
						/* array.		    */
	CS_INT		numbylist;		/* Number of order-by list  */
						/* columns.		    */
	CS_SMALLINT	bylist[GW_MAXCOL];	/* Array of order-by list   */
						/* columns.		    */
	CS_DATAFMT	*fmtp;			/* Format array pointer.*/
	CS_BOOL		ok;			/* Error control flag.	*/
	CS_INT		id;			/* Compute row identifier.*/
	CS_INT		op;			/* Column compute operator.*/
	CS_INT		operand;		/* Select-list argument for */
						/* Column compute operator.*/
	int		i;			/* Loop variable.	*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is well.			*/

	/* Get the compute identifier for this compute row.		*/
	if(ct_compute_info(cmdp, CS_COMP_ID, CS_UNUSED, (CS_VOID *)&id,
		CS_UNUSED, NIL(CS_INT *)) != CS_SUCCEED)
	{
		return(CS_FALSE);
	}

	/* Get the order-by list for this compute row, if any		*/
	if(ct_compute_info(cmdp, CS_COMP_BYLIST, CS_UNUSED, (CS_VOID *)bylist,
		CS_SIZEOF(bylist), &numbylist) != CS_SUCCEED)
	{
		return(CS_FALSE);
	}

	numbylist = numbylist/CS_SIZEOF(CS_SMALLINT);

	/* Define this compute row to the Open Server. */
	if(srv_alt_header(spp, id, numbylist, bylist) != CS_SUCCEED)
	{
		return(CS_FALSE);
	}

	/* Determine the number of columns in the compute row.		*/
	if(ct_res_info(cmdp, CS_NUMDATA, (CS_VOID *)&nocols, CS_UNUSED,
		NIL(CS_INT *)) != CS_SUCCEED)
	{
		return(CS_FALSE);
	}

	/* Calculate memory size of CS_DATAFMT array.			*/
	size = nocols * CS_SIZEOF(CS_DATAFMT);

	/* Allocate space for the data format structures.		*/
	if((fmtp = (CS_DATAFMT *)srv_alloc(size)) == NIL(CS_DATAFMT *))
		return(CS_FALSE);

	/* Clear the CS_DATAFMT space.					*/
	srv_bzero((CS_VOID *)fmtp, size);

	/* Get a description of the column data received from the server.*/
	for(i = 0; i < nocols; i++)
	{
		/* Get a description of the column format.		*/
		if(ct_describe(cmdp, (i + 1), &(fmtp[i])) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
		/* Get the compute operator for this column.		*/
		if(ct_compute_info(cmdp, CS_COMP_OP, (CS_INT)(i+1),
			(CS_VOID *)&op, CS_UNUSED, NIL(CS_INT *))
			!= CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
		/* Get the aggregate operand for this column.		*/
		if(ct_compute_info(cmdp, CS_COMP_COLID, (CS_INT)(i+1),
			(CS_VOID *)&operand, CS_UNUSED, 
			NIL(CS_INT *)) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
		/* Describe the column format to client.		*/
		if(srv_alt_descfmt(spp, id, op, (CS_TINYINT)operand, (i+1),
			&(fmtp[i])) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
	}
	/* Save fmtp for use by proccrows.				*/
	if(ok)
	{
		ok = gw_managefmtp(spp, GW_SETFMTP, GW_COMPUTEFMT, id, &fmtp);
	}

	/* If there was an error release the format array memory.	*/
	if(!ok)
	{
		SRV_FREE(fmtp);
	}
	return(ok);
}

/*
** GW_PROCCROWS
**
**	Process compute rows received from a remote server.
**	It is the caller's responsibility to send any required DONEs based
**	on the return code from this routine.  This routine does not
**	send any DONE to a client.
**
**  Parameters:
**	spp 		Thread control structure.
**	cmdp		The command handle that generated the compute row
**			result.
**  Returns:
**	rows		Number of compute rows sent to a client.
**	-1		An error was detected while processing compute rows.
*/
CS_INT		gw_proccrows(spp, cmdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
{
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_INT		nocols;		/* Number of compute columns.	*/
	CS_INT		rows;		/* Number of compute rows sent.	*/
	int		i;		/* Loop control variable.	*/
	CS_INT		actcols;	/* Actual number of columns	*/
					/* processed.			*/
	CS_INT		cnt;		/* Argument for ct_fetch.	*/
	CS_DATAFMT	*fmtp;		/* Column formats.		*/
	CS_SMALLINT	ind[GW_MAXCOL];	/* Bind indication.		*/
	CS_INT		len[GW_MAXCOL];	/* Length of column data.	*/
	CS_BYTE		*dp[GW_MAXCOL];	/* Column data.			*/
	CS_BOOL		done;		/* Loop control.		*/
	CS_RETCODE	ret;		/* ct_fetch return value.	*/
	CS_INT		id;		/* Compute row identifier.	*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All okay.			*/
	rows = 0;			/* None processed yet.		*/
	done = CS_FALSE;		/* Not done yet.		*/
	actcols = 0;			/* No columns allocated.	*/

	/* Get the compute identifier for this compute row.		*/
	if(ct_compute_info(cmdp, CS_COMP_ID, CS_UNUSED, (CS_VOID *)&id,
		CS_UNUSED, NIL(CS_INT *)) != CS_SUCCEED)
	{
		return((CS_INT)(-1));
	}

	/* Determine the number of columns in the compute row.		*/
	if(ct_res_info(cmdp, CS_NUMDATA, (CS_VOID *)&nocols, CS_UNUSED,
		NIL(CS_INT *)) != CS_SUCCEED)
	{
		return((CS_INT)(-1));
	}

	/* Make sure we don't exceed our storage.			*/
	if(nocols > GW_MAXCOL)
	{
		gw_senderror(spp);
		return((CS_INT)(-1));
	}

	/* Get the format pointer saved by gw_proccrowfmt.		*/
	if(!gw_managefmtp(spp, GW_GETFMTP, GW_COMPUTEFMT, id, &fmtp))
	{
		gw_senderror(spp);
		return((CS_INT)(-1));
	}

	/* Make sure we have a valid fmtp.				*/
	if(fmtp == NIL(CS_DATAFMT *))
	{
		gw_senderror(spp);
		return((CS_INT)(-1));
	}

	/*
	** Allocate space for the data, and bind the results to the allocated 
	** space.
	*/
	for(i = 0; i < nocols; i++)
	{
		/* Allocate space for the column data.			*/
		if((dp[i] = (CS_BYTE *)srv_alloc(fmtp[i].maxlength))
			== NIL(CS_BYTE *))
		{
			ok = CS_FALSE;
			break;
		}
		/* Save number of columns allocated for freeing later.	*/
		actcols++;

		/* Bind the results to the allocated space.		*/
		if(ct_bind(cmdp, (i+1), &(fmtp[i]), (CS_VOID *)dp[i], &(len[i]),
			&(ind[i])) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
		/* Bind the source data for each column.		*/
		if(srv_alt_bind(spp, id, (i+1), &(fmtp[i]), 
			(CS_VOID *)dp[i], &(len[i]), &(ind[i])) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
	}

	/* Fetch rows from server and send them to client.		*/
	while(ok && !done)
	{
		/* Fetch a row.						*/
		if((ret = ct_fetch(cmdp,CS_UNUSED,CS_UNUSED,CS_UNUSED,&cnt))
			!= CS_SUCCEED)
		{
			if (ret == CS_FAIL)
				ok = CS_FALSE;
			else
				done = CS_TRUE;
			continue;
		}
		/* Send the data to the client.				*/
		if(srv_alt_xferdata(spp, id) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			continue;
		}
		/* Increment count of successful compute rows sent.	*/ 
		rows++;
	}
	/* Release allocated memory. 					*/
	for(i = 0; i < actcols; i++)
	{
		SRV_FREE(dp[i]);
	}
	return(ok ? rows : (CS_INT)(-1));
}

/*
** GW_CONTROLROW - Control Row
**
**	This routine processes processes any control row information
**	associated with a result set.
**
**  Parameters:
**	spp 		Thread control structure.
**	cmdp		The command handle to check for control row information.
**	numcols		The number of columns in the result rows.
**  Returns:
**	CS_TRUE		Control row info was processed successfully.
**	CS_FALSE	An error was detected.
*/
CS_BOOL		gw_controlrow(spp, cmdp, numcols)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
CS_INT		numcols;
{
	CS_INT	i;			/* Loop control variable.	*/
	CS_BOOL	ok;			/* Error control flag.		*/
	CS_BYTE	buf[GW_MAXCOL];		/* Control row information.	*/
	CS_INT	ctrllen;		/* Length of control row information.*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/

	/* Get any control row information and send to client.		*/
	for(i = 1; i <= numcols; i++)
	{
		/* Read control information from server.		*/
		if(ct_getformat(cmdp, i, (CS_VOID *)buf, CS_SIZEOF(buf), 
			&ctrllen) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
		/* Write control information to client.			*/
		if(srv_setcontrol(spp, i, buf, ctrllen) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
	}
	return(ok);
}

/*
** GW_DOBROWSE
**
**	This routine processes browse mode results.  
**
**  Parameters:
**	spp		Thread control structure.
**	cmdp		The command handle to check for browse mode information.
**	numcols		The number of columns in the result rows.
**
**  Returns:
**	CS_TRUE		Browse mode information was processed successfully,
**			if it existed. 
**	CS_FALSE	An error was detected.
*/
CS_BOOL		gw_dobrowse(spp, cmdp, numcols)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
CS_INT		numcols;
{
	CS_INT		numtabs;		/* Number of browse mode   */
						/* tables.		   */
	CS_INT		outlen;			/* ct_br_tables argument.*/
	CS_INT		i;			/* Loop control variable.*/
	CS_BOOL		ok;			/* Error control flag.	*/
	CS_BROWSEDESC	bd;			/* Browse descriptor.	*/
	CS_CHAR		tabname[CS_MAX_NAME];	/* Table name buffer.	*/
	CS_BOOL		boolval;		/* Indicate if we have browse.*/

	/* Initialization.						*/
	numtabs = 0;			/* Assume not a browse. 	*/
	ok = CS_TRUE;			/* All is okay.			*/

	/* Check if browse exists at all.				*/
	if(ct_res_info(cmdp, CS_BROWSE_INFO, (CS_VOID *)&boolval, 
		CS_SIZEOF(boolval), NIL(CS_INT *)) != CS_SUCCEED)
	{
		return(CS_FALSE);
	}

	if(boolval == CS_FALSE)
	{
		return(CS_TRUE);
	}

	/* Determine the number of tables in browse mode select.	*/
	if(ct_br_table(cmdp, CS_UNUSED, CS_TABNUM, (CS_VOID *)&numtabs, 
		CS_UNUSED, &outlen) != CS_SUCCEED)
	{ 
		return(CS_FALSE);
	}

	/* Is this a browse mode result set?				*/
	if(numtabs == 0)
	{
		return(CS_TRUE);
	}

	/* Send all table names in browse mode request to client.	*/
	for(i = 1; i <= numtabs; i++)
	{
		/* Get name of table.					*/
		if(ct_br_table(cmdp, i, CS_TABNAME, (CS_VOID *)tabname,
			CS_SIZEOF(tabname), &outlen) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
		/* Send table name to client.				*/
		if(srv_tabname(spp, i, tabname, outlen) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
	}
	/* Return failure if error detected.				*/
	if(!ok)
	{
		return(CS_FALSE);
	}

	/* Link columns associated with browse mode request to tables.	*/
	for(i = 1; i <= numcols; i++)
	{
		/* Get column browse information. 			*/
		if(ct_br_column(cmdp, i, &bd) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
		/* Send column browse information to client.		*/
		if(srv_tabcolname(spp, i, &bd) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			break;
		}
	}
	return(ok);
}

/*
** GW_ORDERBY
**
**	This routine processes any order-by information associated with a 
**	row received from a remote server.
**
**  Parameters:
**	spp 		Thread control structure.
**	cmdp		The command handle to check for order-by information.
**
**  Returns:
**	CS_TRUE		Order-by information was processed successfully,
**			if it existed.
**	CS_FALSE	An error was detected.
*/
CS_BOOL		gw_orderby(spp, cmdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
{
	CS_BOOL		ok;			/* Error control flag.	*/
	CS_INT		numord;			/* Number of order-by columns.*/
	CS_INT		ordcols[GW_MAXCOL];	/* Order-by column list.*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	numord = 0;			/* Assume no order-by columns.	*/

	/* Get number of order-by columns. 				*/
	if(ct_res_info(cmdp, CS_NUMORDERCOLS, (CS_VOID *)&numord, CS_UNUSED, 
		NIL(CS_INT *)) != CS_SUCCEED)
	{
		return(CS_FALSE);
	}

	/* If there were no order-by columns just return success. 	*/
	if(numord == (CS_INT)0)
	{
		return(CS_TRUE);
	}

	/* Get column order in order-by list.				*/
	if(ct_res_info(cmdp, CS_ORDERBY_COLS, (CS_VOID *)ordcols,
		CS_SIZEOF(ordcols), NIL(CS_INT *)) != CS_SUCCEED)
	{
			return(CS_FALSE);
	}

	/* Send order-by information to client.				*/
	if(srv_orderby(spp, numord, ordcols) != CS_SUCCEED)
	{
		ok = CS_FALSE;
	}
	return(ok);
}

/*
**  GW_PROCSTAT
**
**	Read return status codes from a remote server and return the 
**	value to a client.  It is the caller's responsibility to send the 
**	required DONE based on the return code from this routine.  This 
**	routine does not send any DONEs to a client.
**
**  Parameters:
**	spp		Thread control structure.
**	cmdp		The command handle for the language or RPC command.
**
**  Returns:
**	CS_SUCCEED	Status successfully sent to a client.
**	CS_FAIL		An error was detected while sending status to client.
*/
CS_RETCODE	gw_procstat(spp, cmdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
{
	CS_DATAFMT	fmt;		/* Data description of return. 	*/
	CS_INT		len;		/* Return status length.	*/
	CS_SMALLINT	ind;		/* ct_bind() argument.		*/
	CS_INT		status;		/* Return status.		*/
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_BOOL		done;		/* Loop control variable.	*/
	CS_RETCODE	ret;		/* Return value from ct_fetch.	*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is well.			*/
	done = CS_FALSE;		/* Just starting.		*/

	/* Get a description of the return status.			*/
	if(ct_describe(cmdp, 1, &fmt) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/*
	** Make sure we received a data type supported by Open Server as
	** as return status.
	*/
	if(fmt.datatype != CS_INT_TYPE)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Bind the results to status variable.				*/
	if(ct_bind(cmdp, 1, &fmt, (CS_VOID *)&status, &len, &ind) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* Fetch status results from server and send them to client.	*/
	while(ok && !done)
	{
		/* Fetch the status.					*/
		if((ret = ct_fetch(cmdp, CS_UNUSED, CS_UNUSED, CS_UNUSED,
			NIL(CS_INT *))) != CS_SUCCEED)
		{
			if(ret == CS_FAIL)
			{
				ok = CS_FALSE;
			}
			else
			{
				done = CS_TRUE;
			}
			continue;
		}
		/* Send the return status to the client.		*/
		if(srv_sendstatus(spp, status) != CS_SUCCEED)
		{
			ok = CS_FALSE;
		}
	}
	return(ok ? CS_SUCCEED : CS_FAIL);
}

/*
** GW_PROCMSG
**
**	Read a message ID from the server.  Send the message ID to 
**	a client.  It is the caller's responsibility to read any parameters
**	associated with the message and to also send the required DONE based
**	on the return code from this routine.  This routine does not
**	send any DONEs to a client.
**
**  Parameters:
**	spp		Thread control structure.
**	cmdp		The current command handle.
**
**  Returns:
**	CS_SUCCEED	Message ID was processed successfully.
**	CS_FAIL		An error was detected while processing message ID.
*/
CS_RETCODE	gw_procmsg(spp, cmdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
{
	CS_INT		msgid;		/* Current message ID.		*/
	CS_INT		noparm;		/* Number of message parameters.*/
	CS_INT		status;		/* Message status.		*/
	CS_BOOL		ok;		/* Error control flag.		*/

	/* Initialization.						*/
	msgid = 0;
	noparm = 0;
	ok = CS_TRUE;

	/* Read the message ID from the remote server.			*/
	if(ct_res_info(cmdp, CS_MSGTYPE, (CS_VOID *)&msgid, CS_UNUSED,
		NIL(CS_INT *)) != CS_SUCCEED)		
	{
		return(CS_FAIL);
	}

	/* Determine if this message has arguments.			*/
	if(ct_res_info(cmdp, CS_NUMDATA, (CS_VOID *)&noparm, CS_UNUSED,
		NIL(CS_INT *)) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* Set the status flag for this message.			*/
	status = (noparm ? SRV_HASPARAMS : SRV_NOPARAMS);

	/*
	** Send the message information to the client.  Any parameters
	** associated with the message will be processed in gw_srvparam.
	*/
	if(srv_msg(spp, CS_SET, &msgid, &status) != CS_SUCCEED)
	{
		ok = CS_FALSE;
	}
	return(ok ? CS_SUCCEED : CS_FAIL);
}
/*
** GW_CLIENTPARAM
**
**	Read RPC or message parameters from a client.  Send the 
**	parameters to a server.  It is the caller's responsiblity
**	to take any required error recovery based on the return code
**	from this routine.
**
**  Parameters:
**	spp 		Thread control structure.
**	cmdp		The command handle for the outstanding command.
**	type		Parameter type.
**
**  Returns:
**	CS_CANCELED	ct_send returns CS_CANCELED.
**	CS_SUCCEED	Parameters successfully sent to server.
**	CS_FAIL		An error was detected while sending parameters to 
**			server.
*/
CS_RETCODE	gw_clientparam(spp, cmdp, type)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
CS_INT		type;
{
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_DATAFMT	*fmtp;		/* Parameter formats.		*/
	CS_INT		size;		/* Memory size variable.	*/
	CS_INT		*lenp;		/* Data length space.		*/
	CS_VOID		**dp;		/* Data space array.		*/
	int		i;		/* Loop control variable.	*/
	int		j;		/* Parameter array index.	*/
	CS_INT		actparms;	/* Memory allocated for actparms*/
	CS_INT		noparms;	/* Total number of parameters.	*/
	CS_INT		*pp;		/* Parameter array pointer.	*/
	CS_SMALLINT	*indp;		/* NULL indicator.		*/
	CS_RETCODE	ret;		/* Return code from ct_send.	*/
	CS_INT		retcode;	/* Return code for caller.	*/

	/* Initialization.						*/
	ok = CS_TRUE;			/* All is okay.			*/
	i = 1;				/* Parameters start at 1.	*/
	j = 0;				/* Parameter array index 0.	*/
	lenp = NIL(CS_INT *);		/* No length array yet.		*/
	dp = NIL(CS_VOID **);		/* No data space array yet.	*/
	noparms = 0;			/* Assume no parameters.	*/
	actparms = 0;			/* no parameters processed yet.	*/
	indp	= NIL(CS_SMALLINT *);

	/* Check for parameters.					*/
	if(srv_numparams(spp, &noparms) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* If there are no parameters, just send the command to server.	*/
	if(noparms == 0)
	{
		ret = gw_ctsend(spp, cmdp);

		return(ret);
	}

	/* Get pointer to parameter map array.				*/
	if((pp = gw_getparmp(spp)) == NIL(CS_INT *))
	{
		return(CS_FAIL);
	}

	/* Calculate memory size of CS_DATAFMT array.			*/
	size = noparms * CS_SIZEOF(CS_DATAFMT);

	/* Allocate space for the data format structures.		*/
	if((fmtp = (CS_DATAFMT *)srv_alloc(size)) == NIL(CS_DATAFMT *))
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Clear the CS_DATAFMT space.					*/
	srv_bzero((CS_VOID *)fmtp, size);

	/* Calculate memory size of data length array.			*/
	size = noparms * CS_SIZEOF(CS_INT);

	/* Allocate space for the data length array			*/
	if((lenp = (CS_INT *)srv_alloc(size)) == NIL(CS_INT *))
	{
		SRV_FREE(fmtp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Allocate space for NULL indicator.				*/
	if ((indp = (CS_SMALLINT *)srv_alloc(noparms * CS_SIZEOF(CS_SMALLINT)))
		== NIL(CS_SMALLINT *))
	{
		SRV_FREE(lenp);
		SRV_FREE(fmtp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Calculate memory size of data space array.			*/
	size = noparms * CS_SIZEOF(CS_VOID *);

	/* Allocate space for the data space array.			*/
	if((dp = (CS_VOID **)srv_alloc(size)) == NIL(CS_VOID **))
	{
		SRV_FREE(indp);
		SRV_FREE(lenp);
		SRV_FREE(fmtp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Allocate space for parameters and bind allocated space.	*/
	while(ok && (i <= noparms))
	{
		/* Get a description of the parameter.			*/
		if(srv_descfmt(spp, CS_GET, type, i, &(fmtp[i-1]))
			!= CS_SUCCEED)
		{
			ok = CS_FALSE;
			continue;
		}
		/* Allocate space for parameter.			*/
		if((dp[i-1] = srv_alloc((fmtp[i-1]).maxlength)) 
			== NIL(CS_VOID *))
		{
			ok = CS_FALSE;
			gw_senderror(spp);
			continue;
		}
		/* Save number of parameters for srv_free later.	*/
		actparms++;

		/* Bind allocated space to format description.		*/
		if(srv_bind(spp, CS_GET, type, i, &(fmtp[i-1]), dp[i-1],
			&(lenp[i-1]), &(indp[i-1])) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			continue;
		}
		/*  
		** If this is a return parameter, map the argument
		** number into the parameter array.  This parameter map
		** is used by gw_srvparam() to map arguments returned from
		** a server to the argument number expected by a client.
		*/
		if(fmtp[i-1].status & CS_RETURN)
		{
			pp[j++] = i;
		}

		/* Next parameter.					*/
		i++;			
	}

	/* Read parameters into allocated program variables.		*/
	if(ok && (srv_xferdata(spp, CS_GET, type) != CS_SUCCEED)) 
	{
		ok = CS_FALSE;
	}

	/*
	** Define parameters to Client-Library.  All required data 
	** conversions are done by the Client-Library.
	*/
	i = 1;
	while(ok && (i <= noparms))
	{
		if(ct_param(cmdp, &(fmtp[i-1]), dp[i-1], lenp[i-1], indp[i-1])
				!=CS_SUCCEED)
		{
			ok = CS_FALSE;
			continue;
		}
		/* Next parameter.					*/
		i++;
	}


	if(ok)
	{
		/* Send the RPC or message to the server.		*/
		ret = gw_ctsend(spp, cmdp);

		if((ret != CS_SUCCEED) && (ret != CS_CANCELED))
		{
			ok = CS_FALSE;
		}
	}

	/* Release all allocated memory.				*/
	SRV_FREE(fmtp);
	SRV_FREE(lenp);
	SRV_FREE(indp);
	if(dp != NIL(CS_VOID **))
	{
		for(i = 0; i < actparms; i++) SRV_FREE(dp[i]);
		SRV_FREE(dp);
	}

	/*
	** Determine the return code for the caller.
	*/
	if(ret == CS_CANCELED)
	{
		retcode = CS_CANCELED;
	}
	else if(ok)
	{
		retcode = CS_SUCCEED;
	}
	else
	{
		retcode = CS_FAIL;
	}
	return(retcode);
}

/*
** GW_SRVPARAM
**
**	Read RPC or message parameters from a server.  Return the 
**	parameters to a client.  It is the caller's responsibility to send the 
**	required DONE based on the return code from this routine.  This 
**	routine does not send any DONEs to a client.
**
**  Parameters:
**	spp 		Thread control structure
**	cmdp		The command handle for the outstanding command
**  Returns:
**	CS_SUCCEED	Parameters successfully sent to a client
**	CS_FAIL		An error was detected while sending parameters to 
**			client
*/
CS_RETCODE	gw_srvparam(spp, cmdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
{
	CS_INT		noparms;	/* Number of parameters.	*/
	CS_BOOL		rpc;		/* CS_TRUE if RPC parameters.	*/
	CS_INT		type;		/* SRV_RPCDATA or SRV_MSGDATA. 	*/
	int		i;		/* Loop control.		*/
	CS_DATAFMT	*fmtp;		/* Space for format descriptions.*/
	CS_INT		*lenp;		/* Space for data lengths.	*/
	CS_VOID		**dp;		/* Array of ptrs to data space.	*/
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_INT		*pp;		/* Parmeter map array pointer.	*/
	int		item;		/* Parameter argument number.	*/
	CS_INT		actparms;	/* Actual number of parms processed.*/
	CS_INT		cnt;		/* Argument to ct_fetch		*/
	CS_RETCODE	ret;		/* ct_fetch() return code.	*/
	CS_INT          eventtype;

	/* Initialization						*/
	rpc = CS_TRUE;			/* Assume RPC parameters.	*/
	type = SRV_RPCDATA;		/* Assume RPC parameters.	*/
	fmtp = NIL(CS_DATAFMT *);	/* No format descriptors yet.	*/
	dp = NIL(CS_VOID **);		/* No data space array yet.	*/
	lenp = NIL(CS_INT *);		/* No length space array yet.	*/
	ok = CS_TRUE;			/* All is okay.			*/
	i = 1;				/* Parameters start at 1.	*/
	pp = NIL(CS_INT *);		/* Pointer to param map array.	*/
	item = 0;			/* No argument number yet.	*/
	actparms = 0;			/* No parameters processed yet.	*/
	cnt = 0;			/* No parameters fetched.	*/

	/* Determine the number of parameters.				*/
	if(ct_res_info(cmdp, CS_NUMDATA, (CS_VOID *)&noparms, CS_UNUSED,
		NIL(CS_INT *)) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* Find out what event we're handling.				*/
        if(srv_thread_props(spp, CS_GET, SRV_T_EVENT, (CS_INT *)&eventtype,
		sizeof(CS_INT), NIL(CS_INT *)) == CS_FAIL)
	{
		(CS_VOID)ct_cmd_drop(cmdp);
		gw_senderror(spp);
		return(CS_FAIL);
	}
 
	/* Are these RPC or message parameters?				*/
	/* NOTE: return parameters are also sent from SRV_BULK handlers.*/
	if( (eventtype != SRV_BULK) && (srv_rpcname(spp, NIL(CS_INT *))
		== NIL(CS_CHAR *)) )
	{
		rpc = CS_FALSE;
		type = SRV_MSGDATA;
	}
	else if (eventtype == SRV_BULK)
	{
		rpc = CS_FALSE;
	}

	/* If these are RPC parameters, get a pointer to parm map array.*/
	if(rpc && ((pp = gw_getparmp(spp)) == NIL(CS_INT *)))
	{
		return(CS_FAIL);
	}

	/* Allocate space for the data format structures.		*/
	if((fmtp = (CS_DATAFMT *)srv_alloc(noparms * CS_SIZEOF(CS_DATAFMT)))
		== NIL(CS_DATAFMT *))
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Clear the data format space.					*/
	srv_bzero((CS_VOID *)fmtp, (noparms * CS_SIZEOF(CS_DATAFMT)));

	/* Allocate space for the data space array.			*/
	if((dp = (CS_VOID **)srv_alloc(noparms * CS_SIZEOF(CS_VOID *)))
		== NIL(CS_VOID **))
	{
		SRV_FREE(fmtp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Allocate space for the data lengths.				*/
	if((lenp = (CS_INT *)srv_alloc(noparms * CS_SIZEOF(CS_INT)))
		== NIL(CS_INT *))
	{
		SRV_FREE(dp);
		SRV_FREE(fmtp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Allocate space for parameters and bind allocated space.	*/
	while(ok && (i <= noparms))
	{
		/* Get a description of the parameter			*/
		if(ct_describe(cmdp, i, &(fmtp[i-1])) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			continue;
		}

		/* Allocate space for the parameter.			*/
		if((dp[i-1] = srv_alloc((fmtp[i-1]).maxlength))==NIL(CS_VOID *))
		{
			ok = CS_FALSE;
			gw_senderror(spp);
			continue;
		}
		/* Save number of parameters processed for freeing later.*/
		actparms++;

		/* Bind allocated space to format description.		*/
		if(ct_bind(cmdp, i, &(fmtp[i-1]), dp[i-1], &(lenp[i-1]),
			NIL(CS_SMALLINT *)) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			continue;
		}
		/*
		** Set up item number based on type of parameters.
		** If they are RPC parameters, we need to map the
		** item numbers using the retparms[] array.  If these
		** are msg parameters, no mapping is required.
		*/
		if(rpc)
		{
			item = pp[i - 1];
			pp[i - 1] = 0;

			/* Make sure a valid map occurred.		*/
			if(item == 0) 
			{
				ok = CS_FALSE;
				continue;
			}
		}
		else
		{
			item = i;
		}

		/*
		** Only describe MSG data to the Open Server Library.
		** The description of RPC return parameters was received
		** from the client when the RPC was sent.  It cannot be
		** redescribed here
		** For return parameters sent from a SRV_BULK
		** handler, we need to describe here.
		**/
		if (eventtype == SRV_BULK)
		{
			fmtp[i-1].status |= CS_RETURN;
			fmtp[i-1].status &= ~CS_INPUTVALUE;
			if(srv_descfmt(spp, CS_SET, SRV_RPCDATA, item,
				&(fmtp[i-1])) != CS_SUCCEED)
			{
				ok = CS_FALSE;
				continue;
			}
		}
		else if(!rpc && (srv_descfmt(spp, CS_SET, SRV_MSGDATA, item,
			&(fmtp[i-1])) != CS_SUCCEED))
		{
			ok = CS_FALSE;
			continue;
		}

		/* Bind the source data for Open Server.		*/
		if(srv_bind(spp, CS_SET, type, item, &(fmtp[i-1]),
			(CS_VOID *)dp[i-1], &(lenp[i-1]), NIL(CS_SMALLINT *)) 
			!= CS_SUCCEED)
		{
			ok = CS_FALSE;
			continue;
		}
		/* Next parameter.					*/
		i++;			
	}

	/*
	** Fetch the parameters received from the server and send them to 
	** a client.
	*/
	while(ok && ((ret = ct_fetch(cmdp,CS_UNUSED,CS_UNUSED,CS_UNUSED,&cnt))
		== CS_SUCCEED))
	{
		/* Send parameters to the client.			*/
		if(srv_xferdata(spp, CS_SET, type) != CS_SUCCEED)
		{
			ok = CS_FALSE;
			continue;
		}
	}
	/* Check for any errors.
	*/
	if(ok && (ret != CS_END_DATA) && (ret != CS_CANCELED))
	{
		ok = CS_FALSE;
	}

	/* Release all allocated handles and memory.			*/
	SRV_FREE(fmtp);
	SRV_FREE(lenp);
	for(i = 0; i < actparms; i++)
	{
		SRV_FREE(dp[i]);
	}
	SRV_FREE(dp);
	return(ok ? CS_SUCCEED : CS_FAIL);
}

/*
** GW_REMOTEMSGS - Remote Messages.
**
**	This routine is called by Open Client Library when messages are 
**	received from a remote DBMS. It reads the remote message information and
**	sends it back to the Open Server client as a message.
**
**  Parameters:
**	cp		Context.
**	chp		Connection handle.
**	msgp		Structure containing information on the server message.
**
**  Returns:
**	CS_SUCCEED
*/
CS_RETCODE 	gw_remotemsgs(cp, chp, msgp)
CS_CONTEXT	*cp;
CS_CONNECTION	*chp;
CS_SERVERMSG	*msgp;
{
	CS_CHAR		msg[1024];	/* Build message here.		*/
	SRV_PROC	*spp;		/* Client's thread.		*/
	CS_INT		len;		/* Argument to ct_con_props.	*/

	/* Initialization						*/
	spp = NIL(SRV_PROC *);		/* No current client thread.	*/
	len = 0;			/* Message length is 0.		*/

	/* Make sure chp is valid.					*/
	if(chp == NIL(CS_CONNECTION *))
	{
		gw_senderror(spp);
		return(CS_SUCCEED);
	}

	/* Get the current thread for the client.			*/
	if(ct_con_props(chp, CS_GET, CS_USERDATA, &spp, CS_SIZEOF(spp), &len)
		!= CS_SUCCEED)
	{
		return(CS_SUCCEED);
	}

	/* Make sure we have a valid client client thread.		*/
	if(spp == NIL(SRV_PROC *))
	{
		gw_senderror(spp);
		return(CS_SUCCEED);
	}

	/* Build client message in buffer for log.			*/
	SPRINTF(msg, GW_RMTMSG, msgp->svrname, msgp->msgnumber, msgp->severity, 
		msgp->state);

	/* Log server message.						*/
	SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);

	/* Null terminate message.					*/
	switch((int)(msgp->textlen))
	{
	case CS_NULLTERM:
		/* Message already NULL terminated.			*/
		break;

	case CS_UNUSED:
	case 0:
		/* No message specified.				*/
		msgp->text[0] = '\0';
		break;

	default:
		if(msgp->textlen < (CS_MAX_MSG - 1))
		{
			msgp->text[msgp->textlen] = '\0';
		}
		else
		{
			msgp->text[CS_MAX_MSG - 1] = '\0';
		}
		break;
	}

	msgp->status |= (CS_FIRST_CHUNK|CS_LAST_CHUNK);

	/* Copy null terminated message into buffer.			*/
	SPRINTF(msg, "\t\t%s\n", msgp->text);

	/* Log message text.						*/
	SRV_LOG(NIL(SRV_SERVER *), CS_FALSE, msg, CS_NULLTERM);

	/*
	** Send error string to client.
	*/
	(CS_VOID)srv_sendinfo(spp, msgp, CS_TRAN_UNDEFINED);

	return(CS_SUCCEED);
}

/*
** GW_CTERR
**
**	This is the CT-Lib error handler.  All Open Client connections
**	use this error handler.  The received error is passed back to the
**	Open Server client that it affects.
**
**  Parameters:
**	cp		Context structure.
**	chp		Connection handle.
**	emsgp		Pointer to Open Client CS_CLIENTMSG structure.
**
**  Returns:
**	CS_SUCCEED
*/
CS_RETCODE	gw_cterr(cp, chp, emsgp)
CS_CONTEXT	*cp;
CS_CONNECTION	*chp;
CS_CLIENTMSG	*emsgp;
{
	SRV_PROC	*spp;		/* spp of client connection.	*/
	CS_CHAR		msg[1024];	/* Buffer for message.		*/
	CS_INT		len;		/* ct_con_props return size.	*/
	CS_SERVERMSG	smsg;		/* Server message structure.	*/

	/* Initialization. 						*/
	spp = NIL(SRV_PROC *);		/* No SRV_PROC yet.		*/
	len = 0;			/* Clear ct_con_props argument.	*/

	/* 
	** If chp is NIL(CS_CONNECTION *), this error occurred when no 
	** connection handle was active.  Just return cancel.  The error will
	** be processed by the main-line code.
	*/
	if(chp == NIL(CS_CONNECTION *))
	{
		SPRINTF(msg,"gw_cterr: NULLSRVPROC err(%d: %s) oserr(%d: %s)\n",
			emsgp->msgnumber, emsgp->msgstring, emsgp->osnumber, 
			emsgp->osstring);
		SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);
		return(CS_SUCCEED);
	}

	/* Get the current spp from the connection handle.		*/
	if(ct_con_props(chp, CS_GET, CS_USERDATA, &spp, CS_SIZEOF(spp), &len)
		!= CS_SUCCEED)
	{
		return(CS_SUCCEED);
	}

	/* Make sure we have a valid spp.				*/
	if(spp == NIL(SRV_PROC *))
	{
		SPRINTF(msg,"gw_cterr: NULLSRVPROC err(%d: %s) oserr(%d: %s)\n",
			emsgp->msgnumber, emsgp->msgstring, emsgp->osnumber, 
			emsgp->osstring);
		SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);
		return(CS_SUCCEED);	
	}

	/*
	** Log the error numbers and severity.
	*/
	SPRINTF(msg, GW_ERRMSG ,emsgp->msgnumber, emsgp->severity, 
		emsgp->osnumber);
	SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);

	SPRINTF(msg, "\t\t%s\n", emsgp->msgstring);
	SRV_LOG(NIL(SRV_SERVER *), CS_FALSE, msg, CS_NULLTERM);

	/*
	** If there was an Operating System error, log this information.
	*/
	if((emsgp->osstringlen != 0) && (emsgp->osstringlen != CS_UNUSED))
	{
		SPRINTF(msg, GW_OSERRMSG, emsgp->osnumber);
		SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, msg, CS_NULLTERM);
		SPRINTF(msg, "%s\n", emsgp->osstring);
		SRV_LOG(NIL(SRV_SERVER *), CS_FALSE, msg, CS_NULLTERM);
	}

	/*
	** Map the client message structure into the server message 
	** structure so we can send this information on to the client.
	*/
	smsg.msgnumber = emsgp->msgnumber;
	smsg.state = (CS_INT)0;
	smsg.severity = emsgp->severity;
	srv_bmove(emsgp->msgstring, smsg.text, CS_SIZEOF(smsg.text));
	smsg.textlen = emsgp->msgstringlen;
	(CS_VOID)srv_props(Context, CS_GET, SRV_S_SERVERNAME,
		(CS_VOID *)smsg.svrname, CS_SIZEOF(smsg.svrname),
		&(smsg.svrnlen));
	smsg.proclen = (CS_INT)0;
	smsg.line = (CS_INT)0;
	smsg.status = emsgp->status;
	smsg.status |= (CS_FIRST_CHUNK|CS_LAST_CHUNK);

	srv_bmove(emsgp->sqlstate, smsg.sqlstate, CS_SIZEOF(smsg.sqlstate));
	smsg.sqlstatelen = emsgp->sqlstatelen;

	/*
	** Send error string to client.
	*/
	(CS_VOID)srv_sendinfo(spp, &smsg, CS_TRAN_UNDEFINED);

	return(CS_SUCCEED);
}

/*
** GW_DEFN_REMOTE_PWDS
**
**  	Read remote servername/password pairs from remote client
**	and define them for the current connection.
**
**  Parameters:
**	spp	Remote client connection.
**	conn 	Remote server connection.
**
**  Returns:
**	CS_FAIL or CS_SUCCEED.
*/
CS_RETCODE	gw_defnremotepwds(spp, conn)
SRV_PROC	*spp;
CS_CONNECTION	*conn;
{
	CS_BYTE		srvname[100];
	CS_INT		srvnamelen;
	CS_BYTE		pwd[100];
	CS_INT		pwdlen;
	CS_DATAFMT	fmt;
	DBMSCTL_T	*rp;		/* Remote server control structure.*/


	/* Get user data from SRV_PROC structure			*/
	if (srv_thread_props(spp, CS_GET, SRV_T_USERDATA, (CS_VOID *)&rp,
		sizeof(rp), NIL(CS_INT *)) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	/*
	** First we see if there are any server/password pairs.
	** If there aren't any, it's OK.
	*/
	if(srv_negotiate(spp, CS_GET, SRV_NEG_REMPWD) != CS_SUCCEED)
	{
		return(CS_SUCCEED);
	}

	/*
	** See how many server names and passwords were sent from the
	** remote client.
	*/
	if(srv_numparams(spp, &rp->NumEncrypts) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	while(rp->EncryptToGet < rp->NumEncrypts)
	{
		/* Read them in two at a time. */
	
		/* Read the server name. */
		rp->EncryptToGet++;
	
		if(srv_descfmt(spp, CS_GET, SRV_NEGDATA, rp->EncryptToGet,
			&fmt) != CS_SUCCEED)
		{
			return(CS_FAIL);
		}

		if(srv_bind(spp, CS_GET, SRV_NEGDATA, rp->EncryptToGet,
			&fmt, srvname, &srvnamelen, NIL(CS_SMALLINT *))
			!= CS_SUCCEED)
		{
			return(CS_FAIL);
		}

		if(srv_xferdata(spp, CS_GET, SRV_NEGDATA) != CS_SUCCEED)
		{
			return(CS_FAIL);
		}

		/* Then the remote password.  */
		rp->EncryptToGet++;

		if(srv_descfmt(spp, CS_GET, SRV_NEGDATA, rp->EncryptToGet,
			&fmt) != CS_SUCCEED)
		{
			return(CS_FAIL);
		}

		if(srv_bind(spp, CS_GET, SRV_NEGDATA, rp->EncryptToGet,
			&fmt, pwd, &pwdlen, NIL(CS_SMALLINT *)) != CS_SUCCEED)
		{
			return(CS_FAIL);
		}

		if(srv_xferdata(spp, CS_GET, SRV_NEGDATA) != CS_SUCCEED)
		{
			return(CS_FAIL);
		}

		/* Define the pair for this connection */
		if(ct_remote_pwd(conn, CS_SET, (CS_CHAR *)srvname,
			srvnamelen, (CS_CHAR *)pwd, pwdlen) != CS_SUCCEED)
		{
			return(CS_FAIL);
		}
	}

	if(rp->EncryptToGet == rp->NumEncrypts)
	{
		rp->EncryptToGet = rp->NumEncrypts = 0;
	}
	return(CS_SUCCEED);
}

/*
**  GW_ENCRYPTCB
**
**	Callback for negotiated logins (password-encryption type).
**  	Relay encryption keys from remote server
**	to remote client, and encrypted passwords from
**	remote client to remote server.
**
**  Parameters:
**	conn		Remote server connection.
**	pwd		Password to be encrypted (set by CT-Libary).
**	pwdlen 		Original length of password.
**	key		Encryption key from remote server.
**	keylen 		Length of key.
**	buf 		Space for encrypted password from remote client.
**	buflen 		Length of space.
**	outlen		Length of encrypted password.
**
**  Returns:
**	CS_FAIL or CS_SUCCEED.
*/
CS_RETCODE CS_PUBLIC gw_encryptcb(conn, pwd, pwdlen, key, keylen, buf, buflen,
			outlen)
CS_CONNECTION	*conn;
CS_BYTE		*pwd;
CS_INT		pwdlen;
CS_BYTE		*key;
CS_INT		keylen;
CS_BYTE		*buf;
CS_INT		buflen;
CS_INT		*outlen;
{
	SRV_PROC	*spp;
	CS_INT		len;
	CS_DATAFMT	fmt;
	DBMSCTL_T	*rp;		/* Remote server control structure.*/

	/* Get the current thread for the client.			*/
	if(ct_con_props(conn, CS_GET, CS_USERDATA, &spp, CS_SIZEOF(spp), 
		&len) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* Get user data from SRV_PROC structure			*/
	if (srv_thread_props(spp, CS_GET, SRV_T_USERDATA, (CS_VOID *)&rp,
		sizeof(rp), NIL(CS_INT *)) == CS_FAIL)
	{
		return(CS_FAIL);
	}

	/* 
	** If rp->EncryptPwdSent is FALSE, then this is
	** the first invocation of the callback handler
	** for this login, so we need to send an encryption
	** key to the remote client (via the "key" output
	** parameter) and request encrypted information -- an encrypted local
	** password and 0 or more remote svrname/encrypted-pwd pairs --
	** in return.
	**
	** srv_negotiate here tells the client to expect an encryption key.
	*/ 
	if((rp->EncryptPwdSent == CS_FALSE)
		&& (srv_negotiate(spp, CS_SET, SRV_NEG_ENCRYPT) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	srv_bzero(&fmt, sizeof(CS_DATAFMT));
	fmt.datatype = CS_BINARY_TYPE;
	fmt.status = CS_CANBENULL;
	fmt.maxlength = keylen;
	fmt.namelen = 0;
	fmt.format = 0;
	fmt.count = 1;

	/* Describe and send the key itself.				*/
	if((rp->EncryptPwdSent == CS_FALSE)
		&& (srv_descfmt(spp, CS_SET, SRV_NEGDATA, 1, &fmt)
		!= CS_SUCCEED)) 
	{ 
		return(CS_FAIL);
	}

	if((rp->EncryptPwdSent == CS_FALSE)
		&& (srv_bind(spp, CS_SET, SRV_NEGDATA, 1, &fmt, key, &keylen,
		NIL(CS_SMALLINT *)) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	if((rp->EncryptPwdSent == CS_FALSE)
		&& (srv_xferdata(spp, CS_SET, SRV_NEGDATA) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	/* Complete this portion of the exchange.			*/
	if((rp->EncryptPwdSent == CS_FALSE)
		&& (srv_senddone(spp, SRV_DONE_FINAL, CS_TRAN_UNDEFINED,
			(CS_INT)0) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	/*
	** Wait until there's a response from the client for the
	** encrypted password request.
	*/
	if((rp->EncryptPwdSent == CS_FALSE)
		&& (srv_negotiate(spp, CS_GET, SRV_NEG_LOCPWD) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	/*
	** Now we read in the response -- first, the encrypted password.
	** This goes into the "buf" output parameter.
	*/
	if((rp->EncryptPwdSent == CS_FALSE)
		&& (srv_descfmt(spp, CS_GET, SRV_NEGDATA, 1, &fmt)
		!= CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	if((rp->EncryptPwdSent == CS_FALSE)
		&& (srv_bind(spp, CS_GET, SRV_NEGDATA, 1, &fmt, buf, outlen,
		NIL(CS_SMALLINT *)) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	if((rp->EncryptPwdSent == CS_FALSE)
		&& (srv_xferdata(spp, CS_GET, SRV_NEGDATA) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	if(rp->EncryptPwdSent == CS_FALSE)
	{
		/*
		** Define any remote-server/password pairs sent by the client.
		*/
		rp->EncryptPwdSent = CS_TRUE;
		return(gw_defnremotepwds(spp, conn));
	}

	/* If rp->EncryptPwdSent is TRUE, this is the second
	** (or greater) invocation of the callback handler
	** for this login, so we need to read in any existing
	** remote server info -- server name and encrypted
	** remote server password, in pairs -- from the remote
	** client and pass it on to the remote server, via the
	** "buf" output parameter.
	**
	** See how many server names and passwords, in alternation, were sent.
	*/
	if(srv_numparams(spp, &rp->NumEncrypts) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* Get the password, i.e. the second of each pair.		*/
	rp->EncryptToGet = rp->EncryptToGet + 2;

	if(srv_descfmt(spp, CS_GET, SRV_NEGDATA, rp->EncryptToGet, &fmt)
		!= CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	if(srv_bind(spp, CS_GET, SRV_NEGDATA, rp->EncryptToGet, &fmt, buf,
		outlen, NIL(CS_SMALLINT *)) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	if(srv_xferdata(spp, CS_GET, SRV_NEGDATA) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	if(rp->EncryptToGet == rp->NumEncrypts)
	{
		rp->EncryptToGet = rp->NumEncrypts = 0;
		rp->EncryptPwdSent = CS_FALSE;
	}
	return(CS_SUCCEED);
}

/*
** GW_SENDSECLABELS
**	Callback for secure-label negotiated logins.
**  	Relay label requests from remote server
**	to remote client, and secure labels from
**	remote client to remote server.
**
**  Parameters:
**	conn 	Remote server connection.
**	inmsg	Message type received from remote server (set by CT-Library) .
**	outmsg	Message type sent to remote client (set by CT-Library).
**	infmt 	Format of remote server's message.
**	inbuf 	Label request from remote server.
**	outfmt	Format of remote client's message.
**	outbuf 	Secure label from remote client.
**	outlen	Length of secure label.
**
**  Returns:
**	CS_FAIL or CS_SUCCEED
*/
CS_RETCODE CS_PUBLIC gw_sendseclabels(conn, inmsg, outmsg, infmt, inbuf, outfmt,
			outbuf, outlen) 
CS_CONNECTION	*conn; 
CS_INT	 	inmsg; 
CS_INT		*outmsg;
CS_DATAFMT	*infmt;
CS_BYTE		*inbuf;
CS_DATAFMT	*outfmt;
CS_BYTE		*outbuf;
CS_INT		*outlen;
{
	SRV_PROC	*spp;
	CS_INT		len;
	DBMSCTL_T	*rp;		/* Remote server control structure.*/

	/* Get the current spp for the client.				*/
	if(ct_con_props(conn, CS_GET, CS_USERDATA, &spp, CS_SIZEOF(spp), 
		&len) != CS_SUCCEED)
	{
		return(CS_SUCCEED);
	}

	/* Get user data from SRV_PROC structure			*/
	if (srv_thread_props(spp, CS_GET, SRV_T_USERDATA, (CS_VOID *)&rp,
		sizeof(rp), NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* srv_negotiate here tells the client to send labels.		*/ 
	if((rp->NumSecLabels == 0)
		&& (srv_negotiate(spp, CS_SET, SRV_NEG_SECLABEL) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	/* Complete this portion of the exchange.			*/
	if((rp->NumSecLabels == 0)
		&& (srv_senddone(spp, SRV_DONE_FINAL, CS_TRAN_UNDEFINED, 0)
		!= CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	/*
	** OK, now we wait until there's a response from the client.
	*/
	if((rp->NumSecLabels == 0)
		&& (srv_negotiate(spp, CS_GET, SRV_NEG_SECLABEL) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	/* See how many labels were sent.				*/
	if((rp->NumSecLabels == 0)
		&& (srv_numparams(spp, &rp->NumSecLabels) != CS_SUCCEED))
	{
		return(CS_FAIL);
	}

	/* Read in one at a time.					*/
	rp->LabelToSend++;
	srv_bzero(outfmt, sizeof(CS_DATAFMT));
	if(srv_descfmt(spp, CS_GET, SRV_NEGDATA, rp->LabelToSend, outfmt)
		!= CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	if(srv_bind(spp, CS_GET, SRV_NEGDATA, rp->LabelToSend, outfmt, outbuf,
		outlen, NIL(CS_SMALLINT *)) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	if(srv_xferdata(spp, CS_GET, SRV_NEGDATA) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	*outmsg = CS_MSG_LABELS;

	/* If we've sent all the labels, we're done.			*/
	if(rp->LabelToSend == rp->NumSecLabels)
	{
		rp->LabelToSend = rp->NumSecLabels = 0;
		return(CS_SUCCEED);
	}

	/* Otherwise, let the caller know there's more.	*/
	return(CS_CONTINUE);
}

/*
**  GW_RESPONSECB
**
**	Callback for negotiated logins
**	(challenge-response and secure-label types).
**  	Relay challenges (or label requests) from remote server
**	to remote client, and reponses (or secure labels) from
**	remote client to remote server.
**
**  Parameters:
**	conn 	Remote server connection.
**	inmsg	Message type received from remote server (set by CT-Library). 
**	outmsg 	Message type sent to remote client (set by CT-Library).
**	infmt 	Format of remote server's message.
**	inbuf 	Challenge (or label request) from remote server.
**	outfmt 	Format of remote client's message.
**	outbuf 	Response (or secure label) from remote client.
**	outlen	length of response (or secure label).
**
**  Returns:
**	CS_FAIL or CS_SUCCEED
*/
CS_RETCODE CS_PUBLIC gw_responsecb(conn, inmsg, outmsg, infmt, inbuf, outfmt,
			outbuf, outlen)
CS_CONNECTION	*conn;
CS_INT	 	inmsg;
CS_INT		*outmsg;
CS_DATAFMT	*infmt;
CS_BYTE		*inbuf;
CS_DATAFMT	*outfmt;
CS_BYTE		*outbuf;
CS_INT		*outlen;
{
	SRV_PROC	*spp;
	CS_INT		len;
	CS_INT		junk;
	CS_INT		neglogintype;

	/* Get the current spp for the client.				*/
	if(ct_con_props(conn, CS_GET, CS_USERDATA, &spp, CS_SIZEOF(spp), 
		&len) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* See what type of negotiated login was requested.		*/
        if(srv_thread_props(spp, CS_GET, SRV_T_NEGLOGIN,
		(CS_VOID *)&neglogintype, CS_SIZEOF(neglogintype), &junk)
		== CS_FAIL)
	{
		return(CS_FAIL);
	}

	/* If we're doing secure labels, let's call another routine.	*/
	if(neglogintype & SRV_SECLABEL)
	{
		return(gw_sendseclabels(conn, inmsg, outmsg, infmt, inbuf,
			outfmt, outbuf, outlen));
	}

	/* Get the current spp for the client.				*/
	if(ct_con_props(conn, CS_GET, CS_USERDATA, &spp, CS_SIZEOF(spp), 
		&len) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* srv_negotiate here tells the client to expect a challenge.	*/ 
	if(srv_negotiate(spp, CS_SET, SRV_NEG_CHALLENGE) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* Now let's describe and send the challenge itself.		*/
	if(srv_descfmt(spp, CS_SET, SRV_NEGDATA, 1, infmt) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	if(srv_bind(spp, CS_SET, SRV_NEGDATA, 1, infmt, inbuf,
		&(infmt->maxlength), NIL(CS_SMALLINT *)) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	if(srv_xferdata(spp, CS_SET, SRV_NEGDATA) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* Complete this portion of the exchange.			*/
	if(srv_senddone(spp, SRV_DONE_FINAL, CS_TRAN_UNDEFINED, 0)
		!= CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/*
	** Wait until there's a response from the client.
	*/
	if(srv_negotiate(spp, CS_GET, SRV_NEG_CHALLENGE) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	/* Read in the response.					*/
	srv_bzero(outfmt, sizeof(CS_DATAFMT));
	if(srv_descfmt(spp, CS_GET, SRV_NEGDATA, 1, outfmt) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	if(srv_bind(spp, CS_GET, SRV_NEGDATA, 1, outfmt, outbuf, outlen,
		NIL(CS_SMALLINT *)) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}

	if(srv_xferdata(spp, CS_GET, SRV_NEGDATA) != CS_SUCCEED)
	{
		return(CS_FAIL);
	}
	return(CS_SUCCEED);
}

/*
** GW_ATTNHANDLER
**
** 	This event handler is called when the Open Server receives an
**	attention from one of its clients.  It sends a CS_CANCEL_ATTN
**	to the remote server.
**
**	NOTE:  This causes all results for ALL outstanding commands to be 
**	discarded.  It is not possible to selectively cancel a single command 
**	on a server connection.
**
** 	The DONE acknowledgement will be sent to the client by Open Server when
**	the attention handler returns.
**
**  Parameters:
**	spp	Thread control structure.
**
**  Returns:
**	CS_SUCCEED	Attention handled.
**	CS_FAIL		Error detected.
*/
CS_RETCODE	CS_PUBLIC gw_attnhandler(spp)
SRV_PROC	*spp;
{
	CS_CONNECTION	*chp;		/* Connection handle to server.	*/

	/* Get the connection handle for the server connection.		*/
	if((chp = gw_getchp(spp, CS_FALSE)) == NIL(CS_CONNECTION *))
	{
		SRV_LOG(NIL(SRV_SERVER *), CS_TRUE, 
			"gw_getchp() failed in gw_attnhandler", CS_NULLTERM);
		return(CS_FAIL);
	}

	/* Cancel any CT-Library activity on this connection.		*/
	(CS_VOID)ct_cancel(chp, NIL(CS_COMMAND *), CS_CANCEL_ATTN);
	return(CS_SUCCEED);
}

/*
** GW_GETCHP
**
**	Retrieve the connection handle from a remote DBMS control structure.
**	If release is CS_TRUE all control structures except the 
**	CS_CONNECTION handle are released.
**
**  Parameters:
**	spp		Thread control structure.
**	release		CS_TRUE: release the control structures.
**		  	CS_FALSE: do not release the control structure.
**  Returns:
**	A valid connection handle for the spp, or NIL(CS_CONNECTION *) if 
**	a valid connection handle could not be found.
*/
CS_CONNECTION	*gw_getchp(spp, release)
SRV_PROC	*spp;
CS_BOOL		release;
{
	DBMSCTL_T	*rp;		/* Remote server control structure.*/
	CS_CONNECTION	*retchp;	/* connection handler to return. */

	/* Get user data from SRV_PROC structure.			*/
	if(srv_thread_props(spp, CS_GET, SRV_T_USERDATA, (CS_VOID *)&rp,
		CS_SIZEOF(rp), NIL(CS_INT *)) == CS_FAIL)
	{
		return(NIL(CS_CONNECTION *));
	}

	/*
	** Make sure we have a valid remote server control structure.  We
	** won't have one if an error is detected during connection
	** establishment.
	*/
	if(rp == NIL(DBMSCTL_T *))
	{
		return(NIL(CS_CONNECTION *));
	}

	retchp = rp->chp;

	/*
	** Free any allocated control structures if release == CS_TRUE.
	** Remove the control structure from the user data area of the
	** thread control structure.
	*/
	if(release)
	{
		SRV_FREE(rp);
		rp = NIL(DBMSCTL_T *);
		(CS_VOID)srv_thread_props(spp, CS_SET, SRV_T_USERDATA,
			(CS_VOID *)&rp, CS_SIZEOF(rp), NIL(CS_INT *));
	}
	return(retchp);
}

/*
** GW_GETPARMP
**
**	Retrieve a pointer to the parameter array in a remote DBMS control 
**	structure.
**
**  Parameters:
**	spp	Thread control structure.
**
**  Returns:
**	A valid pointer to the parameter array for the spp, or NIL(CS_INT *)
**	if a valid parameter array could not be found.
*/
CS_INT		*gw_getparmp(spp)
SRV_PROC	*spp;
{
	DBMSCTL_T	*rp;		/* Remote server control structure.*/
	CS_INT		*pp;		/* Pointer to parameter array.	*/

	/* Initialization.						*/
	pp = NIL(CS_INT *);		/* Not valid yet.		*/

	/* Get user data from SRV_PROC structure.			*/
	if(srv_thread_props(spp, CS_GET, SRV_T_USERDATA, (CS_VOID *)&rp,
		CS_SIZEOF(rp), NIL(CS_INT *)) == CS_SUCCEED)
	{
		pp = rp->retparms;
	}
	return(pp);
}

/*
** GW_MANAGEFMTP
**
**	Set and retrieve a pointer to the format array in the remote DBMS 
**	control structure.  There are two format arrays stored in the DBMS 
**	control structure.  One for compute rows and the other for regular
**	rows.
**
**  Arguments:
**	spp		Thread control structure.
**	action		GW_GETFMTP - return the format pointer.
**			GW_SETFMTP - set the format pointer.
**			GW_CLEARFMTP - release all allocated format structures.
**	type		GW_REGULARFMT - process regular row format pointer.
**			GW_COMPUTEFMT - process compute row format pointer.
**	computeid	Compute ID of format array to return to caller.
**	fmtpvar		Return pointer or set pointer.
**
**  Returns:
**	CS_TRUE		Action processed successfully.
**	CS_FALSE	Error detected while processing action.
*/
CS_BOOL		gw_managefmtp(spp, action, type, computeid, fmtpvar)
SRV_PROC	*spp;
CS_INT		action;
CS_INT		type;
CS_INT		computeid;
CS_DATAFMT	**fmtpvar;
{
	DBMSCTL_T	*rp;	/* Remote server control structure.	*/
	CS_DATAFMT 	**fmtp;	/* Working CS_DATAFMT structure.	*/
	int		i;	/* Loop variable.			*/

	/* Get user data from SRV_PROC structure.			*/
	if(srv_thread_props(spp, CS_GET, SRV_T_USERDATA, (CS_VOID *)&rp,
		CS_SIZEOF(rp), NIL(CS_INT *)) == CS_FAIL)
	{
		return(CS_FALSE);
	}

	/* Process according to action and type.			*/
	switch((int)action)
	{
	case GW_GETFMTP:
		switch((int)type)
		{
		case GW_REGULARFMT:
			*fmtpvar = rp->rowfmtp;
			break;
		case GW_COMPUTEFMT:
			*fmtpvar = rp->computefmtp[computeid - 1];
			break;
		default:
			/* Can't happen.				*/
			break;
		}
		break;

	case GW_SETFMTP:
		switch((int)type)
		{
		case GW_REGULARFMT:
			rp->rowfmtp = *fmtpvar;
			break;
		case GW_COMPUTEFMT:
			rp->computefmtp[computeid - 1] = *fmtpvar;
			break;
		default:
			/* Can't happen.				*/
			break;
		}
		break;

	case GW_CLEARFMTP:
		/* Release row format structures.			*/
		SRV_FREE(rp->rowfmtp);
		rp->rowfmtp = NIL(CS_DATAFMT *);

		/*
		** Release all compute row format structures.
		** 
		** NOTE:  this code assumes that compute row IDs are
		** sequential.
		*/
		fmtp = rp->computefmtp;
		for(i = 0; i < GW_MAXCOL; i++)
		{
			/* Are we at the end of the formats?		*/
			if(fmtp[i] == NIL(CS_DATAFMT *))
			{
				break;
			}

			/* Release and clear format storage.		*/
			SRV_FREE(fmtp[i]);
			fmtp[i] = NIL(CS_DATAFMT *);
		}
		break;

	default:
		/* Can't happen.					*/
		break;
	}
	return(CS_TRUE);
}

/*
** GW_CHKWRITETEXT
**
**	Check to see if a language-based "writetext bulk"
**	or "insert bulk" was issued by the client.
**	Call srv_bulktype to set the spp field accordingly.
**
**  Parameters:
**	spp		Thread control structure.
**	chp		Connection handle.
**	cmd		The language string.
**
**  Returns:
**	GW_INSRTBLK	"insert bulk": this got processed here,
**			so we needn't pass it on to the server.
**	GW_WRTXTBLK	"writetext bulk": we just set the iotype.
**			in this case, so we still need to send
**			the language.	
**	GW_NONE	  	neither of the above: just pass it on.	
*/
#define WRITE_TEXT_STR		"writetext"
#define INSERT_STR		"insert"
#define BULK_STR		"bulk"

CS_RETCODE	gw_chkwritetext(spp, chp, cmd)
SRV_PROC	*spp;
CS_CONNECTION	*chp;
CS_CHAR		*cmd;
{
	CS_INT		iotype;
	CS_CHAR		*tmpstr;	/* Pointer into language string.*/
	CS_CHAR		tblname[100];	/* Name of bulk table.		*/
	CS_CHAR		*tmp_tblname;	/* Pointer into tblname.	*/
	DBMSCTL_T	*rp;		/* Connection control structure.*/
	CS_IODESC	*docdescptr;
	char		textptrstr[100];
	char		timestampstr[100];
	char 		*tmp_objname;
	char 		*tmp_textptrstr;
	char 		*tmp_timestampstr;
	CS_BOOL		labelflag = CS_FALSE;

	/* Initialization						*/
	tmpstr = cmd;			/* Point at callers lang string.*/
	tmp_tblname = tblname;		/* Start at the beginning.	*/

	/* Get user data from SRV_PROC structure.			*/
	if(srv_thread_props(spp, CS_GET, SRV_T_USERDATA, (CS_VOID *)&rp,
		CS_SIZEOF(rp), NIL(CS_INT *)) == CS_FAIL)
	{
		return(GW_NONE);
	}

	/* Get connection handle for server connection.			*/
	if((chp = gw_getchp(spp, CS_FALSE)) == NIL(CS_CONNECTION *))
	{
		return(GW_NONE);
	}

	/* Go to the first non-blank character string in the buffer.	*/
	while((isspace(*tmpstr)) && (*tmpstr != '\0'))
	{
		tmpstr++;
	}

	/* Be sure there WAS a non-blank character string to go to.	*/
	if(*tmpstr == '\0')
	{
		return(GW_NONE);
	}

	/* Is it "writetext"? 						*/
	if(strncmp(tmpstr, WRITE_TEXT_STR, strlen(WRITE_TEXT_STR)) == 0)
	{
		/* For now, let's assume it's text instead of image.	*/
		iotype = SRV_TEXTLOAD;

		/* Get ready to look for "bulk".			*/
		tmpstr += strlen(WRITE_TEXT_STR);
	}
	/* Is it "insert"?						*/
	else if(strncmp(tmpstr, INSERT_STR, strlen(INSERT_STR)) == 0)
	{
		iotype = SRV_BULKLOAD;

		/* Get ready to look for "bulk".			*/
		tmpstr += strlen(INSERT_STR);
	}
	else
	{
		return(GW_NONE);
	}

	/* We're looking for "bulk" now.				*/
	while((isspace(*tmpstr)) && (*tmpstr != '\0'))
	{
		tmpstr++;
	}

	/* End of string?						*/
	if(*tmpstr == '\0')
	{
		return(GW_NONE);
	}

	/* If we've found it, set the spp's iotype accordingly.	*/
	if(strncmp(tmpstr, BULK_STR, strlen(BULK_STR)) == 0)
	{
		/* If it was a "writetext bulk", we must parse more.	*/
		if(iotype == SRV_TEXTLOAD)
		{
			if(srv_thread_props(spp, CS_SET, SRV_T_BULKTYPE,
		 		(CS_VOID *)&iotype, CS_SIZEOF(iotype),
				NIL(CS_INT *)) == CS_FAIL)
			{
				return(GW_NONE);
			}

			docdescptr = &(rp->iodesc);

			/* First we parse for  the object name.		*/
			tmpstr += strlen(BULK_STR);

			tmp_objname = docdescptr->name;

			while((isspace(*tmpstr)) && (*tmpstr != '\0')) tmpstr++;

			/* End of string?				*/
			if(*tmpstr == '\0') return(GW_NONE);

			while((!(isspace(*tmpstr))) && (*tmpstr != '\0')) 
			{
				*tmp_objname++ = *tmpstr++;
			}
			*tmp_objname = '\0';

			docdescptr->namelen = strlen(docdescptr->name);

			/* Now we parse for the text pointer		*/
			while((isspace(*tmpstr)) && (*tmpstr != '\0')) tmpstr++;

			/* End of string?				*/
			if(*tmpstr == '\0') return(GW_NONE);

			tmp_textptrstr = textptrstr;
			while((!(isspace(*tmpstr))) && (*tmpstr != '\0')) 
			{
				*tmp_textptrstr++ = *tmpstr++;
			}
			*tmp_textptrstr = '\0';

			if (strncmp(textptrstr, "NULL", strlen("NULL")) == 0)
			{
				docdescptr->textptrlen = 0;
			}
			/* Store text pointer (as binary) in text descriptor.*/
			else if (srv_convert(spp, SRVCHAR, textptrstr,
						SRV_NULLTERM, SRVBINARY,
						docdescptr->textptr,
						sizeof(docdescptr->textptr))
				== -1)
			{
				return(GW_NONE);
			}
			else
			{
				docdescptr->textptrlen = CS_TP_SIZE;
			}

			/* Now we parse for the timestamp, if any	*/
			while((isspace(*tmpstr)) && (*tmpstr != '\0')) tmpstr++;

			/* End of string?				*/
			if(*tmpstr == '\0')
			{
				docdescptr->timestamplen = 0;
				docdescptr->log_on_update = CS_FALSE;
				return(GW_WRTXTBLK);
			}

			if (strncmp(tmpstr, "timestamp =",strlen("tempstamp ="))
				 == 0)
			{
				/* Get to the actual timestamp value.	*/
				while((*tmpstr != '\0') && (*tmpstr != '0'))
					tmpstr++;

				/* End of string?			*/
				if(*tmpstr == '\0') return(GW_NONE);

				tmp_timestampstr = timestampstr;
				while((!(isspace(*tmpstr)))
					&& (*tmpstr != '\0')) 
				{
					*tmp_timestampstr++ = *tmpstr++;
				}
				*tmp_timestampstr = '\0';

				if (strncmp(timestampstr, "NULL", 
					strlen("NULL")) == 0)
				{
					docdescptr->timestamplen = 0;
				}
				/*
				** Store time stamp (as binary) in text
				** descriptor.
				*/
				else if (srv_convert(spp, SRVCHAR, timestampstr,
						SRV_NULLTERM, SRVBINARY,
						docdescptr->timestamp,
						sizeof(docdescptr->timestamp))
					== -1)
				{
					return(GW_NONE);
				}
				else
				{
					docdescptr->timestamplen = CS_TS_SIZE;
				}

			}

			/* Now we parse for the logging indicator, if any.*/
			while((isspace(*tmpstr)) && (*tmpstr != '\0')) tmpstr++;

			/* End of string?				*/
			if(*tmpstr == '\0')
			{
				docdescptr->log_on_update = CS_FALSE;
				return(GW_WRTXTBLK);
			}

			if (strncmp(tmpstr, "with log", strlen("with log"))
				== 0)
			{
				docdescptr->log_on_update = CS_TRUE;
			}
			return(GW_WRTXTBLK);
		}
		/* For an "insert bulk", we need the table name.	*/
		tmpstr += strlen(BULK_STR);

		while((isspace(*tmpstr)) && (*tmpstr != '\0'))
		{
			tmpstr++;
		}

		/* End of string?					*/
		if(*tmpstr == '\0')
		{
			return(GW_NONE);
		}

		while((!(isspace(*tmpstr))) && (*tmpstr != '\0')) 
		{
			*tmp_tblname++ = *tmpstr++;
		}
		*tmp_tblname = '\0';

		if(srv_thread_props(spp, CS_SET, SRV_T_BULKTYPE,
			(CS_VOID *)&iotype, CS_SIZEOF(iotype), NIL(CS_INT *))
			== CS_FAIL)
		{
			return(GW_NONE);
		}


		/* Look for "with nodescribe".				*/
		while((isspace(*tmpstr)) && (*tmpstr != '\0'))
		{
			tmpstr++;
		}

		/*
		** Now that we've got our table name, we
		** can initialize the bulk copy here
		** (blk_init() will send the language
		** to the server)
		*/
		if(blk_alloc(chp, BLK_VERSION_100, &(rp->blkdesc)) == CS_FAIL)
		{
			return(GW_NONE);
		}

		/*
		** "with [nodescribe,] labels" means
		** sensitivity labels of data rows
		** are supplied after the formatted
		** portion of the row.
		*/
		if ((*tmpstr != '\0') && (strstr(tmpstr, "labels")
			!= (CS_CHAR *)NULL))
		{
			labelflag = CS_TRUE;
			if (blk_props(rp->blkdesc, CS_SET, BLK_SENSITIVITY_LBL,
				&labelflag, CS_UNUSED, NULL) == CS_FAIL)
			{
				return(GW_NONE);
			}
		}

		if(blk_init(rp->blkdesc, CS_BLK_IN, tblname, strlen(tblname))
			!= CS_FAIL )
		{
			/* Something after the "insert..."? */
			if(*tmpstr != '\0')
			{
				/*
				** "with nodescribe" means there's
				**  another bulk copy coming as part
				**  of a batch. No bcp-protocol
				**  response needed.
				*/
				if(strncmp(tmpstr, "with nodescribe", 
					strlen("with nodescribe")) == 0)
				{
					return(GW_INSRTBLK);
				}
			}

			/*
			**  Have to wait til the blk_init has completed.
			*/
			if((blk_srvinit(spp, rp->blkdesc)) != CS_FAIL)
			{
				return(GW_INSRTBLK);
			}
		}
	}
	return(GW_NONE);
}

/*
** GW_GETDYNP
**
**	This routine returns a pointer to the dynamic SQL control 
**	structure for the thread and dynamic statement id.
**
**  Parameters:
**	spp		Thread control structure.
**	dynid	 	The dynamic statement id.
**
**  Returns:
**	 dp		A valid pointer to a dynamic control 
**			structure.

**	NIL(DYNCTL_T *)	There is no active dynamic control 
**			structure. 
*/
DYNCTL_T	*gw_getdynp(spp, dynid, dynidlen)
SRV_PROC	*spp;
CS_CHAR		*dynid;
CS_INT		dynidlen;
{
	DYNCTL_T	*dp;	/* Dynamic SQL control structure.	*/

	DBMSCTL_T	*rp;	/* DBMS control structure.		*/
	CS_INT		i;

	/* Initialization.						*/
	dp = NIL(DYNCTL_T *);

	/* Get user data from SRV_PROC structure.			*/
	if((rp = (DBMSCTL_T *)srv_getuserdata(spp)) != NIL(DBMSCTL_T *))
	{
		for (i = 0; i < GW_MAXDYNS; i++)
		{
			if(strncmp(dynid, (rp->actdyn[i]).stmtid, dynidlen)
				== 0)
			{
				dp = &(rp->actdyn[i]);
				break;
			}
		}
	}
	return(dp);
}

/*
** GW_GETDYNCONTEXT
**
**	This routine gets the current dynamic SQL context.  The command handle
**	associated with this context is returned to the caller along
**	with the current dynamic SQL statement ID.
**
**  Parameters:
**	spp			Thread control structure.
**	dynid	 		The current dynamic SQL ID is returned here.
**	dynidlen		Length of dynamic ID returned in dynid.
**
**  Returns:
**	cmdp			A valid command handle if action was completed
**		  		successfully.
**	NIL(CS_COMMAND *)	The current context could not be determined.
*/
CS_COMMAND	*gw_getdyncontext(spp, dynid, dynidlen)
SRV_PROC	*spp;
CS_CHAR		*dynid;
CS_INT		*dynidlen;
{
	CS_COMMAND	*cmdp;		/* Current cursor's command handle.*/
	DYNCTL_T	*dp;		/* Control structure.		*/
	CS_CHAR		tmpdynid[CS_MAX_NAME];
	CS_INT		tmpdynidlen;
	CS_INT		outlen;

	/* Initialization.						*/
	cmdp = NIL(CS_COMMAND *);	/* No cursor cmdp yet.		*/
	dp = NIL(DYNCTL_T *);		/* No control structure yet.	*/

	/* Get current dynamic SQL context.				*/
	if(srv_dynamic(spp, CS_GET, SRV_DYN_IDLEN, (CS_VOID *)&tmpdynidlen,
		CS_SIZEOF(tmpdynidlen), &outlen) == CS_SUCCEED)
	{
		if(srv_dynamic(spp, CS_GET, SRV_DYN_ID,  (CS_VOID *)tmpdynid,
			tmpdynidlen, &outlen) == CS_SUCCEED)
		{
			if((dp = gw_getdynp(spp, tmpdynid, tmpdynidlen))
				!= NIL(DYNCTL_T *))
			{
				cmdp = dp->cmdp;
				strncpy(dynid, tmpdynid, tmpdynidlen);
				*dynidlen = tmpdynidlen;
			}
		}
	}
	return(cmdp);
}

/*
** GW_MANDYNID
**
**	This routine manages dynamic SQL identifiers.  
**	It maps command handles and dynamic SQL identifiers.
**
**  Parameters:
**	spp		Thread control structure.
**	action		Requested action.
**		   		GW_SAVEDYNID - return a new ID in dynid.
**		   		GW_RELDYNID - release ID in dynid.
**	cmdp		Command handle associated with this ID.  This
**		   	argument is only valid if `action' is GET_DYNID.
**	dynid		ID affected by this command.
**	dynidlen	ID length.
**
**  Returns:
**	CS_SUCCEED	Requested operation was completed successfully.
**	CS_FAIL		An error occurred during requested operation.
*/
CS_RETCODE	gw_mandynid(spp, action, cmdp, dynid, dynidlen)
SRV_PROC	*spp;
CS_INT		action;
CS_COMMAND	*cmdp;
CS_CHAR		*dynid;
CS_INT		dynidlen;
{
	CS_SMALLINT	i;		/* Loop control variable.	*/
	CS_BOOL		ok;		/* Error control flag.		*/
	CS_BOOL		found;		/* CS_TRUE if cursor ID found.	*/
	DYNCTL_T	*dp;		/* Dynamic SQL control structure.*/
	DBMSCTL_T	*rp;		/* DBMS control structure.	*/

	/* Initialization.						*/
	ok = CS_FALSE;			/* Assume command will fail.	*/
	found = CS_FALSE;		/* Cursor ID not found yet.	*/

	/* Process according to action.					*/
	switch((int)action)
	{
	case GW_RELDYNID:
		if((dp = gw_getdynp(spp, dynid, dynidlen)) != NIL(DYNCTL_T *))
		{
			(CS_VOID)ct_cmd_drop(dp->cmdp);
			dp->cmdp = NIL(CS_COMMAND *);
			(dp->stmtid)[0] = '\0';
			dp->stmtidlen = 0;
			ok = CS_TRUE;
		}
		break;

	case GW_SAVEDYNID:
		/* Get user data from SRV_PROC structure.		*/
		if((rp = (DBMSCTL_T *)srv_getuserdata(spp))
			!= NIL(DBMSCTL_T *))
		{
			/*
	 		** Scan dynamic map array looking
	 		** for an available dynamic SQL ID slot.
	 		*/
			for(i = 0; i < GW_MAXDYNS; i++)
			{
				if((rp->actdyn[i]).stmtidlen == 0)
				{
					dp = &(rp->actdyn[i]);
					found = CS_TRUE;
					break;
				}
			}
		}
		/* If we found a slot return index to caller and save cmdp */
		if(found)
		{
			strncpy(dp->stmtid, dynid, dynidlen);
			dp->stmtidlen = dynidlen;
			dp->stmtidnum = i;
			dp->cmdp = cmdp;
			ok = CS_TRUE;
		}
		break;

	default:
		ok = CS_FALSE;
		break;
	}
	return(ok ? CS_SUCCEED : CS_FAIL);
}

/*
**  GW_SENDERROR
**
**	This routine sends a done error to the client.
**
**  Parameters:
**	spp	Client thread on which to send error.
**
**  Returns:
**	Nothing.
*/
CS_VOID		gw_senderror(spp)
SRV_PROC	*spp;
{
	CS_BOOL		iodead;		/* I/O dead flag.		*/

	/*
	** Make sure that I/O is still active on this connection.  If
	** this routine fails, log an error message and return to the
	** caller.  The Open Server scheduler will kill this thread
	** since we haven't sent a final DONE.
	*/
	if(srv_thread_props(spp, CS_GET, SRV_T_IODEAD, (CS_VOID *)&iodead,
		CS_SIZEOF(iodead), NIL(CS_INT *)) == CS_FAIL)
	{
		return;
	}

	/*
	** If I/O isn't active just return to the caller.  Open Server will
	** kill this thread when it returns to the scheduler.  We don't
	** need to send a DONE here.
	*/
	if(iodead)
	{
		return;
	}

	/*
	** Send a DONE error to the caller.  We ignore the return code
	** from this call since Open Server will kill this thread if
	** a final done isn't sent.
	*/
	(CS_VOID)srv_senddone(spp, (SRV_DONE_FINAL|SRV_DONE_ERROR),
		CS_TRAN_UNDEFINED, (CS_INT)0);
	return;
}

/*
**  GW_INITLOGIN
**
**	This routine initialize the login record for the remote SQL
**	server from the information provided by the remote client.
**	If an error is detected by this routine a error message is logged.
**
**  Parameters:
**	spp	Client thread.
**	chp	Allocated connection handle for remote SQL server connection.
**
**  Returns:
**	CS_SUCCEED	Login record was initialized without any errors.
**	CS_FAIL		An error occured while initializing login record.
*/
CS_RETCODE	gw_initlogin(spp, chp)
SRV_PROC	*spp;
CS_CONNECTION	*chp;
{
	CS_CHAR		buf[256];	/* Buffer for user information.	*/
	CS_INT		outlen;		/* Length of information in buffer. */
	CS_INT		negtype;	/* Negotiated login type.	*/
	CS_INT		logintype = 0;	/* CS-Lib mapped login type.	*/
	CS_BOOL		bool;		/* Working boolean variable.	*/
	CS_BOOL		bcp_on;		/* Enable bulk copy?		*/
	CS_LOCALE	*lp;		/* Thread localization information. */
	CS_CAP_TYPE	capmask;	/* capability mask		*/

	/*
	** Get the user name.
	*/
	if(srv_thread_props(spp, CS_GET, SRV_T_USER, (CS_VOID *)buf,
		CS_SIZEOF(buf), &outlen) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Set the user name in the connection handle.
	*/
	if(ct_con_props(chp, CS_SET, CS_USERNAME, (CS_VOID *)buf, 
		outlen, NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Get the user's password.
	*/
	if(srv_thread_props(spp, CS_GET, SRV_T_PWD, (CS_VOID *)buf,
		CS_SIZEOF(buf), &outlen) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Set the user's password in the connection handle.
	*/
	if(ct_con_props(chp, CS_SET, CS_PASSWORD, (CS_VOID *)buf, 
		outlen, NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Get the user's application name.
	*/
	if(srv_thread_props(spp, CS_GET, SRV_T_APPLNAME, (CS_VOID *)buf,
		CS_SIZEOF(buf), &outlen) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Set the user's application name in the connection handle.
	*/
	if(ct_con_props(chp, CS_SET, CS_APPNAME, (CS_VOID *)buf, 
		outlen, NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Get the user's hostname
	*/
	if(srv_thread_props(spp, CS_GET, SRV_T_HOSTNAME, 
		(CS_VOID *)buf, CS_SIZEOF(buf), &outlen) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Set the user's host name in the connection handle.
	*/
	if(ct_con_props(chp, CS_SET, CS_HOSTNAME, (CS_VOID *)buf, 
		outlen, NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Get the requested negotiated login type.
	*/
	if(srv_thread_props(spp, CS_GET, SRV_T_NEGLOGIN, (CS_VOID *)&negtype,
		CS_SIZEOF(negtype), NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Set the negotiated login type in the connection handle, if
	** one was requested.
	*/
	bool = CS_TRUE;
	if (negtype & SRV_CHALLENGE)
	{
		if(ct_con_props(chp, CS_SET, CS_SEC_CHALLENGE, (CS_VOID *)&bool,
			CS_UNUSED, NIL(CS_INT *)) == CS_FAIL)
		{
			gw_senderror(spp);
			return(CS_FAIL);
		}
	}

	if (negtype & SRV_ENCRYPT)
	{
		if(ct_con_props(chp, CS_SET, CS_SEC_ENCRYPTION, (CS_VOID *)&bool,
			CS_UNUSED, NIL(CS_INT *)) == CS_FAIL)
		{
			gw_senderror(spp);
			return(CS_FAIL);
		}
	}

	if (negtype & SRV_SECLABEL)
	{
		if(ct_con_props(chp, CS_SET, CS_SEC_NEGOTIATE, (CS_VOID *)&bool,
			CS_UNUSED, NIL(CS_INT *)) == CS_FAIL)
		{
			gw_senderror(spp);
			return(CS_FAIL);
		}
	}

	/*
	** Get the bulk copy setting for this client connection.
	*/
	if(srv_thread_props(spp, CS_GET, SRV_T_DUMPLOAD, (CS_VOID *)&bool,
		CS_SIZEOF(bool), NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	bcp_on = !bool;
	/*
	** Set the bulk copy setting for this connection.
	*/
	if(ct_con_props(chp, CS_SET, CS_BULK_LOGIN, (CS_VOID *)&bcp_on,
		CS_UNUSED, NIL(CS_INT *)) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Initialize the connection with the language of the client
	** thread.  The character-set should not be modified, since all
	** data is translated into the Open Server's character set.
	*/
	if (cs_loc_alloc(Context, &lp) != CS_SUCCEED)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Get the client thread's locale 				*/
        if(srv_thread_props(spp, CS_GET, SRV_T_LOCALE, (CS_VOID *)lp,
		CS_SIZEOF(lp), NIL(CS_INT *)) == CS_FAIL)
	{
		(CS_VOID)cs_loc_drop(Context, lp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Get the language name from the locale structure.		*/
	if (cs_locale(Context, CS_GET, lp, CS_SYB_LANG, buf, CS_SIZEOF(buf), 
		&outlen) != CS_SUCCEED)
	{
		(CS_VOID)cs_loc_drop(Context, lp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* Free the client locale copy					*/
	if (cs_loc_drop(Context, lp) != CS_SUCCEED)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/* 
	** Allocate a locale for the connection with the default locale 
	** information for this application
	*/
	if (cs_loc_alloc(Context, &lp) != CS_SUCCEED)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** Modify the language of application's locale to use the 
	** client's language.
	*/
	if (cs_locale(Context, CS_SET, lp, CS_SYB_LANG, buf, outlen, 
		(CS_INT *)NULL) != CS_SUCCEED)
	{
		(CS_VOID)cs_loc_drop(Context, lp);
		gw_senderror(spp);
		return(CS_FAIL);
	}


	/*
	** Set the localization information for this connection.
	*/
	if(ct_con_props(chp, CS_SET, CS_LOC_PROP, (CS_VOID *)lp, 
		CS_UNUSED, NIL(CS_INT *)) == CS_FAIL)
	{
		(CS_VOID)cs_loc_drop(Context, lp);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** We cannot define CS_CAP_REQUEST capabilities on the
	** CTLIB connection, because CTLIB does not let the applications
	** change the library's request capabilities.
	*/

	/* Get and set RESPONSE capabilities                            */
	srv_bzero(&capmask, sizeof(CS_CAP_TYPE));
	if (srv_capability_info(spp, CS_GET, CS_CAP_RESPONSE, CS_ALL_CAPS,
			&capmask) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}
	if (ct_capability(chp, CS_SET, CS_CAP_RESPONSE, CS_ALL_CAPS,
			&capmask) == CS_FAIL)
	{
		gw_senderror(spp);
		return(CS_FAIL);
	}


	/*
	** Release the allocated localization structure.  It is no longer
	** needed.
	*/
	(CS_VOID)cs_loc_drop(Context, lp);
	return(CS_SUCCEED);
}

/*
**  GW_CTSEND
**
**	This routine sends a command to the remote SQL server.  It checks
**	to see if an attention occurred before issuing the send.  This
**	eliminates the processing of a command when the client sent
**	a cancel that was detected by Open Server before we defined
**	the command to CT-Library.
**
**	This routine sends the required DONE to the client if the command
**	was canceled or ct_send fails. 
**
**  Parameters:
**	spp	Client thread.
**	chp	Allocated connection handle for remote SQL server connection.
**
**  Returns:
**	CS_SUCCEED	Command was sent to SQL server successfully.
**	CS_FAIL		ct_send failed, or a cancel was detected.  A done
**			was sent to the client.
*/
CS_RETCODE	gw_ctsend(spp, cmdp)
SRV_PROC	*spp;
CS_COMMAND	*cmdp;
{
	CS_BOOL		gotattention;	/* Attention flag.		*/
	CS_RETCODE	retcode;	/* ct_send return code.		*/

	/*
	** Check to see if we have received an attention on this thread.
	*/
	if(srv_thread_props(spp, CS_GET, SRV_T_GOTATTENTION,
		(CS_VOID *)&gotattention, CS_SIZEOF(gotattention),
		NIL(CS_INT *)) == CS_FAIL)
	{
		(CS_VOID)ct_cancel(NULL, cmdp, CS_CANCEL_ALL);
		gw_senderror(spp);
		return(CS_FAIL);
	}

	/*
	** If we have received an attention on this thread, just send
	** a final done to the client.  We don't want to send the command
	** to the remote SQL server since the client canceled it!  We
	** return CS_FAIL to the caller, so that they return from the
	** event handler.
	*/
	if(gotattention)
	{
		(CS_VOID)ct_cancel(NULL, cmdp, CS_CANCEL_ALL);
		(CS_VOID)srv_senddone(spp, SRV_DONE_FINAL, CS_TRAN_UNDEFINED,
			(CS_INT)0);
		return(CS_FAIL);
	}

	/*
	** Send the command to the remote SQL server.
	*/
	retcode = ct_send(cmdp);

	/*
	** Process according to the return code from ct_send.  If ct_send
	** returns CS_CANCELED it is because ct_cancel was called from
	** our attention handler.  In this case nothing was sent to
	** the SQL server.
	*/
	switch((int)retcode)
	{
	case CS_CANCELED:
		/*
		** Change value to return to caller to CS_FAIL.
		*/
		retcode = CS_FAIL;
		break;

	case CS_FAIL:
		/*
		** ct_send actually failed.
		*/
		break;
	default:
		break;
	}

	/*
	** Send a done to the client if the command was canceled or ct_send
	** failed.
	*/
	if(retcode == CS_FAIL)
	{
		gw_senderror(spp);
	}
	return(retcode);
}
